import Qieyun from 'qieyun';
import { 推導方案 } from 'tshet-uinh-deriver-tools';

const _schemata = {};
const _defaultDerivers = {};

/**
 * 查詢字頭的擬音。
 * @param {string | string[]} schema 推導方案或推導方案陣列
 * @param {string} 字頭 要查詢的字頭
 * @param {Object=} 選項 選項（可選）
 * @return {(Qieyun.資料.字頭檢索結果 & {擬音: string | string[]})[]} 音韻地位、解釋、音韻地位對應的擬音或擬音陣列
 */
export function from字頭(schema, 字頭, 選項) {
  return Qieyun.資料.query字頭(字頭).map(result => ({
    ...result,
    擬音: schema.map
      ? schema.map(schema => _schemata[schema](選項)(result.音韻地位, result.字頭))
      : _schemata[schema](選項)(result.音韻地位, result.字頭),
  }));
}

function _perform(name, derive, 地位, 字頭, 選項) {
  if (選項 != null) {
    throw new Error(`Specifying 選項 directly is no longer supported, use ${name}.schema(選項)(地位, 字頭?) instead`);
  }
  return derive(地位, 字頭);
}

_schemata.tupa = new 推導方案(function (選項, 音韻地位, 字頭) {
if (!音韻地位) return [];

const is = (...x) => 音韻地位.屬於(...x);
const when = (...x) => 音韻地位.判斷(...x);

// 正則化之前需保留該信息
const is脣音咍韻 = is`脣音 咍韻`;

const 正則化 = {
  標準: 'v2',
  寬鬆: 'v2lenient',
}[選項.模式] || 'v2Strict';
音韻地位 = Qieyun.適配分析體系[正則化](音韻地位);

// 恢復脣音咍韻信息
if (is脣音咍韻 && !選項.脣音咍韻歸灰韻) {
  音韻地位 = 音韻地位.調整({ 韻: '咍' });
}

function get聲母() {
  return {
    幫: 'p',  滂: 'ph',  並: 'b',  明: 'm',
    端: 't',  透: 'th',  定: 'd',  泥: 'n',  來: 'l',
    知: 'tr', 徹: 'trh', 澄: 'dr', 孃: 'nr',
    見: 'k',  溪: 'kh',  羣: 'g',  疑: 'ng', 云: '',
    影: 'q',  曉: 'h',   匣: 'gh',
    精: 'ts',  清: 'tsh',  從: 'dz',  心: 's',  邪: 'z',
    莊: 'tsr', 初: 'tsrh', 崇: 'dzr', 生: 'sr', 俟: 'zr',
    章: 'tj',  昌: 'tjh',  常: 'dj',  書: 'sj', 船: 'zj', 日: 'nj', 以: 'j',
  }[音韻地位.母];
}

function get韻母() {
  // 爲了方便推導，對韻類稍作調整
  const 韻 = when([
    ['蒸韻 (重紐B類 或 幫組 或 合口)', '冰'],
    ['東韻 三等', '終'],
    ['清韻', '庚'],
    ['陽韻', '唐'],
    ['臻韻 或 莊組 欣韻 上聲', '眞'],
    ['凡韻', '嚴'],
    ['', 音韻地位.韻],
  ]);
  const 韻到韻尾 = [
    ['脂之尤侯 　佳　模　 支魚虞 麻歌', ''],
    ['冰蒸終東 青耕登冬江 　　鍾 庚唐', 'ng', 'k'],
    ['　微微　 齊皆咍灰　 祭廢廢 夬泰', 'j'],
    ['眞欣文　 先山痕魂　 仙元元 刪寒', 'n', 't'],
    ['幽　　　 蕭　　　　 宵　　 肴豪', 'w'],
    ['侵　　　 添咸　覃　 鹽嚴嚴 銜談', 'm', 'p'],
  ];
  const 元音列表 = [
    'i',       'y',  'u', 'ou',
    'e', 'ee', 'eo', 'o', 'oeu',
    'e',       'yo', 'uo',
         'ae', 'a',
  ];

  let 匹配行 = 韻到韻尾.find(行 => 行[0].includes(韻));
  let 元音 = 元音列表[匹配行[0].replace(/ /g, '')[is`開口` ? 'indexOf' : 'lastIndexOf'](韻)];
  let 韻尾 = 匹配行[1 + is`入聲`];

  // 添加三等 C 介音（僅歌陽韻需要處理）
  if (is`三等` && 元音 === 'a') {
    // 重紐A類用於𩦠小韻
    元音 = (is`脣音 重紐A類` ? 'i' : is`開口` ? 'y' : 'u') + 元音;
  }
  // 添加三等 A、B 介音
  if (is`三等` && ['i', 'e', 'ae'].includes(元音)) {
    if (is`重紐B類 或 云母 或 莊組 或 庚蒸韻 或 幽韻 幫組`) {
      元音 = (is`合口` ? 'u' : 'y') + 元音;
    } else if (元音 !== 'i') {
      // 拼莊組以外的銳音一律視爲 A 類（同《切韻》清韻、《廣韻》諄韻的獨立條件）
      元音 = 'i' + 元音;
    }
  }
  // 添加合口介音
  if (is`合口` && !['u', 'o'].includes(元音[0])) {
    元音 = 'w' + 元音;
  }
  return 元音 + 韻尾;
}

function get聲調() {
  return { 上: 'q', 去: 'h' }[音韻地位.聲] || '';
}

return get聲母() + get韻母() + get聲調();
});

_defaultDerivers.tupa = _schemata.tupa();

/**
 * 切韻拼音
 *
 * https://zhuanlan.zhihu.com/p/478751152
 *
 * 推導器所用的音韻地位無需選項，但在處理其他來源的音韻地位時可指定以下選項：
 *
 * 選項.模式 = '嚴格'; // 僅允許正則地位（如不指定 `模式`，也按嚴格模式處理）
 * 選項.模式 = '標準'; // 允許部分非正則地位（推導爲對應的正則地位），不允許無法自動正則化的聲韻搭配
 * 選項.模式 = '寬鬆'; // 允許非正則地位，無法自動正則化的聲韻搭配保持原樣
 * 選項.脣音咍韻歸灰韻 = false; // 脣音咍韻推導爲開口，僅在「標準」「寬鬆」模式下有效
 *
 * @author unt
 * @param {Qieyun.音韻地位} 音韻地位 切韻音系音韻地位
 * @param {string=} 字頭 字頭（可選）
 * @param {undefined=} 舊選項參數 已棄用，請用 `<方案名>.方案(選項)(音韻地位, 字頭?)` 代替
 * @return {string} 音韻地位對應的切韻拼音
 * @property {推導方案<string>} schema 推導方案，可用於指定推導選項、獲取方案詳細資訊等
 */
export function tupa(音韻地位, 字頭, 舊選項參數) {
  return _perform("tupa", _defaultDerivers.tupa, 音韻地位, 字頭, 舊選項參數);
}
/** @type {推導方案<string>} */
tupa.schema = _schemata.tupa

_schemata.baxter = new 推導方案(function (選項, 音韻地位, 字頭) {
if (!音韻地位) return [
  // 版本可選 '1992' 或 '2014'，預設值為 '2014'
  ['版本', [2, '1992', '2014']],
];

const is = (...x) => 音韻地位.屬於(...x);

let 聲母 = {
  幫: 'p',   滂: 'ph',   並: 'b',   明: 'm',
  端: 't',   透: 'th',   定: 'd',   泥: 'n',  來: 'l',
  知: 'tr',  徹: 'trh',  澄: 'dr',  孃: 'nr',
  精: 'ts',  清: 'tsh',  從: 'dz',                     心: 's',  邪: 'z',
  莊: 'tsr', 初: 'tsrh', 崇: 'dzr',                    生: 'sr', 俟: 'zr',
  章: 'tsy', 昌: 'tsyh', 常: 'dzy', 日: 'ny',          書: 'sy', 船: 'zy', 以: 'y',
  見: 'k',   溪: 'kh',   羣: 'g',   疑: 'ng',
  影: "'",   曉: 'x',    匣: 'h',                                          云: 'h',
}[音韻地位.母];

if (選項.版本 === '1992' && 聲母 === "'") {
  聲母 = 'ʔ';
}

let 韻母 = {
  // 一等韻
  東: 'uwng',
  冬: 'owng',
  模: 'u',
  泰: 'aj',
  灰: 'oj',
  咍: 'oj',
  魂: 'on',
  痕: 'on',
  寒: 'an',
  豪: 'aw',
  歌: 'a',
  唐: 'ang',
  登: 'ong',
  侯: 'uw',
  覃: 'om',
  談: 'am',

  // 二等韻
  江: 'aewng',
  佳: 'ea',
  皆: 'eaj',
  夬: 'aej',
  刪: 'aen',
  山: 'ean',
  肴: 'aew',
  麻: 'ae',
  庚: 'aeng',
  耕: 'eang',
  咸: 'eam',
  銜: 'aem',

  // 四等韻
  齊: 'ej',
  先: 'en',
  蕭: 'ew',
  青: 'eng',
  添: 'em',

  // 三等陰聲韻
  支: 'je',
  脂: 'ij',
  之: 'i',
  微: 'j+j',
  魚: 'jo',
  虞: 'ju',
  祭: 'jej',
  廢: 'joj',
  宵: 'jew',
  // 歌: 'ja',
  // 麻: 'jae',
  尤: 'juw',
  幽: 'jiw',

  // 三等陽聲韻
  // 東: 'juwng',
  鍾: 'jowng',
  眞: 'in',
  臻: 'in',
  文: 'jun',
  欣: 'j+n',
  元: 'jon',
  仙: 'jen',
  陽: 'jang',
  // 庚: 'jaeng',
  清: 'jeng',
  蒸: 'ing',
  侵: 'im',
  鹽: 'jem',
  嚴: 'jaem',
  凡: 'jom',
}[音韻地位.韻];

// 東歌麻庚韻同時含三等與非三等，上文僅處理非三等，此處處理三等
if (is`東歌麻庚韻 三等`) {
  韻母 = 'j' + 韻母;
}

if (選項.版本 === '1992') {
  if (韻母 === 'ea') 韻母 = 'ɛɨ';
  韻母 = 韻母.replace('+', 'ɨ').replace('ae', 'æ').replace('ea', 'ɛ');
}

// 章組或日以母只與三等韻相拼，省去韻母起始的 j
if (is`章組 或 日以母` && 韻母.startsWith('j')) {
  韻母 = 韻母.slice(1);
}

// 重紐 A 類添加 j 或 i
if (is`(重紐A類 非 陽韻) 或 (清韻 ${Qieyun.表達式.重紐母} 非 重紐B類)`) {
  if (韻母.startsWith('j')) 韻母 = 'ji' + 韻母.slice(1);
  else 韻母 = 'j' + 韻母;
}

// 合口字添加 w
if (is`(合口 或 灰魂韻) 非 文凡韻`) {
  if (韻母.startsWith('j')) 韻母 = 'jw' + 韻母.slice(1);
  else 韻母 = 'w' + 韻母;
}

if (is`入聲`) {
  if (韻母.endsWith('m')) 韻母 = 韻母.slice(0, -1) + 'p';
  else if (韻母.endsWith('n')) 韻母 = 韻母.slice(0, -1) + 't';
  else if (韻母.endsWith('ng')) 韻母 = 韻母.slice(0, -2) + 'k';
}

const 聲調 = {
  上: 'X',
  去: 'H',
}[音韻地位.聲] || '';

return 聲母 + 韻母 + 聲調;
});

_defaultDerivers.baxter = _schemata.baxter();

/**
 * 白一平轉寫
 *
 * - Baxter, W. H. (1992). A Handbook of Old Chinese Phonology. De Gruyter Mouton.
 * - Baxter, W. H., & Sagart, L. (2014). Old Chinese: A New Reconstruction. Oxford University Press.
 *
 * @author Ayaka
 * @param {Qieyun.音韻地位} 音韻地位 切韻音系音韻地位
 * @param {string=} 字頭 字頭（可選）
 * @param {undefined=} 舊選項參數 已棄用，請用 `<方案名>.方案(選項)(音韻地位, 字頭?)` 代替
 * @return {string} 音韻地位對應的白一平轉寫
 * @property {推導方案<string>} schema 推導方案，可用於指定推導選項、獲取方案詳細資訊等
 */
export function baxter(音韻地位, 字頭, 舊選項參數) {
  return _perform("baxter", _defaultDerivers.baxter, 音韻地位, 字頭, 舊選項參數);
}
/** @type {推導方案<string>} */
baxter.schema = _schemata.baxter

_schemata.blankego = new 推導方案(function (選項, 音韻地位, 字頭) {
// 可調整參數
if (!音韻地位) {
  return [
    // 祭廢韻平上（章組云以日母）
    // - 停用（預設）：不認可祭廢非去聲，歸為齊灰咍韻
    // - 啟用：認可該類地位（實際上祭、齊韻推導結果相同，僅廢與灰咍會不同）
    ['蟹攝三等平上', false],

    // 莊組眞韻開口上、去聲
    // - 停用（預設）：按原表，均處理為臻韻
    // - 啟用：認可莊組眞韻
    ['莊組眞韻開口', false],

    // 清B處理法（推導器預設資料無此類，僅適用於其他資料音韻地位）
    // - `'清B'`（預設）：按清B拼寫
    // - `'合口'`：按原表處理，以清韻合口拼寫
    // - `'庚三'`：不認可清韻B類，視同庚三
    ['清B脣音', [1, '清B', '合口', '庚三']],
  ];
}

const is = (...x) => 音韻地位.屬於(...x);

// 調整音韻地位

音韻地位.呼 && is`脣音` && (音韻地位 = 音韻地位.調整({ 呼: null }));
音韻地位.重紐 && is`陽蒸幽韻` && (音韻地位 = 音韻地位.調整({ 重紐: null }));
is`莊組 欣韻` && (音韻地位 = 音韻地位.調整({ 韻: '臻' }));

// 選項相關
if (!選項.蟹攝三等平上 && is`祭廢韻 平上聲`) {
  音韻地位 = 音韻地位.調整(
    is`祭韻`
      ? { 韻: '齊', 等: '四' }
      : { 韻: is`開口` ? '咍' : '灰', 等: '一' },
  );
} else if (!選項.莊組眞韻開口 && is`莊組 眞韻 開口`) {
  音韻地位 = 音韻地位.調整({ 韻: '臻' });
} else if (選項.清B脣音 === '庚三' && is`幫組 清韻 重紐B類`) {
  音韻地位 = 音韻地位.調整({ 韻: '庚', 重紐: null });
}

/**
 * 依次匹配列表 `rules` 中的規則，返回第一項測試為 true 的對應結果。
 * 無匹配規則時抛出異常，`name` 會包含於異常信息中。
 *
 * `rules` 每項為 `[條件, 結果]`
 * 條件可為：
 * - 字串，會用 `音韻地位.屬於` 測試
 * - 函數，會直接調用並測試返回結果
 * - 其他，直接測試真值
 */
function 規則(name, rules) {
  for (const [rule, res] of rules) {
    if (typeof rule === 'string') {
      if (is(rule.replace(/　/g, ''))) {
        return res;
      }
    } else if (typeof rule === 'function') {
      if (rule()) {
        return res;
      }
    } else if (rule) {
      return res;
    }
  }
  throw new Error(`無${name}規則：${音韻地位.描述}`);
}

// 聲母

// prettier-ignore
let 母 = {
  幫: 'p',  滂: 'ph', 並: 'b',  明: 'm',
  端: 't',  透: 'th', 定: 'd',  泥: 'n',
  知: 't',  徹: 'th', 澄: 'd',  孃: 'n',
  精: 'z',  清: 'c',  從: 'dz',           心: 's',  邪: 'sz',
  莊: 'tr', 初: 'ch', 崇: 'dr',           生: 'sh', 俟: 'zh',
  章: 'tj', 昌: 'tc', 船: 'dj', 日: 'r',  書: 'sj', 常: 'zj',
  見: 'k',  溪: 'q',  羣: 'g',  疑: 'ng',
  影: '',   曉: 'x',  匣: 'h',  云: 'h',
                                以: 'j',
                                來: 'l',
}[音韻地位.母];

if (母 === undefined) {
  throw new Error(`無聲母規則: ${音韻地位.描述}`);
}

// 韻尾

let 尾 = 音韻地位.判斷(
  [
    ['遇果假攝 或 支脂之佳韻', ''],
    ['通江宕梗曾攝', 'ng'],
    ['蟹攝 或 微韻', 'i'], // 已排除佳韻
    ['臻山攝', 'n'],
    ['效流攝', 'u'],
    ['深咸攝', 'm'],
  ],
  `無韻尾規則: ${音韻地位.描述}`,
);

// 主元音

let 元 = 音韻地位.判斷(
  [
    ['歌麻　唐庚陽　泰夬廢　寒刪元　豪肴　談銜嚴凡韻', 'a'],
    ['佳支　青耕清　齊皆祭　先山仙　蕭宵　添咸鹽　韻', 'e'],
    ['　脂　　　　　　　　　眞　　　　幽　　　侵　韻', 'i'],
    ['　　　　　　　　　　　臻　　　　　　　　　　韻', 'yi'],
    ['　　　登江蒸　咍灰微　痕魂欣　侯尤　覃　　　韻', 'o'],
    ['　之　　　　　　　　　　　　　　　　　　　　韻', 'io'], // 之韻 io 的 i 視為主元音
    ['模虞　東　　　　　　　　　文　　　　　　　　韻', 'u'],
    ['　魚　冬　鍾　　　　　　　　　　　　　　　　韻', 'v'],
  ].map(([cond, ...rest]) => [cond.replace(/　/g, ''), ...rest]),
  `無主元音規則: ${音韻地位.描述}`,
);

// 介音

// 拼寫上使用合口介音的場合
const 合口 = !is`脣音`
  ? is`合口 非 文韻`
  : is`歌陽廢寒元凡灰微魂韻` || is`清韻 重紐B類 ${選項.清B脣音 === '合口'}`;

let 介 = {
  一: ['', 'u'],
  二: ['e', 'o'],
  三: ['i', 'v'],
  四: ['', 'u'],
}[音韻地位.等][+合口];

if (介 === 'i' && (元.startsWith('i') || 元.startsWith('y'))) {
  介 = '';
} else if (介 === 'e' && 元 === 'e') {
  // 二等開口 e 元音寫作 ae
  介 = '';
  元 = 'ae';
} else if (介 === 'o' && 元 === 'e') {
  // oe 上聲要雙寫 o，亦視作整體
  介 = '';
  元 = 'oe';
}

// 三等重紐、重韻
if (
  (is('重紐B類') && (選項.清B脣音 !== '合口' || !is('幫組 清韻'))) ||
  (['i', 'e'].includes(元) &&
    is`(三等 知莊組 或 云母) 非 (知組 (清韻 或 眞韻 合口))`) || // 清、諄韻知組視作A類
  is('麻庚韻 三等 或 幽韻') // 重韻
) {
  介 = {
    i: 'y',
    v: 'w',
    '': '', // 主元音為 i 時
  }[介];
  if (介 === undefined) {
    throw new Error(`無重紐重韻規則: ${音韻地位.描述}`);
  }
  if (元 === 'i') {
    元 = 'y';
  }
}

// 拼寫規則

if (is('章組 或 日以母')) {
  // 章組、日以母省略 i 介音，麻三亦省略 y 介音
  if (介.startsWith('i')) {
    介 = 介.slice(1);
  } else if (is('麻韻 三等') && !合口) {
    介 = '';
  }
} else if (is('莊組')) {
  // 莊組拼 ea 省 e，拼 io, iu, iv 省 i（不含之韻），拼 ye 省 y（支韻除外）
  if (
    (介 === 'e' && 元 === 'a') ||
    (介 === 'i' && ['o', 'u', 'v'].includes(元)) ||
    (介 === 'y' && 元 === 'e' && 尾 !== '')
  ) {
    介 = '';
  }
}

// 聲調
if (音韻地位.聲 === '入') {
  尾 = {
    m: 'p',
    n: 't',
    ng: 'k',
  }[尾];
} else if (音韻地位.聲 !== '平') {
  if (['i', 'u', 'ng'].includes(尾)) {
    尾 = {
      i: ['j', 'y'],
      u: ['v', 'w'],
      ng: ['nk', 'nq'],
    }[尾][Number(音韻地位.聲 === '去')];
  } else if (音韻地位.聲 === '上') {
    元 = ['io', 'ae', 'oe'].includes(元) ? 元[0] + 元 : 元 + 元.slice(-1);
  } else if (音韻地位.聲 === '去' && ['m', 'n'].includes(尾)) {
    尾 = 尾 + 尾;
  } else if (音韻地位.聲 === '去' && 尾 === '') {
    元 = 元 + 'h';
  } else {
    throw new Error(`無聲調規則: ${音韻地位.描述}`);
  }
}
if (尾 === undefined || 元 === undefined) {
  throw new Error(`無聲調規則: ${音韻地位.描述}`);
}

return `${母}${介}${元}${尾}`;
});

_defaultDerivers.blankego = _schemata.blankego();

/**
 * 有女羅馬字
 *
 * https://github.com/BYVoid/ytenx/blob/master/ytenx/sync/kyonh/PrengQim.txt
 *
 * 本版說明：
 *
 * - 基於韻典網數據，並參照有女同車原版《〈廣韻〉全字表》（下稱「原表」）有所校正
 * - 目前推導器預設資料與原表有數十小韻在聲母、開合、等、重紐類等歸類有所不同，但除個別外均不影響原拼寫系統（需擴展原表拼寫系統的，列為選項）
 * - 旨韻（脂上聲）若干精、章組小韻，韻母寫為B類，與他處精章組均用A類不合，現修正
 * - 崇開三眞上（濜）：真諄上去聲莊組開口，原表及拼寫均析為臻韻上去聲，惟此小韻遺漏，現修正
 * - 莊合三真入（𠭴）：韻母寫為A類，與他處莊組均用B類不合，現修正（雖然不排除是因該小韻列於術韻而非質韻，但術韻下亦有「率」小韻用B類拼作 shwyt，故當以B類為正）
 * - 以開一寒入（poem表「䔾」）：原表無寒韻拼章組日以母例，但有談韻書母例與之平行，故可拼作 jat
 * - 幫清B入（碧）：原表處理為清韻合口作 pvek，與「辟」小韻開合對立。亦可處理為庚三即 pyak。原表清韻無B類，但可據體系推出處理為「清B」時的拼法 pyek。本代碼同時支持三種處理法
 *
 * @author SyiMyuZya
 * @param {Qieyun.音韻地位} 音韻地位 切韻音系音韻地位
 * @param {string=} 字頭 字頭（可選）
 * @param {undefined=} 舊選項參數 已棄用，請用 `<方案名>.方案(選項)(音韻地位, 字頭?)` 代替
 * @return {string} 音韻地位對應的有女羅馬字
 * @property {推導方案<string>} schema 推導方案，可用於指定推導選項、獲取方案詳細資訊等
 */
export function blankego(音韻地位, 字頭, 舊選項參數) {
  return _perform("blankego", _defaultDerivers.blankego, 音韻地位, 字頭, 舊選項參數);
}
/** @type {推導方案<string>} */
blankego.schema = _schemata.blankego

_schemata.kyonh = new 推導方案(function (選項, 音韻地位, 字頭) {
if (!音韻地位) return [];

const is = (...x) => 音韻地位.屬於(...x);

音韻地位 = Qieyun.適配分析體系('ytenx')(音韻地位);

function 聲母規則() {
  switch (音韻地位.母) {
    case '幫': return 'p';
    case '滂': return 'ph';
    case '並': return 'b';
    case '明': return 'm';
    case '端': return 't';
    case '透': return 'th';
    case '定': return 'd';
    case '泥': return 'n';
    case '知': return 'tr';
    case '徹': return 'thr';
    case '澄': return 'dr';
    case '孃': return 'nr';
    case '精': return 'c';
    case '清': return 'ch';
    case '從': return 'z';
    case '心': return 's';
    case '邪': return 'zs';
    case '莊': return 'cr';
    case '初': return 'chr';
    case '崇': return 'zr';
    case '生': return 'sr';
    case '俟': return 'zsr';
    case '章': return 'cj';
    case '昌': return 'chj';
    case '船': return 'zsj';
    case '書': return 'sj';
    case '常': return 'zj';
    case '見': return 'k';
    case '溪': return 'kh';
    case '羣': return 'g';
    case '疑': return 'ng';
    case '影': return 'q';
    case '曉': return 'h';
    case '匣': return 'gh';
    case '云': return '';
    case '以': return 'j';
    case '來': return 'l';
    case '日': return 'nj';
    default: throw new Error('無聲母規則');
  }
}

function 韻母規則() {
  // 通攝
  if (is('東韻 一等')) return 'ung';
  if (is('東韻 三等')) return 'iung';
  if (is('冬韻')) return 'uung';
  if (is('鍾韻')) return 'yung';
  // 江攝
  if (is('江韻')) return 'rung';
  // 止攝
  if (is('支韻 合口')) return is('重紐A類') ? 'jye' : 'ye';
  if (is('支韻')) return is('重紐A類') ? 'je' : 'ie';
  if (is('脂韻 合口')) return is('重紐A類') ? 'jyi' : 'yi';
  if (is('脂韻')) return is('重紐A類') ? 'jii' : 'ii';
  if (is('之韻')) return 'i';
  if (is('微韻 開口')) return 'ioi';
  if (is('微韻')) return 'yoi';
  // 遇攝
  if (is('魚韻')) return 'io';
  if (is('虞韻')) return 'yo';
  if (is('模韻')) return 'o';
  // 蟹攝
  if (is('齊韻 合口')) return 'ue';
  if (is('齊韻')) return 'e';
  if (is('祭韻 合口')) return 'yed';
  if (is('祭韻')) return is('重紐A類') ? 'jed' : 'ied';
  if (is('泰韻 合口')) return 'uad';
  if (is('泰韻')) return 'ad';
  if (is('佳韻 合口')) return 'rue';
  if (is('佳韻')) return 're';
  if (is('皆韻 合口')) return 'ruai';
  if (is('皆韻')) return 'rai';
  if (is('夬韻 合口')) return 'ruad';
  if (is('夬韻')) return 'rad';
  if (is('咍韻')) return 'ai';
  if (is('灰韻')) return 'uai';
  if (is('廢韻 開口')) return 'iad';
  if (is('廢韻')) return 'yad';
  // 臻攝
  if (is('眞韻 合口')) return is('重紐A類') ? 'jyn' : 'yn';
  if (is('眞韻')) return is('重紐A類') ? 'jin' : 'in';
  if (is('臻韻')) return 'in';
  if (is('欣韻')) return 'ion';
  if (is('文韻')) return 'yon';
  if (is('元韻 開口')) return 'ian';
  if (is('元韻')) return 'yan';
  if (is('痕韻')) return 'on';
  if (is('魂韻')) return 'uon';
  // 山攝
  if (is('寒韻 開口')) return 'an';
  if (is('寒韻')) return 'uan';
  if (is('刪韻 合口')) return 'ruan';
  if (is('刪韻')) return 'ran';
  if (is('山韻 合口')) return 'ruen';
  if (is('山韻')) return 'ren';
  if (is('仙韻 合口')) return is('重紐A類') ? 'jyen' : 'yen';
  if (is('仙韻')) return is('重紐A類') ? 'jen' : 'ien';
  if (is('先韻 合口')) return 'uen';
  if (is('先韻')) return 'en';
  // 效攝
  if (is('蕭韻')) return 'eu';
  if (is('宵韻')) return is('重紐A類') ? 'jeu' : 'ieu';
  if (is('肴韻')) return 'rau';
  if (is('豪韻')) return 'au';
  // 果攝
  if (is('歌韻 一等 開口')) return 'a';
  if (is('歌韻 一等')) return 'ua';
  if (is('歌韻 三等 開口')) return 'ia';
  if (is('歌韻 三等')) return 'ya';
  // 假攝
  if (is('麻韻 二等 合口')) return 'rua';
  if (is('麻韻 二等')) return 'ra';
  if (is('麻韻 三等')) return 'ia';
  // 宕攝
  if (is('陽韻 開口')) return 'iang';
  if (is('陽韻')) return 'yang';
  if (is('唐韻 合口')) return 'uang';
  if (is('唐韻')) return 'ang';
  // 梗攝
  if (is('庚韻 二等 合口')) return 'ruang';
  if (is('庚韻 二等')) return 'rang';
  if (is('庚韻 三等 合口')) return 'yeng';
  if (is('庚韻 三等')) return 'ieng';
  if (is('耕韻 合口')) return 'rueng';
  if (is('耕韻')) return 'reng';
  if (is('清韻 合口')) return is('重紐A類') ? 'jyeng' : 'yeng';
  if (is('清韻')) return is('重紐A類') ? 'jeng' : 'ieng';
  if (is('青韻 合口')) return 'ueng';
  if (is('青韻')) return 'eng';
  // 曾攝
  if (is('蒸韻 合口')) return 'yng';
  if (is('蒸韻')) return 'ing';
  if (is('登韻 合口')) return 'uong';
  if (is('登韻')) return 'ong';
  // 流攝
  if (is('尤韻')) return 'iu';
  if (is('侯韻')) return 'u';
  if (is('幽韻')) return 'y';
  // 深攝
  if (is('侵韻')) return is('重紐A類') ? 'jim' : 'im';
  // 咸攝
  if (is('覃韻')) return 'om';
  if (is('談韻')) return 'am';
  if (is('鹽韻')) return is('重紐A類') ? 'jem' : 'iem';
  if (is('添韻')) return 'em';
  if (is('咸韻')) return 'rem';
  if (is('銜韻')) return 'ram';
  if (is('嚴韻')) return 'iam';
  if (is('凡韻')) return 'yam';
  throw new Error('無韻母規則');
}

function 聲調規則() {
  if (is('平入聲')) return '';
  if (is('上聲')) return 'x';
  if (is('去聲')) return 'h';
  throw new Error('無聲調規則');
}

let 聲母 = 聲母規則();
let 隔音符號 = "'";
let 韻母 = 韻母規則();
let 聲調 = 聲調規則();

if (is('入聲')) {
  if (韻母.endsWith('m')) 韻母 = `${韻母.slice(0, -1)}p`;
  else if (韻母.endsWith('n')) 韻母 = `${韻母.slice(0, -1)}t`;
  else if (韻母.endsWith('ng')) 韻母 = `${韻母.slice(0, -2)}k`;
}

if (韻母.endsWith('d')) {
  聲調 = '';
}

if (聲母.endsWith('r') && 韻母.startsWith('r')) {
  韻母 = 韻母.slice(1);
}

if (聲母.endsWith('j') && 韻母.startsWith('i') && [...'aeou'].some((x) => 韻母.includes(x))) {
  韻母 = 韻母.slice(1);
}

if (is(`幫組 一二三四等 \
或 端組 一四等 \
或 知組 二三等 \
或 精組 一三四等 \
或 莊組 二三等 \
或 章組 三等 \
或 見溪疑母 一二三四等 \
或 羣母 二三等 \
或 影曉母 一二三四等 \
或 匣母 一二四等 \
或 云以母 三等 \
或 來母 一二三四等 \
或 日母 三等`)) {
  隔音符號 = '';
}

if (is('云母 一二四等')) 聲母 = 'i'; // 如 1444 倄小韻 i'uaix

if (is('端組 三等') && !韻母.startsWith('j')) {
  隔音符號 = ''; // 如 2237 地小韻 diih
}

return 聲母 + 隔音符號 + 韻母 + 聲調;
});

_defaultDerivers.kyonh = _schemata.kyonh();

/**
 * 古韻羅馬字
 *
 * https://zh.wikipedia.org/wiki/User:Polyhedron/中古漢語拼音
 *
 * @author Ayaka
 * @param {Qieyun.音韻地位} 音韻地位 切韻音系音韻地位
 * @param {string=} 字頭 字頭（可選）
 * @param {undefined=} 舊選項參數 已棄用，請用 `<方案名>.方案(選項)(音韻地位, 字頭?)` 代替
 * @return {string} 音韻地位對應的古韻羅馬字
 * @property {推導方案<string>} schema 推導方案，可用於指定推導選項、獲取方案詳細資訊等
 */
export function kyonh(音韻地位, 字頭, 舊選項參數) {
  return _perform("kyonh", _defaultDerivers.kyonh, 音韻地位, 字頭, 舊選項參數);
}
/** @type {推導方案<string>} */
kyonh.schema = _schemata.kyonh

_schemata.zyepheng = new 推導方案(function (選項, 音韻地位, 字頭) {
if (!音韻地位) return [['$legacy', true]];

return {
  EAA: 'tuŋ',
  GAA: 'duŋ',
  JBA: 'tryuŋ',
  LBA: 'dryuŋ',
  XBA: 'tyuŋ',
  KBA: 'thryuŋ',
  UBA: 'dzryuŋ',
  QBA: 'syuŋ',
  cBA: 'nyuŋ',
  dBA: 'kyuŋ',
  lBA: 'jyuŋ',
  kBA: 'xyuŋ',
  DBA: 'myuŋ',
  eBA: 'khyuŋ',
  fBA: 'gyuŋ',
  CBA: 'byuŋ',
  ABA: 'pyuŋ',
  BBA: 'phyuŋ',
  YBA: 'thyuŋ',
  IBA: 'lyuŋ',
  eAA: 'khuŋ',
  dAA: 'kuŋ',
  DAA: 'muŋ',
  IAA: 'luŋ',
  jAA: 'xuŋ',
  PAA: 'dzuŋ',
  hAA: 'quŋ',
  OAA: 'chuŋ',
  FAA: 'thuŋ',
  NAA: 'cuŋ',
  CAA: 'buŋ',
  iAA: 'qhuŋ',
  gAA: 'ŋuŋ',
  QAA: 'suŋ',
  ECA: 'toŋ',
  GCA: 'doŋ',
  PCA: 'dzoŋ',
  HCA: 'noŋ',
  dCA: 'koŋ',
  jCA: 'xoŋ',
  ICA: 'loŋ',
  NCA: 'coŋ',
  QCA: 'soŋ',
  FCA: 'thoŋ',
  XDA: 'tyoŋ',
  IDA: 'lyoŋ',
  aDA: 'shyoŋ',
  RDA: 'zyoŋ',
  YDA: 'thyoŋ',
  lDA: 'jyoŋ',
  ADA: 'pyoŋ',
  iDA: 'qhyoŋ',
  gDA: 'ŋyoŋ',
  hDA: 'qyoŋ',
  MDA: 'nryoŋ',
  LDA: 'dryoŋ',
  PDA: 'dzyoŋ',
  KDA: 'thryoŋ',
  CDA: 'byoŋ',
  BDA: 'phyoŋ',
  NDA: 'cyoŋ',
  cDA: 'nyoŋ',
  fDA: 'gyoŋ',
  ZDA: 'dyoŋ',
  dDA: 'kyoŋ',
  QDA: 'syoŋ',
  ODA: 'chyoŋ',
  eDA: 'khyoŋ',
  dEA: 'kroŋ',
  DEA: 'mroŋ',
  MEA: 'nroŋ',
  TEA: 'chroŋ',
  AEA: 'proŋ',
  jEA: 'xroŋ',
  BEA: 'phroŋ',
  IEA: 'lroŋ',
  VEA: 'sroŋ',
  CEA: 'broŋ',
  iEA: 'qhroŋ',
  hEA: 'qroŋ',
  eEA: 'khroŋ',
  LEA: 'droŋ',
  KEA: 'throŋ',
  JEA: 'troŋ',
  gEA: 'ŋroŋ',
  UEA: 'dzroŋ',
  XFA: 'tie',
  lFA: 'jie',
  kFI: 'xrye',
  dFM: 'krye',
  iFM: 'qhrye',
  hFM: 'qrye',
  DFE: 'mrie',
  iFI: 'qhye',
  LFI: 'drye',
  ZFI: 'dye',
  IFI: 'lye',
  YFI: 'thye',
  BFE: 'phrie',
  AFE: 'prie',
  RFI: 'zye',
  eFM: 'khrye',
  eFI: 'khye',
  fFE: 'grie',
  fFA: 'gie',
  iFE: 'qhrie',
  eFE: 'khrie',
  gFE: 'ŋrie',
  CFE: 'brie',
  ZFA: 'die',
  cFA: 'nie',
  IFA: 'lie',
  PFA: 'dzie',
  NFA: 'cie',
  dFE: 'krie',
  AFA: 'pie',
  CFA: 'bie',
  aFA: 'shie',
  QFA: 'sie',
  TFA: 'chrie',
  KFA: 'thrie',
  DFA: 'mie',
  OFA: 'chie',
  JFA: 'trie',
  hFE: 'qrie',
  LFA: 'drie',
  QFI: 'sye',
  gFM: 'ŋrye',
  iFA: 'qhie',
  VFA: 'srie',
  VFI: 'srye',
  cFI: 'nye',
  dFI: 'kye',
  NFI: 'cye',
  TFI: 'chrye',
  JFI: 'trye',
  YFA: 'thie',
  SFA: 'crie',
  BFA: 'phie',
  lFI: 'jye',
  UFA: 'dzrie',
  XGA: 'ti',
  lGA: 'ji',
  VGA: 'sri',
  CGA: 'bi',
  NGA: 'ci',
  dGE: 'kri',
  YGA: 'thi',
  KGA: 'thri',
  OGA: 'chi',
  PGA: 'dzi',
  MGA: 'nri',
  LGA: 'dri',
  QGA: 'si',
  aGA: 'shi',
  hGA: 'qi',
  IGA: 'li',
  fGI: 'gyi',
  JGI: 'tryi',
  dGM: 'kryi',
  cGI: 'nyi',
  VGI: 'sryi',
  lGI: 'jyi',
  IGI: 'lyi',
  QGI: 'syi',
  fGM: 'gryi',
  DGE: 'mryi',
  AGE: 'pryi',
  XGI: 'tyi',
  ZGI: 'dyi',
  kGI: 'xryi',
  CGE: 'bryi',
  BGE: 'phryi',
  iGI: 'qhyi',
  LGI: 'dryi',
  YGI: 'thyi',
  JGA: 'tri',
  BGA: 'phi',
  NGI: 'cyi',
  eGM: 'khryi',
  gGE: 'ŋri',
  iGA: 'qhi',
  XHA: 'tiə',
  lHA: 'jiə',
  ZHA: 'diə',
  gHA: 'ŋiə',
  QHA: 'siə',
  THA: 'chriə',
  fHA: 'giə',
  aHA: 'shiə',
  cHA: 'niə',
  eHA: 'khiə',
  dHA: 'kiə',
  RHA: 'ziə',
  IHA: 'liə',
  SHA: 'criə',
  iHA: 'qhiə',
  hHA: 'qiə',
  KHA: 'thriə',
  LHA: 'driə',
  YHA: 'thiə',
  PHA: 'dziə',
  NHA: 'ciə',
  UHA: 'dzriə',
  WHA: 'zriə',
  DIA: 'myui',
  iII: 'qhyui',
  kII: 'xyui',
  BIA: 'phyui',
  AIA: 'pyui',
  CIA: 'byui',
  hII: 'qyui',
  fIA: 'giəi',
  dIA: 'kiəi',
  iIA: 'qhiəi',
  hIA: 'qiəi',
  gIA: 'ŋiəi',
  gII: 'ŋyui',
  dII: 'kyui',
  eII: 'khyui',
  gJA: 'ŋio',
  TJA: 'chrio',
  aJA: 'shio',
  dJA: 'kio',
  fJA: 'gio',
  lJA: 'jio',
  QJA: 'sio',
  OJA: 'chio',
  UJA: 'dzrio',
  KJA: 'thrio',
  VJA: 'srio',
  iJA: 'qhio',
  RJA: 'zio',
  hJA: 'qio',
  JJA: 'trio',
  IJA: 'lio',
  XJA: 'tio',
  LJA: 'drio',
  cJA: 'nio',
  NJA: 'cio',
  eJA: 'khio',
  SJA: 'crio',
  ZJA: 'dio',
  MJA: 'nrio',
  gKA: 'ŋyo',
  TKA: 'chryo',
  DKA: 'myo',
  kKA: 'xyo',
  iKA: 'qhyo',
  fKA: 'gyo',
  cKA: 'nyo',
  QKA: 'syo',
  JKA: 'tryo',
  KKA: 'thryo',
  ZKA: 'dyo',
  lKA: 'jyo',
  eKA: 'khyo',
  XKA: 'tyo',
  OKA: 'chyo',
  IKA: 'lyo',
  CKA: 'byo',
  UKA: 'dzryo',
  SKA: 'cryo',
  BKA: 'phyo',
  NKA: 'cyo',
  AKA: 'pyo',
  hKA: 'qyo',
  aKA: 'shyo',
  YKA: 'thyo',
  LKA: 'dryo',
  dKA: 'kyo',
  VKA: 'sryo',
  DLA: 'mo',
  CLA: 'bo',
  jLA: 'xo',
  dLA: 'ko',
  GLA: 'do',
  HLA: 'no',
  iLA: 'qho',
  gLA: 'ŋo',
  NLA: 'co',
  ILA: 'lo',
  QLA: 'so',
  PLA: 'dzo',
  hLA: 'qo',
  ALA: 'po',
  eLA: 'kho',
  OLA: 'cho',
  FLA: 'tho',
  ELA: 'to',
  BLA: 'pho',
  PMA: 'dzei',
  IMA: 'lei',
  OMA: 'chei',
  EMA: 'tei',
  GMA: 'dei',
  AMA: 'pei',
  dMA: 'kei',
  jMA: 'xei',
  hMA: 'qei',
  gMA: 'ŋei',
  iMA: 'qhei',
  QMA: 'sei',
  FMA: 'thei',
  CMA: 'bei',
  BMA: 'phei',
  NMA: 'cei',
  DMA: 'mei',
  HMA: 'nei',
  eMA: 'khei',
  dMI: 'kuei',
  eMI: 'khuei',
  jMI: 'xuei',
  cMA: 'nьʼei',
  ZMA: 'dьʼei',
  hMI: 'quei',
  iMI: 'qhuei',
  dPA: 'kre',
  jPA: 'xre',
  CPA: 'bre',
  dPI: 'krue',
  hPI: 'qrue',
  ePI: 'khrue',
  UPA: 'dzre',
  TPA: 'chre',
  iPI: 'qhrue',
  MPA: 'nre',
  gPA: 'ŋre',
  hPA: 'qre',
  VPA: 'sre',
  iPA: 'qhre',
  KPA: 'thre',
  DPA: 'mre',
  jPI: 'xrue',
  dQA: 'krei',
  hQA: 'qrei',
  jQA: 'xrei',
  CQA: 'brei',
  dQI: 'kruei',
  jQI: 'xruei',
  eQI: 'khruei',
  UQA: 'dzrei',
  TQA: 'chrei',
  iQI: 'qhruei',
  LQI: 'druei',
  DQA: 'mrei',
  SQA: 'crei',
  hQI: 'qruei',
  JQA: 'trei',
  IQA: 'lrei',
  eQA: 'khrei',
  MQA: 'nrei',
  VQA: 'srei',
  gQA: 'ŋrei',
  KQA: 'threi',
  iQA: 'qhrei',
  IQI: 'lruei',
  UQI: 'dzruei',
  iSI: 'qhui',
  eSI: 'khui',
  hSI: 'qui',
  jSI: 'xui',
  DSA: 'mui',
  dSI: 'kui',
  ISI: 'lui',
  GSI: 'dui',
  OSI: 'chui',
  ESI: 'tui',
  QSI: 'sui',
  PSI: 'dzui',
  CSA: 'bui',
  ASA: 'pui',
  BSA: 'phui',
  gSI: 'ŋui',
  FSI: 'thui',
  HSI: 'nui',
  NSI: 'cui',
  iTA: 'qhəi',
  eTA: 'khəi',
  hTA: 'qəi',
  GTA: 'dəi',
  dTA: 'kəi',
  PTA: 'dzəi',
  ITA: 'ləi',
  NTA: 'cəi',
  OTA: 'chəi',
  FTA: 'thəi',
  jTA: 'xəi',
  QTA: 'səi',
  gTA: 'ŋəi',
  HTA: 'nəi',
  ETA: 'təi',
  YTA: 'thьʼəi',
  XVA: 'tin',
  KVA: 'thrin',
  hVA: 'qin',
  QVA: 'sin',
  ZVA: 'din',
  cVA: 'nin',
  bVA: 'zhin',
  OVA: 'chin',
  kVA: 'xrin',
  aVA: 'shin',
  AVA: 'pin',
  IVA: 'lin',
  fVE: 'grin',
  JVA: 'trin',
  LVA: 'drin',
  NVA: 'cin',
  YVA: 'thin',
  PVA: 'dzin',
  lVA: 'jin',
  MVA: 'nrin',
  BVA: 'phin',
  CVA: 'bin',
  gVE: 'ŋrin',
  dVE: 'krin',
  dVM: 'kryn',
  kVI: 'xryn',
  eVM: 'khryn',
  DVE: 'mryn',
  CVE: 'bryn',
  hVE: 'qrin',
  hVM: 'qryn',
  AVE: 'pryn',
  DVA: 'min',
  XVI: 'tyn',
  KVI: 'thryn',
  LVI: 'dryn',
  QVI: 'syn',
  ZVI: 'dyn',
  cVI: 'nyn',
  bVI: 'zhyn',
  IVI: 'lyn',
  JVI: 'tryn',
  OVI: 'chyn',
  NVI: 'cyn',
  YVI: 'thyn',
  PVI: 'dzyn',
  lVI: 'jyn',
  RVI: 'zyn',
  dVI: 'kyn',
  fVA: 'gin',
  BVE: 'phryn',
  SWA: 'crin',
  VWA: 'srin',
  UWA: 'dzrin',
  DXA: 'myun',
  kXI: 'xyun',
  hXI: 'qyun',
  CXA: 'byun',
  AXA: 'pyun',
  fXI: 'gyun',
  iXI: 'qhyun',
  dXI: 'kyun',
  BXA: 'phyun',
  iYA: 'qhiən',
  hYA: 'qiən',
  fYA: 'giən',
  dYA: 'kiən',
  gYA: 'ŋiən',
  gZI: 'ŋyon',
  kZI: 'xyon',
  CZA: 'byon',
  BZA: 'phyon',
  iZI: 'qhyon',
  hZI: 'qyon',
  gZA: 'ŋian',
  eZA: 'khian',
  iZA: 'qhian',
  dZA: 'kian',
  hZA: 'qian',
  AZA: 'pyon',
  fZA: 'gian',
  DZA: 'myon',
  jaI: 'xun',
  daI: 'kun',
  haI: 'qun',
  DaA: 'mun',
  QaI: 'sun',
  NaI: 'cun',
  PaI: 'dzun',
  EaI: 'tun',
  FaI: 'thun',
  GaI: 'dun',
  OaI: 'chun',
  gaI: 'ŋun',
  CaA: 'bun',
  AaA: 'pun',
  IaI: 'lun',
  eaI: 'khun',
  iaI: 'qhun',
  BaA: 'phun',
  HaI: 'nun',
  jbA: 'xən',
  dbA: 'kən',
  hbA: 'qən',
  FbA: 'thən',
  gbA: 'ŋən',
  jcA: 'xan',
  gcA: 'ŋan',
  EcA: 'tan',
  hcA: 'qan',
  HcA: 'nan',
  OcA: 'chan',
  FcA: 'than',
  QcA: 'san',
  GcA: 'dan',
  PcA: 'dzan',
  dcA: 'kan',
  IcA: 'lan',
  ecA: 'khan',
  icA: 'qhan',
  HcI: 'nuan',
  jcI: 'xuan',
  gcI: 'ŋuan',
  EcI: 'tuan',
  hcI: 'quan',
  FcI: 'thuan',
  QcI: 'suan',
  GcI: 'duan',
  PcI: 'dzuan',
  dcI: 'kuan',
  IcI: 'luan',
  icI: 'qhuan',
  ecI: 'khuan',
  NcI: 'cuan',
  CcA: 'ban',
  DcA: 'man',
  BcA: 'phan',
  AcA: 'pan',
  VdA: 'sran',
  ddI: 'kruan',
  hdI: 'qruan',
  VdI: 'sruan',
  jdI: 'xruan',
  AdA: 'pran',
  DdA: 'mran',
  gdA: 'ŋran',
  ddA: 'kran',
  BdA: 'phran',
  MdI: 'nruan',
  gdI: 'ŋruan',
  edA: 'khran',
  SdI: 'cruan',
  VeA: 'sren',
  deI: 'kruen',
  deA: 'kren',
  jeA: 'xren',
  eeA: 'khren',
  UeA: 'dzren',
  ieA: 'qhren',
  geA: 'ŋren',
  heA: 'qren',
  AeA: 'pren',
  IeA: 'lren',
  MeA: 'nren',
  YeA: 'thьʼren',
  heI: 'qruen',
  LeI: 'druen',
  IeI: 'lruen',
  feI: 'gruen',
  JeA: 'tren',
  LeA: 'dren',
  jeI: 'xruen',
  QgA: 'sen',
  PgA: 'dzen',
  OgA: 'chen',
  NgA: 'cen',
  FgA: 'then',
  dgA: 'ken',
  jgA: 'xen',
  hgA: 'qen',
  IgA: 'len',
  GgA: 'den',
  HgA: 'nen',
  EgA: 'ten',
  egA: 'khen',
  ggA: 'ŋen',
  DgA: 'men',
  CgA: 'ben',
  hgI: 'quen',
  dgI: 'kuen',
  igI: 'qhuen',
  AgA: 'pen',
  jgI: 'xuen',
  igA: 'qhen',
  UgI: 'dzrʼuen',
  QfA: 'sien',
  PfA: 'dzien',
  OfA: 'chien',
  NfA: 'cien',
  cfA: 'nien',
  lfA: 'jien',
  XfA: 'tien',
  dfA: 'kien',
  JfA: 'trien',
  UfA: 'dzrien',
  afA: 'shien',
  KfA: 'thrien',
  ZfA: 'dien',
  LfA: 'drien',
  ifE: 'qhrien',
  IfA: 'lien',
  BfA: 'phien',
  CfA: 'bien',
  DfA: 'mien',
  PfI: 'dzyen',
  QfI: 'syen',
  NfI: 'cyen',
  ifI: 'qhyen',
  cfI: 'nyen',
  YfI: 'thyen',
  lfI: 'jyen',
  RfI: 'zyen',
  hfI: 'qyen',
  bfI: 'zhyen',
  AfA: 'pien',
  RfA: 'zien',
  OfI: 'chyen',
  XfI: 'tyen',
  ZfI: 'dyen',
  kfI: 'xryen',
  SfI: 'cryen',
  VfI: 'sryen',
  KfI: 'thryen',
  ffE: 'grien',
  efE: 'khrien',
  ffM: 'gryen',
  LfI: 'dryen',
  IfI: 'lyen',
  efM: 'khryen',
  hfE: 'qrien',
  kfA: 'xrien',
  hfM: 'qryen',
  YfA: 'thien',
  JfI: 'tryen',
  dfM: 'kryen',
  QhA: 'seu',
  FhA: 'theu',
  EhA: 'teu',
  GhA: 'deu',
  dhA: 'keu',
  IhA: 'leu',
  ghA: 'ŋeu',
  ihA: 'qheu',
  hhA: 'qeu',
  ehA: 'kheu',
  QiA: 'sieu',
  KiA: 'thrieu',
  JiA: 'trieu',
  LiA: 'drieu',
  iiE: 'qhrieu',
  PiA: 'dzieu',
  diE: 'krieu',
  NiA: 'cieu',
  ciA: 'nieu',
  aiA: 'shieu',
  liA: 'jieu',
  ZiA: 'dieu',
  XiA: 'tieu',
  AiA: 'pieu',
  AiE: 'prieu',
  CiA: 'bieu',
  DiA: 'mieu',
  DiE: 'mrieu',
  hiA: 'qieu',
  kiA: 'xrieu',
  fiE: 'grieu',
  OiA: 'chieu',
  hiE: 'qrieu',
  eiA: 'khieu',
  YiA: 'thieu',
  BiA: 'phieu',
  fiA: 'gieu',
  IiA: 'lieu',
  eiE: 'khrieu',
  jjA: 'xrau',
  djA: 'krau',
  UjA: 'dzrau',
  MjA: 'nrau',
  VjA: 'srau',
  DjA: 'mrau',
  ijA: 'qhrau',
  AjA: 'prau',
  BjA: 'phrau',
  ejA: 'khrau',
  gjA: 'ŋrau',
  SjA: 'crau',
  JjA: 'trau',
  TjA: 'chrau',
  CjA: 'brau',
  hjA: 'qrau',
  KjA: 'thrau',
  IjA: 'lrau',
  LjA: 'drau',
  jkA: 'xau',
  dkA: 'kau',
  IkA: 'lau',
  ikA: 'qhau',
  DkA: 'mau',
  FkA: 'thau',
  EkA: 'tau',
  QkA: 'sau',
  CkA: 'bau',
  AkA: 'pau',
  GkA: 'dau',
  NkA: 'cau',
  gkA: 'ŋau',
  PkA: 'dzau',
  hkA: 'qau',
  HkA: 'nau',
  ekA: 'khau',
  OkA: 'chau',
  BkA: 'phau',
  dlA: 'ka',
  OlA: 'cha',
  ElA: 'ta',
  QlA: 'sa',
  GlA: 'da',
  PlA: 'dza',
  glA: 'ŋa',
  FlA: 'tha',
  IlA: 'la',
  HlA: 'na',
  jlA: 'xa',
  ilA: 'qha',
  elA: 'kha',
  hlA: 'qa',
  dlI: 'kua',
  OlI: 'chua',
  ElI: 'tua',
  QlI: 'sua',
  ClA: 'ba',
  GlI: 'dua',
  DlA: 'ma',
  PlI: 'dzua',
  glI: 'ŋua',
  FlI: 'thua',
  IlI: 'lua',
  HlI: 'nua',
  AlA: 'pa',
  BlA: 'pha',
  jlI: 'xua',
  elI: 'khua',
  hlI: 'qua',
  imI: 'qhya',
  hmI: 'qya',
  emI: 'khya',
  fmA: 'gia',
  emA: 'khia',
  dmA: 'kia',
  OmI: 'chya',
  NmI: 'cya',
  fmI: 'gya',
  ImI: 'lya',
  DnA: 'mra',
  YoA: 'thia',
  aoA: 'shia',
  loA: 'jia',
  XoA: 'tia',
  NoA: 'cia',
  boA: 'zhia',
  jnI: 'xrua',
  dnI: 'krua',
  inI: 'qhrua',
  enI: 'khrua',
  MnA: 'nra',
  dnA: 'kra',
  jnA: 'xra',
  BnA: 'phra',
  hnA: 'qra',
  AnA: 'pra',
  TnA: 'chra',
  VnA: 'sra',
  gnA: 'ŋra',
  SnA: 'cra',
  LnA: 'dra',
  RoA: 'zia',
  ZoA: 'dia',
  hnI: 'qrua',
  SnI: 'crua',
  JnI: 'trua',
  CnA: 'bra',
  UnA: 'dzra',
  KnA: 'thra',
  JnA: 'tra',
  inA: 'qhra',
  enA: 'khra',
  PoA: 'dzia',
  coA: 'nia',
  QoA: 'sia',
  JoA: 'tria',
  gnI: 'ŋrua',
  lpA: 'jiaŋ',
  RpA: 'ziaŋ',
  IpA: 'liaŋ',
  ipA: 'qhiaŋ',
  apA: 'shiaŋ',
  CpA: 'byaŋ',
  XpA: 'tiaŋ',
  YpA: 'thiaŋ',
  epA: 'khiaŋ',
  dpA: 'kiaŋ',
  LpA: 'driaŋ',
  JpA: 'triaŋ',
  cpA: 'niaŋ',
  ApA: 'pyaŋ',
  QpA: 'siaŋ',
  NpA: 'ciaŋ',
  TpA: 'chriaŋ',
  DpA: 'myaŋ',
  MpA: 'nriaŋ',
  UpA: 'dzriaŋ',
  SpA: 'criaŋ',
  ZpA: 'diaŋ',
  VpA: 'sriaŋ',
  PpA: 'dziaŋ',
  OpA: 'chiaŋ',
  epI: 'khyaŋ',
  kpI: 'xyaŋ',
  hpA: 'qiaŋ',
  fpA: 'giaŋ',
  KpA: 'thriaŋ',
  BpA: 'phyaŋ',
  fpI: 'gyaŋ',
  GqA: 'daŋ',
  IqA: 'laŋ',
  EqA: 'taŋ',
  OqA: 'chaŋ',
  dqA: 'kaŋ',
  QqA: 'saŋ',
  eqA: 'khaŋ',
  iqI: 'qhuaŋ',
  jqI: 'xuaŋ',
  dqI: 'kuaŋ',
  FqA: 'thaŋ',
  BqA: 'phaŋ',
  hqI: 'quaŋ',
  hqA: 'qaŋ',
  iqA: 'qhaŋ',
  jqA: 'xaŋ',
  DqA: 'maŋ',
  NqA: 'caŋ',
  HqA: 'naŋ',
  CqA: 'baŋ',
  gqA: 'ŋaŋ',
  PqA: 'dzaŋ',
  eqI: 'khuaŋ',
  AqA: 'paŋ',
  drA: 'kraŋ',
  erA: 'khraŋ',
  DrA: 'mraŋ',
  jrI: 'xruaŋ',
  ArA: 'praŋ',
  irI: 'qhruaŋ',
  drI: 'kruaŋ',
  CrA: 'braŋ',
  irA: 'qhraŋ',
  KrA: 'thraŋ',
  TrA: 'chraŋ',
  UrA: 'dzraŋ',
  hsA: 'qrieŋ',
  BrA: 'phraŋ',
  CsA: 'brieŋ',
  dsA: 'krieŋ',
  DsA: 'mrieŋ',
  LrA: 'draŋ',
  JrA: 'traŋ',
  ksI: 'xryeŋ',
  AsA: 'prieŋ',
  isI: 'qhryeŋ',
  esA: 'khrieŋ',
  fsA: 'grieŋ',
  gsA: 'ŋrieŋ',
  jrA: 'xraŋ',
  MrA: 'nraŋ',
  dtA: 'kreŋ',
  etA: 'khreŋ',
  DtA: 'mreŋ',
  AtA: 'preŋ',
  jtI: 'xrueŋ',
  jtA: 'xreŋ',
  JtA: 'treŋ',
  htA: 'qreŋ',
  UtA: 'dzreŋ',
  TtA: 'chreŋ',
  MtA: 'nreŋ',
  BtA: 'phreŋ',
  itI: 'qhrueŋ',
  LtA: 'dreŋ',
  htI: 'qrueŋ',
  CtA: 'breŋ',
  gtA: 'ŋreŋ',
  StA: 'creŋ',
  OuA: 'chieŋ',
  PuA: 'dzieŋ',
  NuA: 'cieŋ',
  luA: 'jieŋ',
  luI: 'jyeŋ',
  huA: 'qieŋ',
  JuA: 'trieŋ',
  KuA: 'thrieŋ',
  ZuA: 'dieŋ',
  LuA: 'drieŋ',
  auA: 'shieŋ',
  XuA: 'tieŋ',
  euA: 'khieŋ',
  DuA: 'mieŋ',
  IuA: 'lieŋ',
  AuA: 'pieŋ',
  euI: 'khyeŋ',
  RuA: 'zieŋ',
  huI: 'qyeŋ',
  fuI: 'gyeŋ',
  QuI: 'syeŋ',
  fuA: 'gieŋ',
  iuI: 'qhyeŋ',
  OvA: 'cheŋ',
  dvA: 'keŋ',
  jvA: 'xeŋ',
  GvA: 'deŋ',
  EvA: 'teŋ',
  ivA: 'qheŋ',
  QvA: 'seŋ',
  BvA: 'pheŋ',
  IvA: 'leŋ',
  HvA: 'neŋ',
  FvA: 'theŋ',
  DvA: 'meŋ',
  CvA: 'beŋ',
  jvI: 'xueŋ',
  dvI: 'kueŋ',
  XwA: 'tiəŋ',
  ZwA: 'diəŋ',
  LwA: 'driəŋ',
  IwA: 'liəŋ',
  hwA: 'qiəŋ',
  CwA: 'bryəŋ',
  AwA: 'pryəŋ',
  lwA: 'jiəŋ',
  bwA: 'zhiəŋ',
  awA: 'shiəŋ',
  cwA: 'niəŋ',
  dwA: 'kiəŋ',
  JwA: 'triəŋ',
  PwA: 'dziəŋ',
  gwA: 'ŋiəŋ',
  iwA: 'qhiəŋ',
  YwA: 'thiəŋ',
  fwA: 'giəŋ',
  VwA: 'sriəŋ',
  ewA: 'khiəŋ',
  KwA: 'thriəŋ',
  UwA: 'dzriəŋ',
  BwA: 'phryəŋ',
  ExA: 'təŋ',
  IxA: 'ləŋ',
  QxA: 'səŋ',
  AxA: 'pəŋ',
  NxA: 'cəŋ',
  DxA: 'məŋ',
  PxA: 'dzəŋ',
  CxA: 'bəŋ',
  jxI: 'xuəŋ',
  dxI: 'kuəŋ',
  ixI: 'qhuəŋ',
  HxA: 'nəŋ',
  GxA: 'dəŋ',
  jxA: 'xəŋ',
  dxA: 'kəŋ',
  FxA: 'thəŋ',
  BxA: 'phəŋ',
  kyA: 'xyu',
  hyA: 'qyu',
  IyA: 'lyu',
  OyA: 'chyu',
  lyA: 'jyu',
  gyA: 'ŋyu',
  NyA: 'cyu',
  PyA: 'dzyu',
  QyA: 'syu',
  KyA: 'thryu',
  eyA: 'khyu',
  YyA: 'thyu',
  XyA: 'tyu',
  ZyA: 'dyu',
  cyA: 'nyu',
  ayA: 'shyu',
  ByA: 'phyu',
  dyA: 'kyu',
  AyA: 'pyu',
  VyA: 'sryu',
  TyA: 'chryu',
  SyA: 'cryu',
  UyA: 'dzryu',
  iyA: 'qhyu',
  RyA: 'zyu',
  LyA: 'dryu',
  JyA: 'tryu',
  fyA: 'gyu',
  CyA: 'byu',
  DyA: 'myu',
  jzA: 'xu',
  hzA: 'qu',
  HzA: 'nu',
  IzA: 'lu',
  QzA: 'su',
  ezA: 'khu',
  izA: 'qhu',
  NzA: 'cu',
  FzA: 'thu',
  GzA: 'du',
  gzA: 'ŋu',
  dzA: 'ku',
  EzA: 'tu',
  PzA: 'dzu',
  CzA: 'bu',
  OzA: 'chu',
  DzA: 'mu',
  h0A: 'qiu',
  f0A: 'griu',
  A0A: 'piu',
  I0A: 'liu',
  d0A: 'kriu',
  C0A: 'briu',
  N0A: 'ciu',
  V0A: 'sriu',
  g0A: 'ŋiu',
  i0A: 'qhriu',
  D0A: 'miu',
  O1A: 'chim',
  R1A: 'zim',
  I1A: 'lim',
  K1A: 'thrim',
  X1A: 'tim',
  L1A: 'drim',
  J1A: 'trim',
  Z1A: 'dim',
  c1A: 'nim',
  a1A: 'shim',
  l1A: 'jim',
  Q1A: 'sim',
  h1A: 'qim',
  N1A: 'cim',
  P1A: 'dzim',
  M1A: 'nrim',
  f1E: 'grim',
  e1E: 'khrim',
  g1E: 'ŋrim',
  i1E: 'qhrim',
  d1E: 'krim',
  h1E: 'qrim',
  V1A: 'srim',
  U1A: 'dzrim',
  S1A: 'crim',
  T1A: 'chrim',
  Y1A: 'thim',
  G2A: 'dəm',
  O2A: 'chəm',
  H2A: 'nəm',
  h2A: 'qəm',
  j2A: 'xəm',
  I2A: 'ləm',
  P2A: 'dzəm',
  N2A: 'cəm',
  F2A: 'thəm',
  E2A: 'təm',
  e2A: 'khəm',
  i2A: 'qhəm',
  Q2A: 'səm',
  d2A: 'kəm',
  g2A: 'ŋəm',
  G3A: 'dam',
  d3A: 'kam',
  E3A: 'tam',
  Q3A: 'sam',
  I3A: 'lam',
  e3A: 'kham',
  F3A: 'tham',
  P3A: 'dzam',
  j3A: 'xam',
  D3A: 'mam',
  i3A: 'qham',
  l4A: 'jiem',
  I4A: 'liem',
  A4E: 'priem',
  Q4A: 'siem',
  O4A: 'chiem',
  X4A: 'tiem',
  Z4A: 'diem',
  a4A: 'shiem',
  Y4A: 'thiem',
  c4A: 'niem',
  M4A: 'nriem',
  k4A: 'xriem',
  J4A: 'triem',
  K4A: 'thriem',
  h4E: 'qriem',
  e4E: 'khriem',
  g4E: 'ŋriem',
  N4A: 'ciem',
  P4A: 'dziem',
  f4E: 'griem',
  h4A: 'qiem',
  R4A: 'ziem',
  V4A: 'sriem',
  L4A: 'driem',
  f4A: 'giem',
  F5A: 'them',
  E5A: 'tem',
  G5A: 'dem',
  I5A: 'lem',
  e5A: 'khem',
  d5A: 'kem',
  j5A: 'xem',
  H5A: 'nem',
  i5A: 'qhem',
  j6A: 'xrem',
  d6A: 'krem',
  V6A: 'srem',
  h6A: 'qrem',
  g6A: 'ŋrem',
  i6A: 'qhrem',
  J6A: 'trem',
  M6A: 'nrem',
  U6A: 'dzrem',
  e6A: 'khrem',
  j7A: 'xram',
  U7A: 'dzram',
  g7A: 'ŋram',
  T7A: 'chram',
  V7A: 'sram',
  d7A: 'kram',
  C7A: 'bram',
  e7A: 'khram',
  g8A: 'ŋiam',
  i8A: 'qhiam',
  h8A: 'qiam',
  e8A: 'khiam',
  C9A: 'byom',
  B9A: 'phyom',
  EAB: 'túŋ',
  DAB: 'múŋ',
  eAB: 'khúŋ',
  QAB: 'súŋ',
  FAB: 'thúŋ',
  NAB: 'cúŋ',
  jAB: 'xúŋ',
  hAB: 'qúŋ',
  HAB: 'núŋ',
  AAB: 'púŋ',
  IAB: 'lúŋ',
  iAB: 'qhúŋ',
  GAB: 'dúŋ',
  CAB: 'búŋ',
  XDB: 'tyóŋ',
  KDB: 'thryóŋ',
  IDB: 'lyóŋ',
  hDB: 'qyóŋ',
  cDB: 'nyóŋ',
  LDB: 'dryóŋ',
  JDB: 'tryóŋ',
  CDB: 'byóŋ',
  BDB: 'phyóŋ',
  lDB: 'jyóŋ',
  eDB: 'khyóŋ',
  ZDB: 'dyóŋ',
  dDB: 'kyóŋ',
  QDB: 'syóŋ',
  iDB: 'qhyóŋ',
  ADB: 'pyóŋ',
  ECB: 'tóŋ',
  DCB: 'móŋ',
  fDB: 'gyóŋ',
  YDB: 'thyóŋ',
  NDB: 'cyóŋ',
  dEB: 'króŋ',
  CEB: 'bróŋ',
  hEB: 'qróŋ',
  DEB: 'mróŋ',
  jEB: 'xróŋ',
  AEB: 'próŋ',
  iEB: 'qhróŋ',
  XFB: 'tié',
  ZFB: 'dié',
  DFF: 'mrié',
  AFF: 'prié',
  CFF: 'brié',
  iFN: 'qhryé',
  hFN: 'qryé',
  eFN: 'khryé',
  dFN: 'kryé',
  QFJ: 'syé',
  IFJ: 'lyé',
  fFF: 'grié',
  hFF: 'qrié',
  dFF: 'krié',
  eFF: 'khrié',
  gFF: 'ŋrié',
  kFJ: 'xryé',
  NFJ: 'cyé',
  cFJ: 'nyé',
  OFB: 'chié',
  LFB: 'drié',
  QFB: 'sié',
  lFB: 'jié',
  IFB: 'lié',
  VFB: 'srié',
  AFB: 'pié',
  cFB: 'nié',
  DFB: 'mié',
  CFB: 'bié',
  YFB: 'thié',
  aFB: 'shié',
  NFB: 'cié',
  XFJ: 'tyé',
  TFJ: 'chryé',
  RFJ: 'zyé',
  bFB: 'zhié',
  SFB: 'crié',
  BFF: 'phrié',
  BFB: 'phié',
  lFJ: 'jyé',
  PFJ: 'dzyé',
  eFJ: 'khyé',
  MFB: 'nrié',
  gFN: 'ŋryé',
  fFN: 'gryé',
  KFB: 'thrié',
  JFB: 'trié',
  iFF: 'qhrié',
  eFB: 'khié',
  ZFJ: 'dyé',
  dFB: 'kié',
  XGB: 'tí',
  ZGB: 'dí',
  DGF: 'mrýi',
  AGF: 'prýi',
  RGB: 'zí',
  dGF: 'krí',
  NGB: 'cí',
  AGB: 'pí',
  dGN: 'krýi',
  kGJ: 'xrýi',
  aGB: 'shí',
  LGB: 'drí',
  QGB: 'sí',
  CGB: 'bí',
  IGB: 'lí',
  aGJ: 'shýi',
  IGJ: 'lýi',
  fGJ: 'gýi',
  OGJ: 'chýi',
  MGB: 'nrí',
  dGJ: 'kýi',
  CGF: 'brýi',
  PGJ: 'dzýi',
  BGF: 'phrýi',
  cGJ: 'nýi',
  lGJ: 'jýi',
  hGF: 'qrí',
  NGJ: 'cýi',
  JGB: 'trí',
  KGB: 'thrí',
  iGJ: 'qhýi',
  fGN: 'grýi',
  fGF: 'grí',
  eGN: 'khrýi',
  XHB: 'tiə́',
  ZHB: 'diə́',
  JHB: 'triə́',
  iHB: 'qhiə́',
  dHB: 'kiə́',
  lHB: 'jiə́',
  RHB: 'ziə́',
  VHB: 'sriə́',
  cHB: 'niə́',
  IHB: 'liə́',
  QHB: 'siə́',
  aHB: 'shiə́',
  LHB: 'driə́',
  eHB: 'khiə́',
  UHB: 'dzriə́',
  WHB: 'zriə́',
  NHB: 'ciə́',
  kHB: 'xiə́',
  gHB: 'ŋiə́',
  YHB: 'thiə́',
  KHB: 'thriə́',
  THB: 'chriə́',
  SHB: 'criə́',
  hHB: 'qiə́',
  MHB: 'nriə́',
  DIB: 'myúi',
  hIB: 'qiə́i',
  eIB: 'khiə́i',
  dIB: 'kiə́i',
  BIB: 'phyúi',
  AIB: 'pyúi',
  kIJ: 'xyúi',
  dIJ: 'kyúi',
  iIJ: 'qhyúi',
  gIB: 'ŋiə́i',
  iIB: 'qhiə́i',
  hIJ: 'qyúi',
  CIB: 'byúi',
  gJB: 'ŋió',
  IJB: 'lió',
  LJB: 'drió',
  lJB: 'jió',
  XJB: 'tió',
  cJB: 'nió',
  aJB: 'shió',
  YJB: 'thió',
  JJB: 'trió',
  QJB: 'sió',
  KJB: 'thrió',
  MJB: 'nrió',
  iJB: 'qhió',
  fJB: 'gió',
  VJB: 'srió',
  TJB: 'chrió',
  SJB: 'crió',
  UJB: 'dzrió',
  PJB: 'dzió',
  hJB: 'qió',
  dJB: 'kió',
  RJB: 'zió',
  eJB: 'khió',
  bJB: 'zhió',
  ZJB: 'dió',
  OJB: 'chió',
  NJB: 'ció',
  gKB: 'ŋyó',
  kKB: 'xyó',
  PKB: 'dzyó',
  AKB: 'pyó',
  DKB: 'myó',
  CKB: 'byó',
  BKB: 'phyó',
  LKB: 'dryó',
  iKB: 'qhyó',
  ZKB: 'dyó',
  lKB: 'jyó',
  XKB: 'tyó',
  hKB: 'qyó',
  eKB: 'khyó',
  JKB: 'tryó',
  cKB: 'nyó',
  fKB: 'gyó',
  VKB: 'sryó',
  dKB: 'kyó',
  OKB: 'chyó',
  IKB: 'lyó',
  QKB: 'syó',
  UKB: 'dzryó',
  DLB: 'mó',
  FLB: 'thó',
  GLB: 'dó',
  ILB: 'ló',
  OLB: 'chó',
  ELB: 'tó',
  dLB: 'kó',
  gLB: 'ŋó',
  CLB: 'bó',
  PLB: 'dzó',
  NLB: 'có',
  iLB: 'qhó',
  hLB: 'qó',
  eLB: 'khó',
  HLB: 'nó',
  jLB: 'xó',
  BLB: 'phó',
  ALB: 'pó',
  PMB: 'dzéi',
  IMB: 'léi',
  FMB: 'théi',
  BMB: 'phéi',
  NMB: 'céi',
  EMB: 'téi',
  GMB: 'déi',
  HMB: 'néi',
  QMB: 'séi',
  OMB: 'chéi',
  eMB: 'khéi',
  DMB: 'méi',
  CMB: 'béi',
  hMB: 'qéi',
  gMB: 'ŋéi',
  AMB: 'péi',
  jPB: 'xré',
  DPB: 'mré',
  ePB: 'khré',
  LPB: 'dré',
  MPB: 'nré',
  CPB: 'bré',
  hPB: 'qré',
  APB: 'pré',
  dPB: 'kré',
  VPB: 'sré',
  dPJ: 'krué',
  LPJ: 'drué',
  jPJ: 'xrué',
  iPJ: 'qhrué',
  fPB: 'gré',
  jQB: 'xréi',
  eQB: 'khréi',
  gQB: 'ŋréi',
  hQB: 'qréi',
  iSJ: 'qhúi',
  hSJ: 'qúi',
  ISJ: 'lúi',
  GSJ: 'dúi',
  PSJ: 'dzúi',
  DSB: 'múi',
  FSJ: 'thúi',
  jSJ: 'xúi',
  eSJ: 'khúi',
  ESJ: 'túi',
  HSJ: 'núi',
  gSJ: 'ŋúi',
  OSJ: 'chúi',
  CSB: 'búi',
  NSJ: 'cúi',
  iTB: 'qhə́i',
  eTB: 'khə́i',
  NTB: 'cə́i',
  GTB: 'də́i',
  HTB: 'nə́i',
  dTB: 'kə́i',
  jTB: 'xə́i',
  OTB: 'chə́i',
  YTB: 'thьʼə́i',
  ETB: 'tə́i',
  PTB: 'dzə́i',
  hTB: 'qə́i',
  lTB: 'jьʼə́i',
  FTB: 'thə́i',
  cTB: 'nьʼə́i',
  ITB: 'lə́i',
  XVB: 'tín',
  KVB: 'thrín',
  ZVB: 'dín',
  cVB: 'nín',
  aVB: 'shín',
  IVB: 'lín',
  LVB: 'drín',
  dVB: 'kín',
  PVB: 'dzín',
  NVB: 'cín',
  CVB: 'bín',
  gVF: 'ŋrín',
  OVB: 'chín',
  fVN: 'grýn',
  lVB: 'jín',
  DVF: 'mrýn',
  DVB: 'mín',
  kVJ: 'xrýn',
  XVJ: 'týn',
  lVJ: 'jýn',
  QVJ: 'sýn',
  cVJ: 'nýn',
  YVJ: 'thýn',
  bVJ: 'zhýn',
  KVJ: 'thrýn',
  IVJ: 'lýn',
  eVB: 'khín',
  aVJ: 'shýn',
  UVB: 'dzrín',
  JVB: 'trín',
  DXB: 'myún',
  AXB: 'pyún',
  CXB: 'byún',
  BXB: 'phyún',
  hXJ: 'qyún',
  gXJ: 'ŋyún',
  kXJ: 'xyún',
  eXJ: 'khyún',
  hYB: 'qiə́n',
  dYB: 'kiə́n',
  eYB: 'khiə́n',
  fYB: 'giə́n',
  gYB: 'ŋiə́n',
  iYB: 'qhiə́n',
  gZJ: 'ŋyón',
  kZJ: 'xyón',
  hZB: 'qián',
  dZB: 'kián',
  fZB: 'gián',
  eZB: 'khián',
  gZB: 'ŋián',
  iZB: 'qhián',
  DZB: 'myón',
  AZB: 'pyón',
  fZJ: 'gyón',
  hZJ: 'qyón',
  eZJ: 'khyón',
  iZJ: 'qhyón',
  CZB: 'byón',
  jaJ: 'xún',
  BaB: 'phún',
  OaJ: 'chún',
  AaB: 'pún',
  QaJ: 'sún',
  NaJ: 'cún',
  haJ: 'qún',
  GaJ: 'dún',
  PaJ: 'dzún',
  daJ: 'kún',
  FaJ: 'thún',
  eaJ: 'khún',
  IaJ: 'lún',
  CaB: 'bún',
  DaB: 'mún',
  HaJ: 'nún',
  iaJ: 'qhún',
  jbB: 'xə́n',
  ebB: 'khə́n',
  dbB: 'kə́n',
  jcB: 'xán',
  EcB: 'tán',
  FcB: 'thán',
  QcB: 'sán',
  GcB: 'dán',
  PcB: 'dzán',
  dcB: 'kán',
  IcB: 'lán',
  ecB: 'khán',
  icB: 'qhán',
  NcB: 'cán',
  jcJ: 'xuán',
  EcJ: 'tuán',
  hcJ: 'quán',
  FcJ: 'thuán',
  QcJ: 'suán',
  dcJ: 'kuán',
  IcJ: 'luán',
  ecJ: 'khuán',
  HcJ: 'nuán',
  NcJ: 'cuán',
  CcB: 'bán',
  DcB: 'mán',
  AcB: 'pán',
  RcJ: 'zuán',
  GcJ: 'duán',
  HcB: 'nán',
  BcB: 'phán',
  VdB: 'srán',
  hdJ: 'qruán',
  AdB: 'prán',
  SdB: 'crán',
  MdB: 'nrán',
  jdB: 'xrán',
  jdJ: 'xruán',
  CdB: 'brán',
  DdB: 'mrán',
  UdB: 'dzrán',
  gdB: 'ŋrán',
  TdB: 'chrán',
  BdB: 'phrán',
  UdJ: 'dzruán',
  VeB: 'srén',
  jeB: 'xrén',
  deB: 'krén',
  TeB: 'chrén',
  UeB: 'dzrén',
  geB: 'ŋrén',
  SeB: 'crén',
  TeJ: 'chruén',
  eeB: 'khrén',
  QgB: 'sén',
  FgB: 'thén',
  EgB: 'tén',
  hgB: 'qén',
  GgB: 'dén',
  dgB: 'kén',
  jgB: 'xén',
  igB: 'qhén',
  DgB: 'mén',
  HgB: 'nén',
  AgB: 'pén',
  jgJ: 'xuén',
  dgJ: 'kuén',
  CgB: 'bén',
  egJ: 'khuén',
  egB: 'khén',
  ggB: 'ŋén',
  QfB: 'sién',
  lfB: 'jién',
  PfB: 'dzién',
  JfB: 'trién',
  XfB: 'tién',
  MfB: 'nrién',
  OfB: 'chién',
  YfB: 'thién',
  efB: 'khién',
  dfF: 'krién',
  ZfB: 'dién',
  NfB: 'cién',
  cfB: 'nién',
  RfB: 'zién',
  IfB: 'lién',
  gfF: 'ŋrién',
  ffF: 'grién',
  CfF: 'bryén',
  DfB: 'mién',
  AfB: 'pién',
  NfJ: 'cyén',
  PfJ: 'dzyén',
  AfF: 'pryén',
  lfJ: 'jyén',
  IfJ: 'lyén',
  JfJ: 'tryén',
  dfN: 'kryén',
  ffN: 'gryén',
  cfJ: 'nyén',
  YfJ: 'thyén',
  ZfJ: 'dyén',
  LfJ: 'dryén',
  XfJ: 'tyén',
  QfJ: 'syén',
  UfJ: 'dzryén',
  ffJ: 'gyén',
  ifJ: 'qhyén',
  CfB: 'bién',
  DfF: 'mryén',
  KfB: 'thrién',
  BfF: 'phryén',
  afB: 'shién',
  hfF: 'qrién',
  LfB: 'drién',
  UfB: 'dzrién',
  QhB: 'séu',
  dhB: 'kéu',
  EhB: 'téu',
  IhB: 'léu',
  FhB: 'théu',
  ihB: 'qhéu',
  hhB: 'qéu',
  HhB: 'néu',
  jhB: 'xéu',
  GhB: 'déu',
  ehB: 'khéu',
  NhB: 'céu',
  QiB: 'siéu',
  LiB: 'driéu',
  XiB: 'tiéu',
  hiF: 'qriéu',
  KiB: 'thriéu',
  aiB: 'shiéu',
  ciB: 'niéu',
  CiB: 'biéu',
  YiB: 'thiéu',
  BiB: 'phiéu',
  DiB: 'miéu',
  ZiB: 'diéu',
  diF: 'kriéu',
  AiF: 'priéu',
  AiB: 'piéu',
  CiF: 'briéu',
  liB: 'jiéu',
  OiB: 'chiéu',
  NiB: 'ciéu',
  fiF: 'griéu',
  IiB: 'liéu',
  BiF: 'phriéu',
  hiB: 'qiéu',
  ejB: 'khráu',
  jjB: 'xráu',
  AjB: 'práu',
  MjB: 'nráu',
  DjB: 'mráu',
  djB: 'kráu',
  SjB: 'cráu',
  hjB: 'qráu',
  CjB: 'bráu',
  gjB: 'ŋráu',
  UjB: 'dzráu',
  TjB: 'chráu',
  JjB: 'tráu',
  VjB: 'sráu',
  jkB: 'xáu',
  CkB: 'báu',
  IkB: 'láu',
  FkB: 'tháu',
  GkB: 'dáu',
  HkB: 'náu',
  QkB: 'sáu',
  EkB: 'táu',
  OkB: 'cháu',
  NkB: 'cáu',
  PkB: 'dzáu',
  dkB: 'káu',
  ikB: 'qháu',
  DkB: 'máu',
  AkB: 'páu',
  hkB: 'qáu',
  ekB: 'kháu',
  gkB: 'ŋáu',
  dlB: 'ká',
  OlB: 'chá',
  ElB: 'tá',
  QlB: 'sá',
  GlB: 'dá',
  glB: 'ŋá',
  FlB: 'thá',
  IlB: 'lá',
  HlB: 'ná',
  jlB: 'xá',
  ilB: 'qhá',
  elB: 'khá',
  hlB: 'qá',
  NlB: 'cá',
  dlJ: 'kuá',
  ElJ: 'tuá',
  QlJ: 'suá',
  GlJ: 'duá',
  FlJ: 'thuá',
  DlB: 'má',
  PlJ: 'dzuá',
  glJ: 'ŋuá',
  IlJ: 'luá',
  hlJ: 'quá',
  HlJ: 'nuá',
  AlB: 'pá',
  BlB: 'phá',
  jlJ: 'xuá',
  ilJ: 'qhuá',
  elJ: 'khuá',
  ClB: 'bá',
  OlJ: 'chuá',
  DnB: 'mrá',
  XoB: 'tiá',
  loB: 'jiá',
  gnB: 'ŋrá',
  dnB: 'krá',
  VnB: 'srá',
  hnB: 'qrá',
  RoB: 'ziá',
  jnB: 'xrá',
  QoB: 'siá',
  DoB: 'miá',
  OoB: 'chiá',
  inB: 'qhrá',
  ZoB: 'diá',
  enB: 'khrá',
  CnB: 'brá',
  aoB: 'shiá',
  NoB: 'ciá',
  AnB: 'prá',
  jnJ: 'xruá',
  dnJ: 'kruá',
  gnJ: 'ŋruá',
  coB: 'niá',
  SnB: 'crá',
  JnB: 'trá',
  UnB: 'dzrá',
  YoB: 'thiá',
  MnB: 'nrá',
  enJ: 'khruá',
  TnJ: 'chruá',
  KnJ: 'thruá',
  VnJ: 'sruá',
  KnB: 'thrá',
  InB: 'lrá',
  lpB: 'jiáŋ',
  RpB: 'ziáŋ',
  NpB: 'ciáŋ',
  IpB: 'liáŋ',
  hpB: 'qiáŋ',
  fpB: 'giáŋ',
  gpB: 'ŋiáŋ',
  TpB: 'chriáŋ',
  QpB: 'siáŋ',
  XpB: 'tiáŋ',
  VpB: 'sriáŋ',
  ipB: 'qhiáŋ',
  YpB: 'thiáŋ',
  dpB: 'kiáŋ',
  LpB: 'driáŋ',
  KpB: 'thriáŋ',
  dpJ: 'kyáŋ',
  cpB: 'niáŋ',
  apB: 'shiáŋ',
  BpB: 'phyáŋ',
  DpB: 'myáŋ',
  ApB: 'pyáŋ',
  hpJ: 'qyáŋ',
  kpJ: 'xyáŋ',
  ipJ: 'qhyáŋ',
  OpB: 'chiáŋ',
  JpB: 'triáŋ',
  ZpB: 'diáŋ',
  fpJ: 'gyáŋ',
  CpB: 'byáŋ',
  GqB: 'dáŋ',
  QqB: 'sáŋ',
  dqJ: 'kuáŋ',
  AqB: 'páŋ',
  NqB: 'cáŋ',
  HqB: 'náŋ',
  jqB: 'xáŋ',
  FqB: 'tháŋ',
  DqB: 'máŋ',
  EqB: 'táŋ',
  IqB: 'láŋ',
  hqB: 'qáŋ',
  eqB: 'kháŋ',
  hqJ: 'quáŋ',
  jqJ: 'xuáŋ',
  BqB: 'pháŋ',
  iqJ: 'qhuáŋ',
  dqB: 'káŋ',
  gqB: 'ŋáŋ',
  PqB: 'dzáŋ',
  OqB: 'cháŋ',
  iqB: 'qháŋ',
  eqJ: 'khuáŋ',
  drB: 'kráŋ',
  AsB: 'priéŋ',
  dsB: 'kriéŋ',
  hsB: 'qriéŋ',
  VsB: 'sriéŋ',
  ksJ: 'xryéŋ',
  isJ: 'qhryéŋ',
  DsB: 'mriéŋ',
  dsJ: 'kryéŋ',
  jrB: 'xráŋ',
  DrB: 'mráŋ',
  drJ: 'kruáŋ',
  ArB: 'práŋ',
  JrB: 'tráŋ',
  LrB: 'dráŋ',
  hrJ: 'qruáŋ',
  IrB: 'lráŋ',
  jrJ: 'xruáŋ',
  CrB: 'bráŋ',
  erJ: 'khruáŋ',
  MrB: 'nráŋ',
  dtB: 'kréŋ',
  DtB: 'mréŋ',
  jtB: 'xréŋ',
  CtB: 'bréŋ',
  BtB: 'phréŋ',
  PuB: 'dziéŋ',
  XuB: 'tiéŋ',
  KuB: 'thriéŋ',
  luB: 'jiéŋ',
  fuB: 'giéŋ',
  luJ: 'jyéŋ',
  IuB: 'liéŋ',
  duB: 'kiéŋ',
  AuB: 'piéŋ',
  euJ: 'khyéŋ',
  NuB: 'ciéŋ',
  huB: 'qiéŋ',
  OuB: 'chiéŋ',
  QuB: 'siéŋ',
  DuB: 'miéŋ',
  jvJ: 'xuéŋ',
  dvJ: 'kuéŋ',
  DvB: 'méŋ',
  EvB: 'téŋ',
  GvB: 'déŋ',
  FvB: 'théŋ',
  PvB: 'dzéŋ',
  hvJ: 'quéŋ',
  evB: 'khéŋ',
  HvB: 'néŋ',
  jvB: 'xéŋ',
  QvB: 'séŋ',
  AvB: 'péŋ',
  evJ: 'khuéŋ',
  dvB: 'kéŋ',
  BvB: 'phéŋ',
  hvB: 'qéŋ',
  IvB: 'léŋ',
  CvB: 'béŋ',
  ivJ: 'qhuéŋ',
  gvB: 'ŋéŋ',
  XwB: 'tiə́ŋ',
  KwB: 'thriə́ŋ',
  fwB: 'giə́ŋ',
  VwB: 'sriə́ŋ',
  ExB: 'tə́ŋ',
  BxB: 'phə́ŋ',
  exB: 'khə́ŋ',
  HxB: 'nə́ŋ',
  kyB: 'xyú',
  IyB: 'lyú',
  MyB: 'nryú',
  KyB: 'thryú',
  JyB: 'tryú',
  iyB: 'qhyú',
  dyB: 'kyú',
  ayB: 'shyú',
  YyB: 'thyú',
  PyB: 'dzyú',
  CyB: 'byú',
  AyB: 'pyú',
  eyB: 'khyú',
  cyB: 'nyú',
  fyB: 'gyú',
  LyB: 'dryú',
  lyB: 'jyú',
  ZyB: 'dyú',
  QyB: 'syú',
  hyB: 'qyú',
  NyB: 'cyú',
  VyB: 'sryú',
  XyB: 'tyú',
  ByB: 'phyú',
  SyB: 'cryú',
  UyB: 'dzryú',
  TyB: 'chryú',
  jzB: 'xú',
  DzB: 'mú',
  CzB: 'bú',
  EzB: 'tú',
  FzB: 'thú',
  dzB: 'kú',
  gzB: 'ŋú',
  AzB: 'pú',
  HzB: 'nú',
  QzB: 'sú',
  izB: 'qhú',
  BzB: 'phú',
  hzB: 'qú',
  IzB: 'lú',
  NzB: 'cú',
  ezB: 'khú',
  GzB: 'dú',
  OzB: 'chú',
  h0B: 'qríu',
  d0B: 'kíu',
  f0B: 'gríu',
  O1B: 'chím',
  L1B: 'drím',
  e1F: 'khrím',
  I1B: 'lím',
  Q1B: 'sím',
  K1B: 'thrím',
  N1B: 'cím',
  c1B: 'ním',
  X1B: 'tím',
  a1B: 'shím',
  b1B: 'zhím',
  Z1B: 'dím',
  Y1B: 'thím',
  T1B: 'chrím',
  M1B: 'nrím',
  f1F: 'grím',
  d1F: 'krím',
  P1B: 'dzím',
  g1F: 'ŋrím',
  V1B: 'srím',
  A1F: 'prím',
  h1F: 'qrím',
  B1F: 'phrím',
  i1F: 'qhrím',
  J1B: 'trím',
  l1B: 'jím',
  U1B: 'dzrím',
  d2B: 'kə́m',
  G2B: 'də́m',
  h2B: 'qə́m',
  H2B: 'nə́m',
  F2B: 'thə́m',
  P2B: 'dzə́m',
  O2B: 'chə́m',
  g2B: 'ŋə́m',
  N2B: 'cə́m',
  Q2B: 'sə́m',
  e2B: 'khə́m',
  j2B: 'xə́m',
  I2B: 'lə́m',
  E2B: 'tə́m',
  i2B: 'qhə́m',
  d3B: 'kám',
  a3B: 'shьʼám',
  I3B: 'lám',
  F3B: 'thám',
  E3B: 'tám',
  G3B: 'dám',
  O3B: 'chám',
  D3B: 'mám',
  N3B: 'cám',
  P3B: 'dzám',
  i3B: 'qhám',
  h3B: 'qám',
  e3B: 'khám',
  l4B: 'jiém',
  I4B: 'liém',
  i4F: 'qhriém',
  A4F: 'priém',
  X4B: 'tiém',
  e4F: 'khriém',
  g4F: 'ŋriém',
  f4F: 'griém',
  d4F: 'kriém',
  h4B: 'qiém',
  c4B: 'niém',
  a4B: 'shiém',
  K4B: 'thriém',
  h4F: 'qriém',
  P4B: 'dziém',
  O4B: 'chiém',
  N4B: 'ciém',
  e4B: 'khiém',
  Z4B: 'diém',
  F5B: 'thém',
  H5B: 'ném',
  E5B: 'tém',
  G5B: 'dém',
  e5B: 'khém',
  I5B: 'lém',
  j5B: 'xém',
  O5B: 'chém',
  d5B: 'kém',
  D5B: 'mém',
  g8B: 'ŋiám',
  e8B: 'khiám',
  h8B: 'qiám',
  j6B: 'xrém',
  L6B: 'drém',
  e6B: 'khrém',
  d6B: 'krém',
  U6B: 'dzrém',
  I6B: 'lrém',
  S6B: 'crém',
  T6B: 'chrém',
  i6B: 'qhrém',
  V6B: 'srém',
  h6B: 'qrém',
  M6B: 'nrém',
  K6B: 'thrém',
  j7B: 'xrám',
  e7B: 'khrám',
  V7B: 'srám',
  h7B: 'qrám',
  T7B: 'chrám',
  i7B: 'qhrám',
  U7B: 'dzrám',
  C9B: 'byóm',
  D9B: 'myóm',
  A9B: 'pyóm',
  B9B: 'phyóm',
  QAC: 'sûŋ',
  CBC: 'byûŋ',
  dAC: 'kûŋ',
  IAC: 'lûŋ',
  EAC: 'tûŋ',
  eAC: 'khûŋ',
  NAC: 'cûŋ',
  hAC: 'qûŋ',
  OAC: 'chûŋ',
  GAC: 'dûŋ',
  FAC: 'thûŋ',
  LBC: 'dryûŋ',
  ABC: 'pyûŋ',
  eBC: 'khyûŋ',
  DBC: 'myûŋ',
  iBC: 'qhyûŋ',
  DAC: 'mûŋ',
  OBC: 'chyûŋ',
  BBC: 'phyûŋ',
  PAC: 'dzûŋ',
  JBC: 'tryûŋ',
  jAC: 'xûŋ',
  HAC: 'nûŋ',
  XBC: 'tyûŋ',
  YBC: 'thyûŋ',
  UBC: 'dzryûŋ',
  iAC: 'qhûŋ',
  QCC: 'sôŋ',
  NCC: 'côŋ',
  FCC: 'thôŋ',
  DCC: 'môŋ',
  jCC: 'xôŋ',
  lDC: 'jyôŋ',
  RDC: 'zyôŋ',
  CDC: 'byôŋ',
  fDC: 'gyôŋ',
  ADC: 'pyôŋ',
  dDC: 'kyôŋ',
  hDC: 'qyôŋ',
  JDC: 'tryôŋ',
  NDC: 'cyôŋ',
  KDC: 'thryôŋ',
  cDC: 'nyôŋ',
  XDC: 'tyôŋ',
  LDC: 'dryôŋ',
  IDC: 'lyôŋ',
  eDC: 'khyôŋ',
  PDC: 'dzyôŋ',
  MDC: 'nryôŋ',
  dEC: 'krôŋ',
  jEC: 'xrôŋ',
  JEC: 'trôŋ',
  LEC: 'drôŋ',
  KEC: 'thrôŋ',
  UEC: 'dzrôŋ',
  BEC: 'phrôŋ',
  TEC: 'chrôŋ',
  VEC: 'srôŋ',
  XFC: 'tiê',
  CFC: 'biê',
  XFK: 'tyê',
  IFC: 'liê',
  ZFC: 'diê',
  NFC: 'ciê',
  QFC: 'siê',
  kFK: 'xryê',
  dFO: 'kryê',
  BFG: 'phriê',
  AFG: 'priê',
  CFG: 'briê',
  IFK: 'lyê',
  dFG: 'kriê',
  AFC: 'piê',
  fFG: 'griê',
  OFC: 'chiê',
  lFC: 'jiê',
  gFG: 'ŋriê',
  BFC: 'phiê',
  PFC: 'dziê',
  JFC: 'triê',
  hFG: 'qriê',
  LFK: 'dryê',
  YFK: 'thyê',
  iFG: 'qhriê',
  eFC: 'khiê',
  hFC: 'qiê',
  aFC: 'shiê',
  VFC: 'sriê',
  eFK: 'khyê',
  hFO: 'qryê',
  gFO: 'ŋryê',
  iFO: 'qhryê',
  hFK: 'qyê',
  ZFK: 'dyê',
  dFC: 'kiê',
  cFK: 'nyê',
  YFC: 'thiê',
  QFK: 'syê',
  JFK: 'tryê',
  lFK: 'jyê',
  MFK: 'nryê',
  dFK: 'kyê',
  SFC: 'criê',
  iFK: 'qhyê',
  eFG: 'khriê',
  XGC: 'tî',
  kGK: 'xrŷi',
  DGG: 'mrŷi',
  RGK: 'zŷi',
  NGK: 'cŷi',
  QGK: 'sŷi',
  IGK: 'lŷi',
  AGG: 'prŷi',
  fGO: 'grŷi',
  BGG: 'phrŷi',
  CGG: 'brŷi',
  dGO: 'krŷi',
  VGK: 'srŷi',
  eGO: 'khrŷi',
  iGO: 'qhrŷi',
  ZGC: 'dî',
  IGC: 'lî',
  MGC: 'nrî',
  BGC: 'phî',
  gGG: 'ŋrî',
  JGC: 'trî',
  eGC: 'khî',
  LGC: 'drî',
  DGC: 'mî',
  KGC: 'thrî',
  dGG: 'krî',
  fGG: 'grî',
  fGK: 'gŷi',
  OGK: 'chŷi',
  cGC: 'nî',
  NGC: 'cî',
  OGC: 'chî',
  hGG: 'qrî',
  QGC: 'sî',
  eGG: 'khrî',
  dGK: 'kŷi',
  CGC: 'bî',
  iGK: 'qhŷi',
  AGC: 'pî',
  PGK: 'dzŷi',
  GGC: 'dʼî',
  iGG: 'qhrî',
  lGC: 'jî',
  bGC: 'zhî',
  PGC: 'dzî',
  LGK: 'drŷi',
  YGK: 'thŷi',
  lGK: 'jŷi',
  JGK: 'trŷi',
  YGC: 'thî',
  aGC: 'shî',
  TGK: 'chrŷi',
  aGK: 'shŷi',
  XHC: 'tiə̂',
  LHC: 'driə̂',
  RHC: 'ziə̂',
  OHC: 'chiə̂',
  QHC: 'siə̂',
  aHC: 'shiə̂',
  SHC: 'criə̂',
  IHC: 'liə̂',
  PHC: 'dziə̂',
  KHC: 'thriə̂',
  cHC: 'niə̂',
  VHC: 'sriə̂',
  THC: 'chriə̂',
  lHC: 'jiə̂',
  JHC: 'triə̂',
  ZHC: 'diə̂',
  UHC: 'dzriə̂',
  fHC: 'giə̂',
  YHC: 'thiə̂',
  hHC: 'qiə̂',
  dHC: 'kiə̂',
  iHC: 'qhiə̂',
  gHC: 'ŋiə̂',
  eHC: 'khiə̂',
  DIC: 'myûi',
  dIK: 'kyûi',
  kIK: 'xyûi',
  gIK: 'ŋyûi',
  AIC: 'pyûi',
  BIC: 'phyûi',
  eIK: 'khyûi',
  hIK: 'qyûi',
  iIK: 'qhyûi',
  CIC: 'byûi',
  dIC: 'kiə̂i',
  gIC: 'ŋiə̂i',
  eIC: 'khiə̂i',
  iIC: 'qhiə̂i',
  fIC: 'giə̂i',
  hIC: 'qiə̂i',
  gJC: 'ŋiô',
  IJC: 'liô',
  dJC: 'kiô',
  OJC: 'chiô',
  eJC: 'khiô',
  ZJC: 'diô',
  aJC: 'shiô',
  JJC: 'triô',
  XJC: 'tiô',
  VJC: 'sriô',
  hJC: 'qiô',
  LJC: 'driô',
  fJC: 'giô',
  QJC: 'siô',
  UJC: 'dzriô',
  NJC: 'ciô',
  SJC: 'criô',
  cJC: 'niô',
  lJC: 'jiô',
  iJC: 'qhiô',
  MJC: 'nriô',
  TJC: 'chriô',
  YJC: 'thiô',
  KJC: 'thriô',
  RJC: 'ziô',
  gKC: 'ŋyô',
  hKC: 'qyô',
  ZKC: 'dyô',
  LKC: 'dryô',
  CKC: 'byô',
  XKC: 'tyô',
  dKC: 'kyô',
  iKC: 'qhyô',
  aKC: 'shyô',
  lKC: 'jyô',
  cKC: 'nyô',
  BKC: 'phyô',
  DKC: 'myô',
  NKC: 'cyô',
  fKC: 'gyô',
  kKC: 'xyô',
  PKC: 'dzyô',
  VKC: 'sryô',
  AKC: 'pyô',
  OKC: 'chyô',
  JKC: 'tryô',
  eKC: 'khyô',
  TKC: 'chryô',
  KKC: 'thryô',
  QKC: 'syô',
  IKC: 'lyô',
  DLC: 'mô',
  GLC: 'dô',
  ILC: 'lô',
  ELC: 'tô',
  FLC: 'thô',
  dLC: 'kô',
  gLC: 'ŋô',
  jLC: 'xô',
  QLC: 'sô',
  PLC: 'dzô',
  HLC: 'nô',
  ALC: 'pô',
  hLC: 'qô',
  BLC: 'phô',
  OLC: 'chô',
  eLC: 'khô',
  CLC: 'bô',
  iLC: 'qhô',
  NLC: 'cô',
  NMC: 'cêi',
  EMC: 'têi',
  PMC: 'dzêi',
  FMC: 'thêi',
  GMC: 'dêi',
  OMC: 'chêi',
  QMC: 'sêi',
  gMC: 'ŋêi',
  dMC: 'kêi',
  jMC: 'xêi',
  eMC: 'khêi',
  hMC: 'qêi',
  DMC: 'mêi',
  AMC: 'pêi',
  jMK: 'xuêi',
  dMK: 'kuêi',
  iMK: 'qhuêi',
  BMC: 'phêi',
  CMC: 'bêi',
  IMC: 'lêi',
  HMC: 'nêi',
  iMC: 'qhêi',
  NNC: 'ciêi',
  QNK: 'syêi',
  kNK: 'xryêi',
  cNK: 'nyêi',
  XNK: 'tyêi',
  VNK: 'sryêi',
  ONK: 'chyêi',
  lNK: 'jyêi',
  JNK: 'tryêi',
  aNK: 'shyêi',
  CNC: 'biêi',
  TNK: 'chryêi',
  RNK: 'zyêi',
  ANC: 'piêi',
  dNO: 'kryêi',
  DNC: 'miêi',
  VNC: 'sriêi',
  YNC: 'thiêi',
  XNC: 'tiêi',
  ZNC: 'diêi',
  lNC: 'jiêi',
  hNG: 'qriêi',
  gNC: 'ŋiêi',
  LNC: 'driêi',
  INC: 'liêi',
  eNG: 'khriêi',
  aNC: 'shiêi',
  dNG: 'kriêi',
  JNC: 'triêi',
  fNG: 'griêi',
  ZNK: 'dyêi',
  KNC: 'thriêi',
  LNK: 'dryêi',
  gNG: 'ŋriêi',
  NNK: 'cyêi',
  BNC: 'phiêi',
  eUK: 'khyôi',
  iUK: 'qhyôi',
  FOC: 'thâi',
  dOC: 'kâi',
  gOC: 'ŋâi',
  hOC: 'qâi',
  HOC: 'nâi',
  GOC: 'dâi',
  jOC: 'xâi',
  EOC: 'tâi',
  AOC: 'pâi',
  BOC: 'phâi',
  jOK: 'xuâi',
  GOK: 'duâi',
  dOK: 'kuâi',
  NOK: 'cuâi',
  iOK: 'qhuâi',
  eOK: 'khuâi',
  IOK: 'luâi',
  gOK: 'ŋuâi',
  EOK: 'tuâi',
  hOK: 'quâi',
  POK: 'dzuâi',
  QOK: 'suâi',
  OOK: 'chuâi',
  COC: 'bâi',
  eOC: 'khâi',
  OOC: 'châi',
  IOC: 'lâi',
  iOC: 'qhâi',
  FOK: 'thuâi',
  DOC: 'mâi',
  dPK: 'kruê',
  dPC: 'krê',
  hPC: 'qrê',
  jPC: 'xrê',
  DPC: 'mrê',
  jPK: 'xruê',
  TPC: 'chrê',
  gPC: 'ŋrê',
  iPC: 'qhrê',
  APC: 'prê',
  CPC: 'brê',
  UPC: 'dzrê',
  BPC: 'phrê',
  SPC: 'crê',
  ePC: 'khrê',
  VPC: 'srê',
  iPK: 'qhruê',
  dQK: 'kruêi',
  hQC: 'qrêi',
  SQC: 'crêi',
  dQC: 'krêi',
  MQC: 'nrêi',
  iQC: 'qhrêi',
  jQC: 'xrêi',
  gQC: 'ŋrêi',
  eQK: 'khruêi',
  AQC: 'prêi',
  BQC: 'phrêi',
  jQK: 'xruêi',
  gQK: 'ŋruêi',
  CQC: 'brêi',
  DQC: 'mrêi',
  iQK: 'qhruêi',
  VQC: 'srêi',
  eQC: 'khrêi',
  dRK: 'kruâi',
  eRK: 'khruâi',
  DRC: 'mrâi',
  jRK: 'xruâi',
  CRC: 'brâi',
  hRK: 'qruâi',
  TRK: 'chruâi',
  dRC: 'krâi',
  KRC: 'thrâi',
  hRC: 'qrâi',
  VRC: 'srâi',
  LRC: 'drâi',
  iRC: 'qhrâi',
  iRK: 'qhruâi',
  ARC: 'prâi',
  URC: 'dzrâi',
  jRC: 'xrâi',
  GSK: 'dûi',
  CSC: 'bûi',
  DSC: 'mûi',
  BSC: 'phûi',
  iSK: 'qhûi',
  ESK: 'tûi',
  OSK: 'chûi',
  NSK: 'cûi',
  hSK: 'qûi',
  FSK: 'thûi',
  dSK: 'kûi',
  jSK: 'xûi',
  eSK: 'khûi',
  QSK: 'sûi',
  HSK: 'nûi',
  ISK: 'lûi',
  ASC: 'pûi',
  gSK: 'ŋûi',
  GTC: 'də̂i',
  NTC: 'cə̂i',
  QTC: 'sə̂i',
  FTC: 'thə̂i',
  dTC: 'kə̂i',
  eTC: 'khə̂i',
  gTC: 'ŋə̂i',
  hTC: 'qə̂i',
  jTC: 'xə̂i',
  HTC: 'nə̂i',
  ETC: 'tə̂i',
  ITC: 'lə̂i',
  OTC: 'chə̂i',
  PTC: 'dzə̂i',
  iTC: 'qhə̂i',
  AUC: 'pyôi',
  BUC: 'phyôi',
  hUK: 'qyôi',
  CUC: 'byôi',
  fUK: 'gyôi',
  gUC: 'ŋiâi',
  XVC: 'tîn',
  QVC: 'sîn',
  cVC: 'nîn',
  lVC: 'jîn',
  IVC: 'lîn',
  AVC: 'pîn',
  LVC: 'drîn',
  ZVC: 'dîn',
  aVC: 'shîn',
  eVC: 'khrîn',
  RVC: 'zîn',
  gVG: 'ŋrîn',
  NVC: 'cîn',
  iVG: 'qhrîn',
  JVC: 'trîn',
  fVG: 'grîn',
  TVC: 'chrîn',
  hVC: 'qîn',
  KVC: 'thrîn',
  BVC: 'phîn',
  OVC: 'chîn',
  dVK: 'kŷn',
  XVK: 'tŷn',
  QVK: 'sŷn',
  RVK: 'zŷn',
  NVK: 'cŷn',
  aVK: 'shŷn',
  cVK: 'nŷn',
  bVK: 'zhŷn',
  DXC: 'myûn',
  kXK: 'xyûn',
  iXK: 'qhyûn',
  BXC: 'phyûn',
  AXC: 'pyûn',
  hXK: 'qyûn',
  dXK: 'kyûn',
  fXK: 'gyûn',
  CXC: 'byûn',
  iYC: 'qhiə̂n',
  dYC: 'kiə̂n',
  fYC: 'giə̂n',
  hYC: 'qiə̂n',
  gYC: 'ŋiə̂n',
  gZK: 'ŋyôn',
  hZK: 'qyôn',
  AZC: 'pyôn',
  eZK: 'khyôn',
  DZC: 'myôn',
  CZC: 'byôn',
  BZC: 'phyôn',
  dZC: 'kiân',
  hZC: 'qiân',
  iZC: 'qhiân',
  iZK: 'qhyôn',
  fZC: 'giân',
  kZK: 'xyôn',
  gZC: 'ŋiân',
  fZK: 'gyôn',
  dZK: 'kyôn',
  jaK: 'xûn',
  EaK: 'tûn',
  QaK: 'sûn',
  eaK: 'khûn',
  HaK: 'nûn',
  haK: 'qûn',
  DaC: 'mûn',
  PaK: 'dzûn',
  daK: 'kûn',
  BaC: 'phûn',
  GaK: 'dûn',
  OaK: 'chûn',
  CaC: 'bûn',
  gaK: 'ŋûn',
  IaK: 'lûn',
  AaC: 'pûn',
  iaK: 'qhûn',
  NaK: 'cûn',
  jbC: 'xə̂n',
  dbC: 'kə̂n',
  gbC: 'ŋə̂n',
  hbC: 'qə̂n',
  jcC: 'xân',
  FcC: 'thân',
  hcC: 'qân',
  EcC: 'tân',
  GcC: 'dân',
  dcC: 'kân',
  gcC: 'ŋân',
  ecC: 'khân',
  icC: 'qhân',
  IcC: 'lân',
  HcC: 'nân',
  OcC: 'chân',
  QcC: 'sân',
  NcC: 'cân',
  PcC: 'dzân',
  jcK: 'xuân',
  NcK: 'cuân',
  hcK: 'quân',
  dcK: 'kuân',
  OcK: 'chuân',
  gcK: 'ŋuân',
  GcK: 'duân',
  IcK: 'luân',
  EcK: 'tuân',
  FcK: 'thuân',
  icK: 'qhuân',
  QcK: 'suân',
  DcC: 'mân',
  AcC: 'pân',
  BcC: 'phân',
  CcC: 'bân',
  HcK: 'nuân',
  PcK: 'dzuân',
  ecK: 'khuân',
  ddC: 'krân',
  gdC: 'ŋrân',
  hdC: 'qrân',
  VdC: 'srân',
  jdC: 'xrân',
  DdC: 'mrân',
  hdK: 'qruân',
  jdK: 'xruân',
  ddK: 'kruân',
  VdK: 'sruân',
  TdK: 'chruân',
  gdK: 'ŋruân',
  UdC: 'dzrân',
  BdC: 'phrân',
  MdK: 'nruân',
  TdC: 'chrân',
  KdC: 'thrân',
  deC: 'krên',
  jeC: 'xrên',
  CeC: 'brên',
  BeC: 'phrên',
  jeK: 'xruên',
  DeC: 'mrên',
  AeC: 'prên',
  deK: 'kruên',
  QgC: 'sên',
  OgC: 'chên',
  igK: 'qhuên',
  jgK: 'xuên',
  dgK: 'kuên',
  GgC: 'dên',
  FgC: 'thên',
  IgC: 'lên',
  dgC: 'kên',
  HgC: 'nên',
  egC: 'khên',
  jgC: 'xên',
  ggC: 'ŋên',
  hgC: 'qên',
  NgC: 'cên',
  DgC: 'mên',
  BgC: 'phên',
  PgC: 'dzên',
  hgK: 'quên',
  EgC: 'tên',
  igC: 'qhên',
  QfC: 'siên',
  XfC: 'tiên',
  ZfC: 'diên',
  gfG: 'ŋriên',
  efC: 'khiên',
  dfK: 'kyên',
  kfK: 'xryên',
  DfC: 'miên',
  YfK: 'thyên',
  lfK: 'jyên',
  JfC: 'triên',
  cfK: 'nyên',
  NfC: 'ciên',
  YfC: 'thiên',
  afC: 'shiên',
  dfO: 'kryên',
  ffO: 'gryên',
  IfK: 'lyên',
  KfK: 'thryên',
  AfG: 'pryên',
  VfK: 'sryên',
  OfK: 'chyên',
  CfG: 'bryên',
  RfK: 'zyên',
  QfK: 'syên',
  UfK: 'dzryên',
  SfK: 'cryên',
  LfK: 'dryên',
  PfC: 'dziên',
  RfC: 'ziên',
  BfC: 'phiên',
  ZfK: 'dyên',
  MfC: 'nriên',
  JfK: 'tryên',
  lfC: 'jiên',
  CfC: 'biên',
  LfC: 'driên',
  IfC: 'liên',
  efO: 'khryên',
  XfK: 'tyên',
  AgC: 'pên',
  QhC: 'sêu',
  FhC: 'thêu',
  EhC: 'têu',
  dhC: 'kêu',
  HhC: 'nêu',
  GhC: 'dêu',
  ehC: 'khêu',
  IhC: 'lêu',
  ghC: 'ŋêu',
  hhC: 'qêu',
  ihC: 'qhêu',
  QiC: 'siêu',
  XiC: 'tiêu',
  liC: 'jiêu',
  hiC: 'qiêu',
  LiC: 'driêu',
  ZiC: 'diêu',
  fiG: 'griêu',
  BiC: 'phiêu',
  PiC: 'dziêu',
  DiC: 'miêu',
  OiC: 'chiêu',
  IiC: 'liêu',
  eiG: 'khriêu',
  giG: 'ŋriêu',
  NiC: 'ciêu',
  DiG: 'mriêu',
  CiC: 'biêu',
  aiC: 'shiêu',
  KiC: 'thriêu',
  AiG: 'priêu',
  fiC: 'giêu',
  ciC: 'niêu',
  jjC: 'xrâu',
  djC: 'krâu',
  ijC: 'qhrâu',
  JjC: 'trâu',
  AjC: 'prâu',
  ejC: 'khrâu',
  DjC: 'mrâu',
  BjC: 'phrâu',
  KjC: 'thrâu',
  VjC: 'srâu',
  LjC: 'drâu',
  MjC: 'nrâu',
  SjC: 'crâu',
  CjC: 'brâu',
  TjC: 'chrâu',
  hjC: 'qrâu',
  gjC: 'ŋrâu',
  UjC: 'dzrâu',
  jkC: 'xâu',
  GkC: 'dâu',
  EkC: 'tâu',
  dkC: 'kâu',
  gkC: 'ŋâu',
  DkC: 'mâu',
  IkC: 'lâu',
  OkC: 'châu',
  CkC: 'bâu',
  AkC: 'pâu',
  PkC: 'dzâu',
  hkC: 'qâu',
  QkC: 'sâu',
  ekC: 'khâu',
  NkC: 'câu',
  ikC: 'qhâu',
  HkC: 'nâu',
  dlC: 'kâ',
  jlC: 'xâ',
  NlC: 'câ',
  ElC: 'tâ',
  IlC: 'lâ',
  elC: 'khâ',
  glC: 'ŋâ',
  GlC: 'dâ',
  HlC: 'nâ',
  QlC: 'sâ',
  ilC: 'qhâ',
  FlC: 'thâ',
  dlK: 'kuâ',
  jlK: 'xuâ',
  NlK: 'cuâ',
  elK: 'khuâ',
  FlK: 'thuâ',
  AlC: 'pâ',
  OlK: 'chuâ',
  DlC: 'mâ',
  HlK: 'nuâ',
  BlC: 'phâ',
  PlK: 'dzuâ',
  glK: 'ŋuâ',
  ilK: 'qhuâ',
  GlK: 'duâ',
  ClC: 'bâ',
  IlK: 'luâ',
  ElK: 'tuâ',
  QlK: 'suâ',
  hlC: 'qâ',
  hlK: 'quâ',
  DnC: 'mrâ',
  dnC: 'krâ',
  hnC: 'qrâ',
  inC: 'qhrâ',
  gnC: 'ŋrâ',
  KnC: 'thrâ',
  JnC: 'trâ',
  SnC: 'crâ',
  UnC: 'dzrâ',
  RoC: 'ziâ',
  enC: 'khrâ',
  jnC: 'xrâ',
  PoC: 'dziâ',
  loC: 'jiâ',
  YoC: 'thiâ',
  QoC: 'siâ',
  XoC: 'tiâ',
  NoC: 'ciâ',
  aoC: 'shiâ',
  boC: 'zhiâ',
  AnC: 'prâ',
  BnC: 'phrâ',
  jnK: 'xruâ',
  inK: 'qhruâ',
  enK: 'khruâ',
  VnK: 'sruâ',
  CnC: 'brâ',
  MnC: 'nrâ',
  OoC: 'chiâ',
  VnC: 'srâ',
  dnK: 'kruâ',
  LnC: 'drâ',
  gnK: 'ŋruâ',
  hnK: 'qruâ',
  lpC: 'jiâŋ',
  IpC: 'liâŋ',
  UpC: 'dzriâŋ',
  cpC: 'niâŋ',
  apC: 'shiâŋ',
  JpC: 'triâŋ',
  KpC: 'thriâŋ',
  ipC: 'qhiâŋ',
  LpC: 'driâŋ',
  MpC: 'nriâŋ',
  PpC: 'dziâŋ',
  XpC: 'tiâŋ',
  ZpC: 'diâŋ',
  SpC: 'criâŋ',
  hpC: 'qiâŋ',
  fpC: 'giâŋ',
  YpC: 'thiâŋ',
  TpC: 'chriâŋ',
  NpC: 'ciâŋ',
  gpC: 'ŋiâŋ',
  BpC: 'phyâŋ',
  DpC: 'myâŋ',
  ipK: 'qhyâŋ',
  dpK: 'kyâŋ',
  kpK: 'xyâŋ',
  ApC: 'pyâŋ',
  QpC: 'siâŋ',
  dpC: 'kiâŋ',
  epC: 'khiâŋ',
  OpC: 'chiâŋ',
  fpK: 'gyâŋ',
  CpC: 'byâŋ',
  GqC: 'dâŋ',
  IqC: 'lâŋ',
  jqC: 'xâŋ',
  hqC: 'qâŋ',
  gqC: 'ŋâŋ',
  NqC: 'câŋ',
  CqC: 'bâŋ',
  PqC: 'dzâŋ',
  EqC: 'tâŋ',
  eqC: 'khâŋ',
  AqC: 'pâŋ',
  FqC: 'thâŋ',
  eqK: 'khuâŋ',
  HqC: 'nâŋ',
  QqC: 'sâŋ',
  jqK: 'xuâŋ',
  dqK: 'kuâŋ',
  dqC: 'kâŋ',
  iqK: 'qhuâŋ',
  DqC: 'mâŋ',
  hqK: 'quâŋ',
  hsC: 'qriêŋ',
  dsC: 'kriêŋ',
  fsC: 'griêŋ',
  esC: 'khriêŋ',
  drC: 'krâŋ',
  DsC: 'mriêŋ',
  CsC: 'briêŋ',
  DrC: 'mrâŋ',
  jrK: 'xruâŋ',
  AsC: 'priêŋ',
  ksK: 'xryêŋ',
  jrC: 'xrâŋ',
  hrC: 'qrâŋ',
  TsC: 'chriêŋ',
  JrC: 'trâŋ',
  KrC: 'thrâŋ',
  ArC: 'prâŋ',
  LrC: 'drâŋ',
  VsC: 'sriêŋ',
  gsC: 'ŋriêŋ',
  CrC: 'brâŋ',
  irC: 'qhrâŋ',
  hrK: 'qruâŋ',
  StC: 'crêŋ',
  AtC: 'prêŋ',
  CtC: 'brêŋ',
  htC: 'qrêŋ',
  gtC: 'ŋrêŋ',
  itK: 'qhruêŋ',
  duC: 'kiêŋ',
  OuC: 'chiêŋ',
  XuC: 'tiêŋ',
  auC: 'shiêŋ',
  LuC: 'driêŋ',
  KuC: 'thriêŋ',
  QuC: 'siêŋ',
  IuC: 'liêŋ',
  BuC: 'phiêŋ',
  iuK: 'qhyêŋ',
  AuC: 'piêŋ',
  CuC: 'biêŋ',
  PuC: 'dziêŋ',
  ZuC: 'diêŋ',
  DuC: 'miêŋ',
  euC: 'khiêŋ',
  iuC: 'qhiêŋ',
  NuC: 'ciêŋ',
  dvC: 'kêŋ',
  HvC: 'nêŋ',
  QvC: 'sêŋ',
  jvC: 'xêŋ',
  GvC: 'dêŋ',
  EvC: 'têŋ',
  evC: 'khêŋ',
  FvC: 'thêŋ',
  OvC: 'chêŋ',
  DvC: 'mêŋ',
  IvC: 'lêŋ',
  XwC: 'tiə̂ŋ',
  lwC: 'jiə̂ŋ',
  bwC: 'zhiə̂ŋ',
  cwC: 'niə̂ŋ',
  hwC: 'qiə̂ŋ',
  NwC: 'ciə̂ŋ',
  iwC: 'qhiə̂ŋ',
  awC: 'shiə̂ŋ',
  LwC: 'driə̂ŋ',
  CwC: 'bryə̂ŋ',
  YwC: 'thiə̂ŋ',
  gwC: 'ŋiə̂ŋ',
  ZwC: 'diə̂ŋ',
  KwC: 'thriə̂ŋ',
  fwC: 'giə̂ŋ',
  ExC: 'tə̂ŋ',
  PxC: 'dzə̂ŋ',
  dxC: 'kə̂ŋ',
  OxC: 'chə̂ŋ',
  GxC: 'də̂ŋ',
  DxC: 'mə̂ŋ',
  AxC: 'pə̂ŋ',
  CxC: 'bə̂ŋ',
  IxC: 'lə̂ŋ',
  NxC: 'cə̂ŋ',
  QxC: 'sə̂ŋ',
  FxC: 'thə̂ŋ',
  kyC: 'xyû',
  dyC: 'kyû',
  LyC: 'dryû',
  JyC: 'tryû',
  ayC: 'shyû',
  YyC: 'thyû',
  RyC: 'zyû',
  iyC: 'qhyû',
  XyC: 'tyû',
  fyC: 'gyû',
  VyC: 'sryû',
  SyC: 'cryû',
  ByC: 'phyû',
  TyC: 'chryû',
  AyC: 'pyû',
  KyC: 'thryû',
  IyC: 'lyû',
  QyC: 'syû',
  NyC: 'cyû',
  UyC: 'dzryû',
  PyC: 'dzyû',
  MyC: 'nryû',
  CyC: 'byû',
  lyC: 'jyû',
  ZyC: 'dyû',
  cyC: 'nyû',
  DyC: 'myû',
  gyC: 'ŋyû',
  eyC: 'khyû',
  OyC: 'chyû',
  jzC: 'xû',
  ezC: 'khû',
  DzC: 'mû',
  BzC: 'phû',
  GzC: 'dû',
  EzC: 'tû',
  HzC: 'nû',
  QzC: 'sû',
  NzC: 'cû',
  FzC: 'thû',
  hzC: 'qû',
  dzC: 'kû',
  OzC: 'chû',
  IzC: 'lû',
  izC: 'qhû',
  CzC: 'bû',
  gzC: 'ŋû',
  PzC: 'dzû',
  h0C: 'qrîu',
  D0C: 'mrîu',
  e0C: 'khrîu',
  f0C: 'grîu',
  O1C: 'chîm',
  N1C: 'cîm',
  c1C: 'nîm',
  L1C: 'drîm',
  X1C: 'tîm',
  f1G: 'grîm',
  d1G: 'krîm',
  M1C: 'nrîm',
  h1G: 'qrîm',
  V1C: 'srîm',
  K1C: 'thrîm',
  S1C: 'crîm',
  T1C: 'chrîm',
  g1G: 'ŋrîm',
  J1C: 'trîm',
  I1C: 'lîm',
  Z1C: 'dîm',
  k1C: 'xrîm',
  a1C: 'shîm',
  e2C: 'khə̂m',
  d2C: 'kə̂m',
  j2C: 'xə̂m',
  h2C: 'qə̂m',
  F2C: 'thə̂m',
  Q2C: 'sə̂m',
  O2C: 'chə̂m',
  G2C: 'də̂m',
  g2C: 'ŋə̂m',
  E2C: 'tə̂m',
  H2C: 'nə̂m',
  I2C: 'lə̂m',
  i2C: 'qhə̂m',
  N2C: 'cə̂m',
  e3C: 'khâm',
  I3C: 'lâm',
  F3C: 'thâm',
  d3C: 'kâm',
  i3C: 'qhâm',
  j3C: 'xâm',
  G3C: 'dâm',
  P3C: 'dzâm',
  E3C: 'tâm',
  Q3C: 'sâm',
  l4C: 'jiêm',
  Z4C: 'diêm',
  c4C: 'niêm',
  h4C: 'qiêm',
  A4G: 'priêm',
  g4G: 'ŋriêm',
  a4C: 'shiêm',
  N4C: 'ciêm',
  O4C: 'chiêm',
  I4C: 'liêm',
  K4C: 'thriêm',
  Y4C: 'thiêm',
  h4G: 'qriêm',
  P4C: 'dziêm',
  X4C: 'tiêm',
  F5C: 'thêm',
  H5C: 'nêm',
  E5C: 'têm',
  Q5C: 'sêm',
  G5C: 'dêm',
  d5C: 'kêm',
  h5C: 'qêm',
  N5C: 'cêm',
  P5C: 'dzêm',
  e5C: 'khêm',
  I5C: 'lêm',
  g8C: 'ŋiâm',
  i8C: 'qhiâm',
  e8C: 'khiâm',
  D9C: 'myôm',
  j6C: 'xrêm',
  h6C: 'qrêm',
  S6C: 'crêm',
  J6C: 'trêm',
  e6C: 'khrêm',
  L6C: 'drêm',
  U6C: 'dzrêm',
  d6C: 'krêm',
  M6C: 'nrêm',
  g6C: 'ŋrêm',
  d7C: 'krâm',
  T7C: 'chrâm',
  S7C: 'crâm',
  V7C: 'srâm',
  i7C: 'qhrâm',
  C7C: 'brâm',
  j7C: 'xrâm',
  U7C: 'dzrâm',
  h7C: 'qrâm',
  C9C: 'byôm',
  B9C: 'phyôm',
  d8C: 'kiâm',
  h8C: 'qiâm',
  hAD: 'quk',
  GAD: 'duk',
  dAD: 'kuk',
  jAD: 'xuk',
  eAD: 'khuk',
  FAD: 'thuk',
  EAD: 'tuk',
  QAD: 'suk',
  IAD: 'luk',
  iAD: 'qhuk',
  PAD: 'dzuk',
  OAD: 'chuk',
  NAD: 'cuk',
  CAD: 'buk',
  BAD: 'phuk',
  AAD: 'puk',
  DAD: 'muk',
  ABD: 'pyuk',
  CBD: 'byuk',
  VBD: 'sryuk',
  IBD: 'lyuk',
  LBD: 'dryuk',
  dBD: 'kyuk',
  eBD: 'khyuk',
  ZBD: 'dyuk',
  YBD: 'thyuk',
  lBD: 'jyuk',
  fBD: 'gyuk',
  OBD: 'chyuk',
  cBD: 'nyuk',
  XBD: 'tyuk',
  aBD: 'shyuk',
  iBD: 'qhyuk',
  JBD: 'tryuk',
  NBD: 'cyuk',
  TBD: 'chryuk',
  MBD: 'nryuk',
  SBD: 'cryuk',
  BBD: 'phyuk',
  hBD: 'qyuk',
  QBD: 'syuk',
  DBD: 'myuk',
  kBD: 'xyuk',
  KBD: 'thryuk',
  gBD: 'ŋyuk',
  PBD: 'dzyuk',
  hCD: 'qok',
  GCD: 'dok',
  ECD: 'tok',
  eCD: 'khok',
  jCD: 'xok',
  CCD: 'bok',
  QCD: 'sok',
  dCD: 'kok',
  DCD: 'mok',
  iCD: 'qhok',
  HCD: 'nok',
  NCD: 'cok',
  ACD: 'pok',
  ICD: 'lok',
  gCD: 'ŋok',
  XDD: 'tyok',
  gDD: 'ŋyok',
  iDD: 'qhyok',
  dDD: 'kyok',
  fDD: 'gyok',
  ZDD: 'dyok',
  YDD: 'thyok',
  cDD: 'nyok',
  aDD: 'shyok',
  lDD: 'jyok',
  LDD: 'dryok',
  IDD: 'lyok',
  eDD: 'khyok',
  JDD: 'tryok',
  NDD: 'cyok',
  bDD: 'zhyok',
  CDD: 'byok',
  ODD: 'chyok',
  RDD: 'zyok',
  QDD: 'syok',
  ADD: 'pyok',
  KDD: 'thryok',
  dED: 'krok',
  gED: 'ŋrok',
  UED: 'dzrok',
  SED: 'crok',
  VED: 'srok',
  JED: 'trok',
  AED: 'prok',
  DED: 'mrok',
  CED: 'brok',
  BED: 'phrok',
  eED: 'khrok',
  LED: 'drok',
  hED: 'qrok',
  MED: 'nrok',
  KED: 'throk',
  IED: 'lrok',
  jED: 'xrok',
  iED: 'qhrok',
  TED: 'chrok',
  XVD: 'tit',
  cVD: 'nit',
  bVD: 'zhit',
  LVD: 'drit',
  QVD: 'sit',
  hVD: 'qit',
  OVD: 'chit',
  BVD: 'phit',
  dVD: 'kit',
  MVD: 'nrit',
  lVD: 'jit',
  eVD: 'khit',
  iVD: 'qhit',
  KVD: 'thrit',
  IVD: 'lit',
  JVD: 'trit',
  PVD: 'dzit',
  TVD: 'chrit',
  aVD: 'shit',
  NVD: 'cit',
  DVD: 'mit',
  AVD: 'pit',
  fVH: 'grit',
  CVD: 'bit',
  kVL: 'xryt',
  VVL: 'sryt',
  YVD: 'thit',
  UVD: 'dzrit',
  DVH: 'mryt',
  CVH: 'bryt',
  hVH: 'qrit',
  gVH: 'ŋrit',
  AVH: 'pryt',
  JVL: 'tryt',
  iVH: 'qhrit',
  dVH: 'krit',
  iVL: 'qhyt',
  bVL: 'zhyt',
  dVL: 'kyt',
  PVL: 'dzyt',
  lVL: 'jyt',
  NVL: 'cyt',
  QVL: 'syt',
  IVL: 'lyt',
  KVL: 'thryt',
  LVL: 'dryt',
  YVL: 'thyt',
  OVL: 'chyt',
  SVL: 'cryt',
  SWD: 'crit',
  VWD: 'srit',
  UWD: 'dzrit',
  DXD: 'myut',
  AXD: 'pyut',
  hXL: 'qyut',
  dXL: 'kyut',
  eXL: 'khyut',
  fXL: 'gyut',
  CXD: 'byut',
  iXL: 'qhyut',
  kXL: 'xyut',
  BXD: 'phyut',
  gXL: 'ŋyut',
  iYD: 'qhiət',
  dYD: 'kiət',
  gYD: 'ŋiət',
  fYD: 'giət',
  eYD: 'khiət',
  gZL: 'ŋyot',
  CZD: 'byot',
  kZL: 'xyot',
  dZL: 'kyot',
  hZL: 'qyot',
  fZL: 'gyot',
  eZL: 'khyot',
  AZD: 'pyot',
  DZD: 'myot',
  iZL: 'qhyot',
  hZD: 'qiat',
  iZD: 'qhiat',
  dZD: 'kiat',
  fZD: 'giat',
  BZD: 'phyot',
  gZD: 'ŋiat',
  DaD: 'mut',
  FaL: 'thut',
  daL: 'kut',
  CaD: 'but',
  EaL: 'tut',
  GaL: 'dut',
  haL: 'qut',
  iaL: 'qhut',
  gaL: 'ŋut',
  BaD: 'phut',
  IaL: 'lut',
  eaL: 'khut',
  HaL: 'nut',
  QaL: 'sut',
  OaL: 'chut',
  PaL: 'dzut',
  jbD: 'xət',
  jaL: 'xut',
  NaL: 'cut',
  jcD: 'xat',
  icD: 'qhat',
  EcD: 'tat',
  FcD: 'that',
  hcD: 'qat',
  IcD: 'lat',
  ecD: 'khat',
  GcD: 'dat',
  PcD: 'dzat',
  gcD: 'ŋat',
  dcD: 'kat',
  QcD: 'sat',
  OcD: 'chat',
  HcD: 'nat',
  DcD: 'mat',
  gcL: 'ŋuat',
  PcL: 'dzuat',
  AcD: 'pat',
  dcL: 'kuat',
  ecL: 'khuat',
  jcL: 'xuat',
  GcL: 'duat',
  icL: 'qhuat',
  hcL: 'quat',
  NcL: 'cuat',
  BcD: 'phat',
  FcL: 'thuat',
  IcL: 'luat',
  EcL: 'tuat',
  OcL: 'chuat',
  CcD: 'bat',
  jeD: 'xret',
  SeD: 'cret',
  CeD: 'bret',
  eeD: 'khret',
  jeL: 'xruet',
  AeD: 'pret',
  JeL: 'truet',
  heL: 'qruet',
  MeL: 'nruet',
  TeD: 'chret',
  deL: 'kruet',
  deD: 'kret',
  heD: 'qret',
  VeD: 'sret',
  DeD: 'mret',
  ieL: 'qhruet',
  MeD: 'nret',
  geL: 'ŋruet',
  eeL: 'khruet',
  BeD: 'phret',
  SeL: 'cruet',
  jdD: 'xrat',
  hdD: 'qrat',
  gdD: 'ŋrat',
  TdD: 'chrat',
  edD: 'khrat',
  idD: 'qhrat',
  KdD: 'thrat',
  ddL: 'kruat',
  jdL: 'xruat',
  KdL: 'thruat',
  JdL: 'truat',
  VdL: 'sruat',
  gdL: 'ŋruat',
  TdL: 'chruat',
  DdD: 'mrat',
  MdL: 'nruat',
  AdD: 'prat',
  ddD: 'krat',
  UdD: 'dzrat',
  JdD: 'trat',
  cdD: 'nьʼrat',
  QgD: 'set',
  OgD: 'chet',
  dgD: 'ket',
  NgD: 'cet',
  igL: 'qhuet',
  egL: 'khuet',
  dgL: 'kuet',
  jgL: 'xuet',
  hgL: 'quet',
  GgD: 'det',
  FgD: 'thet',
  jgD: 'xet',
  HgD: 'net',
  PgD: 'dzet',
  ggD: 'ŋet',
  DgD: 'met',
  AgD: 'pet',
  hgD: 'qet',
  egD: 'khet',
  igD: 'qhet',
  BgD: 'phet',
  CgD: 'bet',
  EgD: 'tet',
  IgD: 'let',
  QfD: 'siet',
  IfD: 'liet',
  JfD: 'triet',
  ffH: 'griet',
  cfD: 'niet',
  XfD: 'tiet',
  bfD: 'zhiet',
  ZfD: 'diet',
  gfH: 'ŋriet',
  DfD: 'miet',
  efH: 'khriet',
  AfD: 'piet',
  PfL: 'dzyet',
  NfL: 'cyet',
  QfL: 'syet',
  KfL: 'thryet',
  lfL: 'jyet',
  efL: 'khyet',
  hfP: 'qryet',
  cfL: 'nyet',
  afL: 'shyet',
  XfL: 'tyet',
  YfL: 'thyet',
  JfL: 'tryet',
  IfL: 'lyet',
  BfD: 'phiet',
  CfH: 'bryet',
  LfD: 'driet',
  AfH: 'pryet',
  VfL: 'sryet',
  afD: 'shiet',
  MfL: 'nryet',
  hfL: 'qyet',
  dfP: 'kryet',
  SfL: 'cryet',
  OfL: 'chyet',
  VfD: 'sriet',
  hfH: 'qriet',
  KfD: 'thriet',
  ifH: 'qhriet',
  RfL: 'zyet',
  YfD: 'thiet',
  lfD: 'jiet',
  UfD: 'dzriet',
  lpD: 'jiak',
  IpD: 'liak',
  dpD: 'kiak',
  XpD: 'tiak',
  apD: 'shiak',
  cpD: 'niak',
  YpD: 'thiak',
  hpD: 'qiak',
  epD: 'khiak',
  gpD: 'ŋiak',
  ZpD: 'diak',
  KpD: 'thriak',
  QpD: 'siak',
  SpD: 'criak',
  NpD: 'ciak',
  PpD: 'dziak',
  OpD: 'chiak',
  fpD: 'giak',
  hpL: 'qyak',
  CpD: 'byak',
  ipL: 'qhyak',
  kpL: 'xyak',
  dpL: 'kyak',
  JpD: 'triak',
  LpD: 'driak',
  epL: 'khyak',
  fpL: 'gyak',
  MpD: 'nriak',
  BpD: 'phyak',
  ipD: 'qhiak',
  GqD: 'dak',
  DqD: 'mak',
  IqD: 'lak',
  FqD: 'thak',
  NqD: 'cak',
  OqD: 'chak',
  dqD: 'kak',
  eqD: 'khak',
  gqD: 'ŋak',
  BqD: 'phak',
  hqD: 'qak',
  CqD: 'bak',
  iqD: 'qhak',
  QqD: 'sak',
  jqD: 'xak',
  PqD: 'dzak',
  AqD: 'pak',
  HqD: 'nak',
  iqL: 'qhuak',
  dqL: 'kuak',
  hqL: 'quak',
  jqL: 'xuak',
  eqL: 'khuak',
  gqL: 'ŋuak',
  IqL: 'luak',
  NqL: 'cuak',
  DrD: 'mrak',
  JrD: 'trak',
  CrD: 'brak',
  ArD: 'prak',
  fsD: 'griek',
  dsD: 'kriek',
  VsD: 'sriek',
  TsD: 'chriek',
  jrL: 'xruak',
  SrD: 'crak',
  UrD: 'dzrak',
  esD: 'khriek',
  grD: 'ŋrak',
  gsD: 'ŋriek',
  erD: 'khrak',
  hrD: 'qrak',
  isD: 'qhriek',
  KrD: 'thrak',
  BrD: 'phrak',
  irD: 'qhrak',
  jrD: 'xrak',
  drD: 'krak',
  irL: 'qhruak',
  LrD: 'drak',
  drL: 'kruak',
  hrL: 'qruak',
  erL: 'khruak',
  CsD: 'briek',
  DtD: 'mrek',
  jtL: 'xruek',
  dtL: 'kruek',
  AtD: 'prek',
  CtD: 'brek',
  UtD: 'dzrek',
  StD: 'crek',
  UtL: 'dzruek',
  TtD: 'chrek',
  etD: 'khrek',
  itL: 'qhruek',
  jtD: 'xrek',
  dtD: 'krek',
  JtD: 'trek',
  htD: 'qrek',
  VtD: 'srek',
  BtD: 'phrek',
  gtD: 'ŋrek',
  ItD: 'lrek',
  StL: 'cruek',
  VtL: 'sruek',
  ftL: 'gruek',
  MtD: 'nrek',
  QuD: 'siek',
  NuD: 'ciek',
  huD: 'qiek',
  luD: 'jiek',
  auD: 'shiek',
  YuD: 'thiek',
  ZuD: 'diek',
  XuD: 'tiek',
  LuD: 'driek',
  OuD: 'chiek',
  RuD: 'ziek',
  PuD: 'dziek',
  CuD: 'biek',
  luL: 'jyek',
  iuL: 'qhyek',
  AuD: 'piek',
  BuD: 'phiek',
  buD: 'zhiek',
  AuH: 'priek',
  JuD: 'triek',
  KuD: 'thriek',
  XuL: 'tyek',
  OuL: 'chyek',
  QvD: 'sek',
  dvD: 'kek',
  BvD: 'phek',
  IvD: 'lek',
  EvD: 'tek',
  jvD: 'xek',
  gvD: 'ŋek',
  GvD: 'dek',
  FvD: 'thek',
  NvD: 'cek',
  evD: 'khek',
  HvD: 'nek',
  PvD: 'dzek',
  DvD: 'mek',
  CvD: 'bek',
  AvD: 'pek',
  evL: 'khuek',
  dvL: 'kuek',
  OvD: 'chek',
  ivD: 'qhek',
  ivL: 'qhuek',
  XwD: 'tiək',
  LwD: 'driək',
  IwD: 'liək',
  KwD: 'thriək',
  JwD: 'triək',
  bwD: 'zhiək',
  QwD: 'siək',
  ZwD: 'diək',
  awD: 'shiək',
  iwD: 'qhiək',
  UwD: 'dzriək',
  fwD: 'giək',
  MwD: 'nriək',
  TwD: 'chriək',
  hwD: 'qiək',
  VwD: 'sriək',
  ewD: 'khiək',
  dwD: 'kiək',
  lwD: 'jiək',
  NwD: 'ciək',
  AwD: 'pryək',
  kwL: 'xryək',
  iwL: 'qhryək',
  BwD: 'phryək',
  SwD: 'criək',
  CwD: 'bryək',
  gwD: 'ŋiək',
  PwD: 'dziək',
  DwD: 'mryək',
  YwD: 'thiək',
  ExD: 'tək',
  NxD: 'cək',
  IxD: 'lək',
  FxD: 'thək',
  exD: 'khək',
  GxD: 'dək',
  ixD: 'qhək',
  DxD: 'mək',
  PxD: 'dzək',
  QxD: 'sək',
  AxD: 'pək',
  CxD: 'bək',
  jxL: 'xuək',
  dxL: 'kuək',
  hxD: 'qək',
  jxD: 'xək',
  HxD: 'nək',
  dxD: 'kək',
  OxD: 'chək',
  BxD: 'phək',
  ixL: 'qhuək',
  O1D: 'chip',
  Z1D: 'dip',
  X1D: 'tip',
  R1D: 'zip',
  P1D: 'dzip',
  c1D: 'nip',
  h1D: 'qip',
  a1D: 'ship',
  N1D: 'cip',
  f1H: 'grip',
  L1D: 'drip',
  J1D: 'trip',
  I1D: 'lip',
  d1H: 'krip',
  g1H: 'ŋrip',
  e1H: 'khrip',
  Q1D: 'sip',
  V1D: 'srip',
  i1H: 'qhrip',
  S1D: 'crip',
  h1H: 'qrip',
  K1D: 'thrip',
  k1D: 'xrip',
  l1D: 'jip',
  M1D: 'nrip',
  U1D: 'dzrip',
  C1H: 'brip',
  A1H: 'prip',
  T1D: 'chrip',
  Y1D: 'thip',
  j2D: 'xəp',
  d2D: 'kəp',
  E2D: 'təp',
  Q2D: 'səp',
  G2D: 'dəp',
  F2D: 'thəp',
  P2D: 'dzəp',
  N2D: 'cəp',
  I2D: 'ləp',
  H2D: 'nəp',
  e2D: 'khəp',
  h2D: 'qəp',
  i2D: 'qhəp',
  g2D: 'ŋəp',
  O2D: 'chəp',
  j3D: 'xap',
  I3D: 'lap',
  E3D: 'tap',
  F3D: 'thap',
  i3D: 'qhap',
  H3D: 'nap',
  G3D: 'dap',
  Q3D: 'sap',
  g3D: 'ŋap',
  P3D: 'dzap',
  d3D: 'kap',
  e3D: 'khap',
  h3D: 'qap',
  O3D: 'chap',
  X3D: 'tьʼap',
  l4D: 'jiep',
  N4D: 'ciep',
  a4D: 'shiep',
  Z4D: 'diep',
  I4D: 'liep',
  P4D: 'dziep',
  L4D: 'driep',
  h4H: 'qriep',
  M4D: 'nriep',
  Y4D: 'thiep',
  c4D: 'niep',
  X4D: 'tiep',
  O4D: 'chiep',
  K4D: 'thriep',
  f4H: 'griep',
  J4D: 'triep',
  k4D: 'xriep',
  V4D: 'sriep',
  d4H: 'kriep',
  h4D: 'qiep',
  F5D: 'thep',
  j5D: 'xep',
  d5D: 'kep',
  e5D: 'khep',
  G5D: 'dep',
  H5D: 'nep',
  Q5D: 'sep',
  I5D: 'lep',
  E5D: 'tep',
  P5D: 'dzep',
  N5D: 'cep',
  i5D: 'qhep',
  j6D: 'xrep',
  e6D: 'khrep',
  U6D: 'dzrep',
  d6D: 'krep',
  S6D: 'crep',
  T6D: 'chrep',
  M6D: 'nrep',
  i6D: 'qhrep',
  V6D: 'srep',
  J6D: 'trep',
  h6D: 'qrep',
  g6D: 'ŋrep',
  K6D: 'threp',
  j7D: 'xrap',
  h7D: 'qrap',
  d7D: 'krap',
  V7D: 'srap',
  i7D: 'qhrap',
  g8D: 'ŋiap',
  i8D: 'qhiap',
  e8D: 'khiap',
  d8D: 'kiap',
  h8D: 'qiap',
  l8D: 'jiap',
  f8D: 'giap',
  C9D: 'byop',
  A9D: 'pyop',
  B9D: 'phyop',
}[音韻地位.編碼] || '?';
});

_defaultDerivers.zyepheng = _schemata.zyepheng();

/**
 * 隋拼
 *
 * https://github.com/Magnezone462/Zyevio
 *
 * @author N/A
 * @param {Qieyun.音韻地位} 音韻地位 切韻音系音韻地位
 * @param {string=} 字頭 字頭（可選）
 * @param {undefined=} 舊選項參數 已棄用，請用 `<方案名>.方案(選項)(音韻地位, 字頭?)` 代替
 * @return {string} 音韻地位對應的隋拼
 * @property {推導方案<string>} schema 推導方案，可用於指定推導選項、獲取方案詳細資訊等
 */
export function zyepheng(音韻地位, 字頭, 舊選項參數) {
  return _perform("zyepheng", _defaultDerivers.zyepheng, 音韻地位, 字頭, 舊選項參數);
}
/** @type {推導方案<string>} */
zyepheng.schema = _schemata.zyepheng

_schemata.sliark_peengqvim = new 推導方案(function (選項, 音韻地位, 字頭) {
if (!音韻地位)
  return [
    ["$legacy", true],
    ['字母ŋ', true],
    ['字母ü', true],
    ['字母ö', false],
    ['痕韵的主元音：', [1, 'ə', 'ë', 'eo']],
    ['字母æ', false],
    ['字母å', false],
    ['之韵的元音和BC类开口介音:', [1, 'ı', 'ɨ', 'v', 'ii']],
    ['二等使用下加点', false],
    ['影母标记：', [2, '无标记', 'q', 'ʔ', 'ʾ']],
    [
      '云母开口写法：',
      [1, '视同匣母', '无标记（除礥小韵外,仅影母有标记时有效）'],
    ],
    ['大写类型：', [1, '不大写', '首字母大写', '全部大写']],
  ];

for (var key in 選項) {
  選項[key.replace(':', '')] = 選項[key];
  選項[key.replace('：', '')] = 選項[key];
  // 去除冒號，方便下面代碼中引用
}

const is = (x) => 音韻地位.屬於(x);

function get聲母() {
  if (音韻地位.描述 === '知開三麻平' || (音韻地位.描述 === '知開二庚上' && 字頭 === '打')) return 't'; // 爹打
  return {
    幫: 'p',  滂: 'ph',  並: 'b',  明: 'm',
    端: 't',  透: 'th',  定: 'd',  泥: 'n',  來: 'l',
    知: 'tr', 徹: 'thr', 澄: 'dr', 孃: 'nr',
    見: 'k',  溪: 'kh',  羣: 'g',  疑: (選項.字母ŋ ? 'ŋ' : 'ng'), 云: '',
    影: 'q',  曉: 'h',   匣: 'x',
    精: 'ts',  清: 'tsh',  從: 'dz',  心: 's',  邪: 'z',
    莊: 'tsr', 初: 'tshr', 崇: 'dzr', 生: 'sr', 俟: 'zr',
    章: 't',  昌: 'th',  常: 'd',  書: 's', 船: 'z', 日: 'n', 以: '',
  }[音韻地位.母];
}

function get韻母() {
  const 表達式到韻 = [
    // 爲了方便推導，根據拼寫對韻類稍作調整
    ['臻韻', '眞'],
    ['蒸韻 幫組 或 蒸韻 合口 或 蒸韻 溪母 平聲', '冰'], // 硱（溪開三蒸平）歸 B 類
    ['東韻 三等', '終'],
    ['清韻', '庚'],
    ['陽韻', '唐'],
  ];
  const 韻到韻尾 = [
    ['脂之尤侯 　佳　模　 支魚虞 麻歌', ''],
    ['冰蒸終東 青耕登冬江 　　鍾 庚唐', (選項.字母ŋ ? 'ŋ' : 'ng'), 'k'],
    ['　微微　 齊皆咍灰　 祭廢廢 夬泰', 'j'],
    ['眞欣文　 先山痕魂　 仙元元 刪寒', 'n', 't'],
    ['　　幽　 蕭　　　　 宵　　 肴豪', 'w'],
    ['侵　　　 添咸　覃　 鹽嚴凡 銜談', 'm', 'p'],
  ];
  const 元音列表 = [
    'i',				'ı', 'u', 'u',
    'e', 'ee', 'ə', 'o', 'o',
    'e',			 'ə', 'o',
          'ae', 'a',
  ];

  let 韻 = 音韻地位.韻;
  表達式到韻.some((pair) => {
    if (is(pair[0])) return (韻 = pair[1]);
  });

  let 元音;
  let 韻尾;
  韻到韻尾.some((item) => {
    if (item[0].includes(韻)) {
      元音 =
        元音列表[
          item[0].replace(/ /g, '')[is('開口') ? 'indexOf' : 'lastIndexOf'](韻)
        ];
      韻尾 = item[1 + is('入聲')];
      return true;
    }
  });

  // 添加三等介音
  let 介音;
  if (is('三等')) {
    if (is('幫組')) {
      if ('𩦠'.includes(字頭)) {
        介音 = 'i';
      } else if (is('重紐A類 或 清韻')) {
        if ('碧'.includes(字頭)) {
          介音 = 'ı';
        } else {
          介音 = 'i';
        }
      } else if (is('重紐B類 或 庚韻 或 幽韻 或 蒸韻')) {
        介音 = 'ı';
      } else {
        介音 = 'u';
      }
    }
    //章組的介音同時作為章組標記
    if (is('章昌常日書船以母')) {
      介音 = is('開口') ? 'j' : 'y';
    }
    //云母的介音同時作為云母標記
    if (is('云母')) {
      if ('礥'.includes(字頭)) {
        介音 = 'xi';
      } else {
        介音 = is('開口') ? 'xı' : 'w';
      }
    }
    //普通銳音
    if (is('精組 或 知組 或 來母 或 端組')) {
      if (is('虞鍾尤東韻')) {
        介音 = 'u';
      } else if (is('之蒸韻')) {
        介音 = 'ı';
      } else {
        介音 = is('開口') ? 'i' : 'ü';
      }
    }
    //莊組永遠視作B類
    if (is('莊組')) {
      介音 = is('開口') ? 'ı' : 'u';
    }
    if (is('見溪羣疑影曉匣母')) {
      if (is('幽韻')) {
        if (音韻地位.描述 === '曉三幽平' && '烋休𠇾'.includes(字頭)) {
          介音 = 'ı';
        } else {
          介音 = 'i';
        }
      } else if (is('重紐A類 或 清韻')) {
        介音 = is('開口') ? 'i' : 'ü';
      } else {
        介音 = is('開口') ? 'ı' : 'u';
      }
    }
  }
  // 非三等介音
  else {
    if (is('幫組')) {
      介音 = '';
    } else if (is('章昌常日書船以母')) {
      介音 = is('開口') ? 'j' : 'y';
    } else if (is('云母')) {
      介音 = is('開口') ? 'x' : 'xo'; //匣云混切归匣
    } else {
      介音 = is('開口') ? '' : 'o';
    }
  }
  let 韻母;
  韻母 = 介音 + 元音 + 韻尾;
  韻母 = 韻母.replace('ii', 'i');
  韻母 = 韻母.replace('ıı', 'ı');
  韻母 = 韻母.replace('uu', 'u');
  if (!is('江韻')) {
    韻母 = 韻母.replace('oo', 'o');
  }
  韻母 = 韻母.replace('uw', 'u');
  return 韻母;
}

function get聲調() {
  return { 上: 'x', 去: 'h' }[音韻地位.聲] || '';
}

let 拼音 = get聲母() + get韻母() + get聲調();

if (選項.二等使用下加点) {
  拼音 = 拼音.replace('ee', 'ẹ');
  拼音 = 拼音.replace('ae', 'ạ');
  拼音 = 拼音.replace('oo', 'ọ');
}
拼音 = 選項.字母ö ? 拼音.replace('oe', 'ö') : 拼音;
if (選項.痕韵的主元音 == 'ë') {
  拼音 = 拼音.replace('ə', 'ë');
}
if (選項.痕韵的主元音 == 'eo') {
  拼音 = 拼音.replace('ıə', 'ıo');
  拼音 = 拼音.replace('iə', 'io');
  拼音 = 拼音.replace('jə', 'jo');
  拼音 = 拼音.replace('ə', 'eo');
}
if (!選項.字母ü) {
  拼音 = 拼音.replace('y', 'jy');
  拼音 = 拼音.replace('ü', 'y');
}
拼音 = 選項.字母ŋ ? 拼音.replace('ng', 'ŋ') : 拼音;
拼音 = 選項.字母å ? 拼音.replace('oa', 'å') : 拼音;
if (選項.字母æ) {
  拼音 = 拼音.replace('ae', 'æ');
  拼音 = 拼音.replace('ạ', 'æ');
}
if (選項.影母标记 === '无标记') {
  拼音 = 拼音.replace('q', '');
} else {
  拼音 = 拼音.replace('q', 選項.影母标记);
  if (選項.云母开口写法 === '无标记（除礥小韵外,仅影母有标记时有效）') {
    拼音 = 拼音.replace('xı', 'ı');
  }
}
if (選項.之韵的元音和BC类开口介音 != 'ı') {
  拼音 = 拼音.replace('ı', 選項.之韵的元音和BC类开口介音);
}

function minu2maju(chr) {
  return {
    'a':'A', 'å':'Å', 'b':'B', 'g':'G', 'd':'D', 'e':'E', 'ə':'∃', 'ë':'Ë',
    'z':'Z', 'h':'H', 'i':'I', 'ı':'Ɨ', 'ɨ':'Ɨ', 'j':'J', 'k':'K', 'l':'L',
    'm':'M', 'n':'N', 'ŋ':'Ŋ', 'o':'O', 'ö':'Ö', 'p':'P', 'q':'Q', 'r':'R',
    's':'S', 't':'T', 'u':'U', 'ü':'Ü', 'v':'V', 'w':'W', 'y':'Y', 'x':'X', 'æ':'Æ',
  }[chr] || chr;
}

if (選項.大写类型 === '首字母大写') {
  拼音 = minu2maju(拼音[0]) + 拼音.slice(1);
}

if (選項.大写类型 === '全部大写') {
  拼音 = 拼音.split('').map(minu2maju).join('');
}

return 拼音;
});

_defaultDerivers.sliark_peengqvim = _schemata.sliark_peengqvim();

/**
 * Sliark 拼音
 *
 * https://zhuanlan.zhihu.com/p/482113176
 *
 * @author Sliark
 * @param {Qieyun.音韻地位} 音韻地位 切韻音系音韻地位
 * @param {string=} 字頭 字頭（可選）
 * @param {undefined=} 舊選項參數 已棄用，請用 `<方案名>.方案(選項)(音韻地位, 字頭?)` 代替
 * @return {string} 音韻地位對應的Sliark 拼音
 * @property {推導方案<string>} schema 推導方案，可用於指定推導選項、獲取方案詳細資訊等
 */
export function sliark_peengqvim(音韻地位, 字頭, 舊選項參數) {
  return _perform("sliark_peengqvim", _defaultDerivers.sliark_peengqvim, 音韻地位, 字頭, 舊選項參數);
}
/** @type {推導方案<string>} */
sliark_peengqvim.schema = _schemata.sliark_peengqvim

_schemata.panwuyun = new 推導方案(function (選項, 音韻地位, 字頭) {
const is = (...x) => 音韻地位.屬於(...x);
const when = (...x) => 音韻地位.判斷(...x);
const is2000 = Boolean(選項.版本?.includes('2000'));

if (!音韻地位) return [
  ['版本', [2,
    '2000：漢語歷史音韻學',
    '2013：漢語中古音',
  ]],
  ['非前三等介音', [2, 'i（原書簡寫）', 'ɨ（實際音值）']],
  ['聲調記號', is2000 ?
    [2, '隱藏', '五度符號（按照 2013）', '調值數字（按照 2013）'] : // 《漢語歷史音韻學》未給具體調值
    [2, '隱藏', '五度符號', '調值數字']
  ],
  ['送氣記號', is2000 ? [1, 'ʰ（通用）', 'h（原書）'] : null],
  ['支韻', is2000 ? [1, 'iɛ（簡寫）', 'iᵉ（實際音值）'] : null],
  ['虞韻', is2000 ? [2, 'io（簡寫）', 'iʊ（實際音值）'] : null],
];
/* 韻典網與本方案 2000 擬音不同之處：
 *
 * - 歌一合誤作 uɑ，應爲 ʷɑ
 * - 部分祭合、薛合誤作 iei、iet，應爲 iɛi、iɛt
 * - 幫組陽韻歸合口。原書雖未指明，但暗示爲開口；本方案歸開口
 * - 送氣記號改作 ʰ，原書作 h；本方案可自選
 * - 支韻、虞韻作 iɛ、io，原書韻母擬音比較表作 iᵉ、iʊ；本方案可自選
 */

function get聲母() {
  let 聲母 = {
    幫: 'p', 滂: 'pʰ', 並: 'b', 明: 'm',
    端: 't', 透: 'tʰ', 定: 'd', 泥: 'n', 來: 'l',
    知: 'ʈ', 徹: 'ʈʰ', 澄: 'ɖ', 孃: 'ɳ',
    見: 'k', 溪: 'kʰ', 羣: 'ɡ', 疑: 'ŋ',
    影: 'ʔ', 曉: 'h', 匣: 'ɦ', 云: 'ɦ',
    精: 'ts', 清: 'tsʰ', 從: 'dz', 心: 's', 邪: 'z',
    莊: 'tʂ', 初: 'tʂʰ', 崇: 'dʐ', 生: 'ʂ', 俟: 'ʐ',
    章: 'tɕ', 昌: 'tɕʰ', 常: 'dʑ', 書: 'ɕ', 船: 'ʑ', 日: 'ȵ', 以: 'j',
    // 生母《漢語歷史音韻學》作“ʃ”，係誤植
    // 從母《漢語中古音》聲母表作“ʣ”，係排版錯誤，正文作“dz”
    // 俟母《漢語中古音》聲母表未列，此處補上
  }[音韻地位.母];
  if (is2000) 聲母 = 聲母.replace('ʰ', 選項.送氣記號[0]);
  return 聲母;
}

function get韻母() {
  if (is`凡韻`) 音韻地位 = 音韻地位.調整({ 韻: '嚴' });
  const 元音表 = {
    ɪ: '　　　臻　　',
    i: '脂　侵眞　幽', ɨ: '之蒸　欣微尤', u: '侯東　文　　',
    e: '　青添先齊蕭', ə: '　登覃痕咍　', o: '模冬　魂灰　',
    ᴇ: '支清鹽仙祭宵', ɤ: '魚　　元　　', ʊ: '虞鍾　　　　',
    ɛ: '佳耕咸山皆　', a: '　陽嚴　廢　', ɔ: '　江　　　　',
    æ: '麻庚銜刪夬肴', ɑ: '歌唐談寒泰豪',
  };
  const 韻尾列表 = [''].concat(is`舒聲` ? [...'ŋmniu'] : [...'kpt']);
  let 韻核 = Object.keys(元音表).find(e => 元音表[e].includes(音韻地位.韻));
  let 韻尾 = 韻尾列表[元音表[韻核].indexOf(音韻地位.韻)];

  let 介音 = '';
  if (is`合口` && ![...'mpu'].includes(韻尾) && ![...'uoʊɔ'].includes(韻核))
    介音 += 'ʷ';
  if (is`幫組` && ![...'ŋkmpu'].includes(韻尾) && [...'ɨəɤaɑ'].includes(韻核) && !is`泰韻`)
    介音 += is2000 ? 'ʷ' : 'u̯';
  if (is`二等 或 庚韻 或 重紐B類 非 蒸幽韻`)
    介音 += is2000 ? 'ɯ' : 'ɣ';
  if (is`三等` && ![...'ɪiɨ'].includes(韻核))
    介音 += [...'ᴇæ'].includes(韻核) ? 'i' : 選項.非前三等介音[0];
  if ([...'oʊ'].includes(韻核))
    介音 += is2000 ? 介音 ? '' : 'u' : 'u̯';

  // 調整韻核
  if (is2000) {
    const 韻核鏈移列表 = [...'ᴇɛæaɐ']; // “鏈移”只是比喻
    if (韻核鏈移列表.includes(韻核)) 韻核 = 韻核鏈移列表[韻核鏈移列表.indexOf(韻核) + 1];

    韻核 = {
      尤: 'i', 幽: 'ɨ', 侯: 'əu',
      支: 選項.支韻[1], 魚: 'ɔ', 虞: 選項.虞韻[1],
      元: 'ɐ', 鍾: 'o',
    }[音韻地位.韻] ?? 韻核;
  } else {
    韻核 = 韻核.replace('ʊ', 'o̝');
  }
  return 介音 + 韻核 + 韻尾;
}

function get聲調() {
  return 選項.聲調記號 === '隱藏' ? '' : {
    '五度符號': ['˧', '˧˥', '˥˩', '꜊'],
    '調值數字': ['³³', '³⁵', '⁵¹', '³'],
  }[選項.聲調記號.slice(0, 4)]['平上去入'.indexOf(音韻地位.聲)];
}

return get聲母() + get韻母() + get聲調();
});

_defaultDerivers.panwuyun = _schemata.panwuyun();

/**
 * 潘悟雲擬音
 *
 * 兩個版本：
 *
 * - 潘悟雲. 漢語歷史音韻學. 上海: 上海教育出版社, 2000.
 * - 潘悟雲, 張洪明. 漢語中古音. 語言研究, 2013, 33 (2): 1–7.
 *
 * @author unt
 * @param {Qieyun.音韻地位} 音韻地位 切韻音系音韻地位
 * @param {string=} 字頭 字頭（可選）
 * @param {undefined=} 舊選項參數 已棄用，請用 `<方案名>.方案(選項)(音韻地位, 字頭?)` 代替
 * @return {string} 音韻地位對應的潘悟雲擬音
 * @property {推導方案<string>} schema 推導方案，可用於指定推導選項、獲取方案詳細資訊等
 */
export function panwuyun(音韻地位, 字頭, 舊選項參數) {
  return _perform("panwuyun", _defaultDerivers.panwuyun, 音韻地位, 字頭, 舊選項參數);
}
/** @type {推導方案<string>} */
panwuyun.schema = _schemata.panwuyun

_schemata.unt = new 推導方案(function (選項, 音韻地位, 字頭) {
const is = (...x) => 音韻地位.屬於(...x);
const when = (...x) => 音韻地位.判斷(...x);

const is專業模式 = 選項.專業模式 ?? false;
const isL = 選項.版本?.includes('L') ?? true;
const isJ新版 = Boolean(選項.版本?.includes('J') && 選項.版本?.includes('新版'));
const isJ原版 = Boolean(選項.版本?.includes('J') && 選項.版本?.includes('原版'));
const is朗讀音 = Boolean(選項.版本?.includes('朗讀音'));

function get選項列表() {
  return [
    ['版本', [is專業模式 ? 4 : 2,
      '2019：切韻朗讀音',
      '2020：切韻擬音 J（原版）',
      '2020：切韻擬音 J（2022 新版）',
      '2022：切韻擬音 L',
    ].filter((_, i) => is專業模式 || [0, 3, 4].includes(i))],
    ['專業模式', is專業模式],

    // 以下一些選項內容後加了空格，是爲了在切換回其他方案時刷新選中的內容
    '基本選項',
    ['後低元音', !is朗讀音 ? [1, 'a', 'ɑ'] : [1, 'ɑ ']],
    ['祭泰夬廢韻尾', !is朗讀音 ? [2, 'j', 'ɹ'] : [1, 'jɕ']],
    ['聲調記號', [1,
      '上ˊ 去ˋ',
      '上ʔ 去h',
      '上ˀ 去ʰ', // 上標的 ʔ Unicode 未收，以 ˀ 代替
      '五度符號',
      '五度符號（帶拖腔）',
    ].filter((_, i) => (is專業模式 || [0, 1, 3, 4].includes(i)) && !(is朗讀音 && i === 4))],
    ['鈍C介音',
      isL && [1, '開∅ 合w'] ||
      // β、ʋ 都代表雙唇近音（即 β̞ = ʋ̟）
      isJ新版 && [1, '開j̈ 唇ʋ 合w', '開j̈ 唇ʋ 合ɥ̈', '開j̈ 唇β 合ɥ̈'].slice(0, is專業模式 ? 4 : 3) ||
      isJ原版 && [1, '開j̈ 唇β 合ɥ̈ '] ||
      is朗讀音 && [1, '開j̈ 唇ɥ̈ 合ẅ'],
    ],
  ].concat(is專業模式 ? [
    '高級選項',
    ['知組', !isJ原版 ? [1, 'ʈ', 'tɹ'] : [2, 'ʈ ', 'tɹ ']],
    ['二等元音記號', !is朗讀音 ? [1,
      '咽化 ◌ˤ',
      'r音鉤（帶空隙）◌˞ ',
      'r音鉤（無空隙）◌˞',
      '下等號 ◌͇',
      '雙下橫線 ◌̳', // 僅在下等號顯示不正常時使用
    ] : null],
    ['顯示純音位形式', isL ? false : null],
    '', // 在 v0.1 推導器中使「恢復預設值」按鈕位於下一行
  ] : []);
}

function 調整選項() {
  選項.後低元音 = 選項.後低元音.trim();
  選項.知組 = 選項.知組?.trim();
}

function 調整音韻地位() {
  function 調整(表達式, 調整屬性) { if (is(表達式)) 音韻地位 = 音韻地位.調整(調整屬性); }
  調整('明母 尤韻', { 等: '一', 韻: '侯' }); // 實爲一等
  調整('云母 非 三等 或 云母 重紐A類', { 母: '匣' }); // 現在廣韻資料中無，但其他資料中可能有
  調整('銳音 幽韻', { 韻: '尤' });
}

function get聲母() {
  return when([
    ['云母 開口 非 (侵鹽韻 入聲)', { 云: '' }], // 煜曄兩小韻爲“合口”
    [選項.顯示純音位形式 === true && '精組 非 三等', {
      精: 'tᵴ', 清: 'tᵴʰ', 從: 'dᵶ', 心: 'ᵴ', 邪: 'ᵶ', // 精組僅在音位形式中區分三等非三等
    }],
    ['(鈍音 非 羣母 非 三等) 或 (來母 非 三等) 或 (匣母)', {
      // 羣母一律按三等寫，匣母一律按非三等寫
      幫: 'ᵱ', 滂: 'ᵱʰ', 並: 'ᵬ', 明: 'ᵯ',
      見: 'q', 溪: 'qʰ', 疑: 'ɴ',
      影: 'ʡ', 曉: 'χ', 匣: 'ʁ',
      來: 'ɫ',
    }],
    ['', {
      幫: 'p', 滂: 'pʰ', 並: 'b', 明: 'm',
      端: 't', 透: 'tʰ', 定: 'd', 泥: 'n', 來: 'l',
      知: 'ʈ', 徹: 'ʈʰ', 澄: 'ɖ', 孃: 'ɳ',
      見: 'k', 溪: 'kʰ', 羣: 'ɡ', 疑: 'ŋ', 云: 'w',
      影: 'ʔ', 曉: 'x',
      精: 'ts', 清: 'tsʰ', 從: 'dz', 心: 's', 邪: 'z',
      莊: 'tʂ', 初: 'tʂʰ', 崇: 'dʐ', 生: 'ʂ', 俟: 'ʐ',
      章: 'tɕ', 昌: 'tɕʰ', 常: 'dʑ', 書: 'ɕ', 船: 'ʑ', 日: 'ɲ', 以: 'j',
    }]
  ])[音韻地位.母];
}

function get韻基() {
  const 所有韻 = [
    // 介音（AB 韻只列在第一列，開合兼有的韻不列在最後一列）
    // 三　等：j|ɹ|∅|w
    // 非三等：e̯|ʕ|∅|o̯
    ['ɨ', '　　　脂眞幽侵|　　　　臻　　|　蒸　微欣　　|　　東　文　　'],
    ['ə', '　青　齊先蕭添|佳耕江皆山　咸|　登　咍痕豪覃|模　冬灰魂　　'], // 非三等
    ['ə', '　　　祭仙宵鹽|　　　　　　　|　　　廢元　嚴|　　鍾　　　凡'], // 三等
    ['a', '　清陽　　　　|麻庚　夬刪肴銜|歌　唐泰寒　談'], // 陽應與唐重合，排在前面節省空間
    ['ɨ', '　　　　　　　之尤支魚'], // 二合元音
    ['ɨ', '　　　　　　　　侯　虞'],
  ];
  const 韻尾列表 = [''].concat(is`舒聲` ? [...'ŋɴjnwmɨueə'] : [...'kq t p']);

  let 匹配行 = 所有韻.find(e => e[1].includes(音韻地位.韻));
  let 韻核 = 匹配行[0];
  let 韻尾 = 韻尾列表[匹配行[1].split('|').find(v => v.includes(音韻地位.韻)).indexOf(音韻地位.韻)];
  if (is`祭泰夬廢韻 去聲`) 韻尾 = 選項.祭泰夬廢韻尾;
  return [韻核, 韻尾];
}

function get介音() {
  const A = 'j';
  const B = 'ɹ';
  const C = '';
  let 等類介音 = when([
    ['四等', A],
    ['二等', B],
    ['一等', C],

    ['銳音', [ // 銳音包含以母
      ['端精組', A], // 加入端組是爲了包含爹小韻
      ['', ''],
    ]],
    ['重紐A類', A],
    ['重紐B類', B],
    ['云母 支脂祭眞臻仙宵麻庚清蒸幽侵鹽韻', B], // TODO: qieyun-js 更新後簡化
    ['庚韻', B],
    ['幽韻 幫組', B],
    ['蒸韻 非 開口', B],
    ['麻清幽韻', A],
    ['', C],
  ]);
  let 合口介音 = is`(合口 或 虞韻) 非 (幫組 或 云母)` ? 'w' : ''; // TODO: qieyun-js 更新後刪去虞韻
  return 等類介音 + 合口介音;
}

function get聲調() {
  if (!選項.聲調記號.includes('五度符號')) {
    if (is`平入聲`) return '';
    let 聲調記號 = 選項.聲調記號.split(' ')[+is`去聲`].slice(1);
    if (is朗讀音 && is`祭泰夬廢韻 去聲` && ['ʰ', 'h'].includes(聲調記號)) return '';
    return { 'ˊ': '́', 'ˋ': '̀' }[聲調記號] ?? 聲調記號;
  }

  const 五度符號列表 = is朗讀音 ? [
    '˦', '˦˦˥', '˥˩', '˥',
    '˨˩', '˨˨˧', '˧˩˨', '˨˩',
  ] : 選項.聲調記號.includes('帶拖腔') ? [
    '˦˦˨', '˦˥', '˦˩˨', '˦',
    '˨˨˩', '˨˦', '˨˩˨', '˨',
  ] : [
    '˦', '˦˥', '˦˩', '˦',
    '˨', '˨˦', '˨˩', '˨',
  ];
  const is陽調 = is`全濁 或 次濁 非 上聲` && !(is朗讀音 && is`次濁 入聲`);
  return 五度符號列表['平上去入'.indexOf(音韻地位.聲) + is陽調 * 4];
}

function 音位to音值(音節) {
  const is雙唇韻尾 = [...'wmp'].includes(音節.韻尾);
  const is展唇鈍尾 = [...'ɨŋk'].includes(音節.韻尾);
  const is圓唇鈍尾 = [...'uɴq'].includes(音節.韻尾);
  const is後部韻尾 = is展唇鈍尾 || is圓唇鈍尾 || 音節.韻尾 === 'ə'; // ə 介於圓展之間
  const is音節首含銳 = !['', 'w'].includes(音節.介音) || is`銳音 三等`;
  const is音節首含唇 = 音節.介音.includes('w') || is`幫組` || 音節.聲母 === 'w';

  // (1) 韻核前化
  //     韻核前的非小舌化銳音使韻核前化，但後部韻尾使韻核不被前化（-K 只使 /ɨ/ 不被前化）。
  if (
    is音節首含銳 &&
    !(is後部韻尾 && (is圓唇鈍尾 || 音節.韻核 === 'ɨ')) &&
    !is`廢韻 非 去聲` // 茝佁䑂 3 小韻也除外
  ) {
    音節.替換('韻核', 'ɨ', 'i');
    音節.替換('韻核', 'ə', 'e');
  }

  // (2) 韻核圓唇化
  //     a.  圓唇鈍韻尾使韻核圓唇化
  //     b1. 含唇音的三　等音節首使 ɨ 圓唇化（三　等 w 實現爲 u̯，同化 ɨ）
  //     b2. 含唇音的非三等音節首使 ə 圓唇化（非三等 w 實現爲 o̯，同化 ə）
  //     b3. 但展唇鈍韻尾和雙唇韻尾排斥圓唇，使韻核保持展唇
  //     c.  模韻圓唇化（原因不明）
  if (is圓唇鈍尾) {
    音節.替換('韻核', 'ɨ', 'u');
    音節.替換('韻核', 'ə', 'o');
  }
  if (is音節首含唇 && !is展唇鈍尾 && !is雙唇韻尾) {
    if (is`三等`) {
      音節.替換('韻核', 'ɨ', 'u');
      音節.替換('韻尾', 'ə', 'o'); // 虞韻
    } else {
      音節.替換('韻核', 'ə', 'o');
    }
  } // 豪韻唇音可能也是獨立的，但爲了簡便不處理它
  if (!is`三等` && !音節.韻尾) 音節.替換('韻核', 'ə', 'o');

  // (3) 韻核咽化
  //     非三等 ɹ 實現爲 ʕ，使韻核咽化
  if (!is`三等` && 音節.介音.includes('ɹ')) {
    音節.韻核 += 'ˤ';
  }

  // (4) 省略與韻核同質的介音、韻尾
  [ // 介音, 韻核, 韻尾, 條件
    ['j', 'i', 'j', true],
    [' ', 'ɨ', 'ɨ', true],
    ['w', 'u', 'u', true],
    ['j', 'e', ' ', !is`三等`], // 非三等 j 實現爲 e̯
    ['w', 'o', ' ', !is`三等`],
    ['ɹ', 'ˤ', ' ', !is`三等`],
  ].some(e => {
    if (e[3] && 音節.韻核.endsWith(e[1])) {
      音節.替換('介音', e[0], '');
      音節.替換('韻尾', e[2], '');
      return true;
    }
  });

  // (5) 豪覃韻韻核寫作 ʌ
  if (!is`三等` && is雙唇韻尾) 音節.替換('韻核', 'ə', 'ʌ');

  // 後處理：按需顯示 ɑ，包括𦣛小韻（銳音歌三合）
  if (!is音節首含銳 || is圓唇鈍尾 || is`歌韻 三等 合口`) 音節.替換('韻核', 'a', 選項.後低元音);
}

function 調整聲母(音節) {
  const 舊版聲母字典 = [
    [!isL, { 'ᵱ': 'p', 'ᵬ': 'b', 'ᵯ': 'm' }],
    [isJ原版 || is朗讀音, { 'ʡ': 'ʔ', 'ɫ': 'l', 'x': 'h' }],
    [選項.知組 === 'tɹ', { 'ʈ': 'tɹ', 'ɖ': 'dɹ', 'ɳ': 'nɹ' }],
  ].reduce((prev, cur) => Object.assign(prev, cur[0] ? cur[1] : {}), {});

  if (Object.keys(舊版聲母字典).includes(音節.聲母[0])) {
    音節.聲母 = 舊版聲母字典[音節.聲母[0]] + 音節.聲母.slice(1);
  }
}

function 調整鈍C介音(音節) {
  if (選項.鈍C介音.includes('∅')) return;
  const [鈍C開, 鈍C唇, 鈍C合] = 選項.鈍C介音.trim().split(' ').map(e => e.slice(1));
  音節.介音 = when([
    [音節.韻核 !== 'i' && '三等 鈍音 非 云母', [
      [音節.介音 === 'w', 鈍C合],
      [音節.介音 === '', [
        ['幫組', 鈍C唇],
        [!['ɨ', 'u'].includes(音節.韻核) || '東尤韻', 鈍C開],
      ]],
    ]],
    ['', 音節.介音],
  ], '', true);
}

function 調整二等元音(音節) {
  const 默認記號 = 'ˤ';
  let 新記號 = 選項.二等元音記號?.split('◌')[1];
  if (!新記號 || 新記號 === 默認記號 || !音節.韻核.includes(默認記號)) return;

  if (新記號[0] === '˞' && is`端組 或 來母 庚韻`) 新記號 = '';
  音節.替換('韻核', 默認記號, 新記號);
  if (音節.韻核 === 'e' && !音節.韻尾) { // 箉小韻，無附加符號時改作 -aj
    音節.韻核 = 'a';
    音節.韻尾 = 'j';
  }
}

function 音值toJ(音節) {
  if (選項.鈍C介音.includes('ɥ̈') && is`三等 非 東尤韻`) 音節.替換('韻核', 'u', 'ʉ');
  if (isJ新版) return;

  if (
    音節.介音.includes('w') ||
    音節.介音 === 'ɥ̈' ||
    [...'ʉuo'].includes(音節.韻核) && is`(合口 或 鍾虞韻) 非 (幫組 或 云母)` // TODO: qieyun-js 更新後刪去虞韻
  ) {
    音節.聲母 += 'ʷ';
    音節.替換('聲母', 'ʰʷ', 'ʷʰ');
    音節.替換('聲母', 'jʷ', 'ɥ');
    音節.替換('介音', 'w', '');
    音節.替換('介音', 'j', 'ɥ');
  }

  音節.韻核 = when([
    ['蒸韻', 'i'],
    ['莊組 眞臻欣韻 開口', 'ɹ̩'],
    ['侯韻 非 明母', 'ɘu'],

    ['清韻 非 重紐B類', 'iæ'],
    ['陽韻', (is`開口 或 重紐A類` ? 'ɨ' : 'ʉ') + 'ɐ'],
    ['鍾韻', 'ʉɔ'],

    ['江韻', 'œˤ'],
    ['嚴凡韻 幫組', 'œ'],
    ['', 音節.韻核],
  ]);

  音節.介音 = when([
    [音節.聲母.includes('ʷ') && 音節.韻核[0] !== 'ʉ' && '精組 三等', 'ɹ'],
    ['精組 三等 東尤韻', 'ɹ'],
    [音節.韻核[0] === 'i' && (['j', 'ɥ'].includes(音節.介音) || '知組 或 來母'), ''],
    [[...'iɨʉ'].includes(音節.韻核[0]) && (['j̈', 'ɥ̈'].includes(音節.介音) || '銳音'), ''],
    [選項.知組 !== 'tɹ' && '知組 三等', 'ɹ'],
    ['來母 三等', 'ɹ'],
    ['', 音節.介音],
  ]);

  音節.韻尾 = {
    微: 'i', 幽: 'u',
    支: 'ɛ', 魚: 'ʌ', 虞: 'ɔ',
  }[音韻地位.韻] ?? 音節.韻尾;
}

function 音值to朗讀音(音節) {
  if (is朗讀音 && 音節.聲母.replace('ʰ', '').length > 1) {
    音節.聲母 = 音節.聲母[0] + '͡' + 音節.聲母.slice(1);
  }

  const hasW = 音節.聲母 === 'w' || 音節.介音.includes('w') || ['u', 'o'].includes(音節.韻核);
  音節.替換('聲母', 'w', '');
  音節.介音 = when([
    ['三等', [
      ['幽韻', is`幫組` ? 'j' : 'ɥ'],
      [[...'iea'].includes(音節.韻核[0]) || '蒸韻 或 重紐A類 或 重紐B類', [
        [音節.介音.includes('ɹ') || '知莊組 或 蒸韻 鈍音', hasW ? 'ɻɥ' : 'ɻj'], // B
        ['', hasW ? 'ɥ' : 'j'], // A
      ]],
      ['', is`幫組 或 尤韻` ? 'ɥ̈' : hasW ? 'ẅ' : 'j̈'], // C
    ]],
    ['二等 非 (端組 或 來母 庚韻)', is`知莊組` ? 'ɻ' + 音節.介音 : 音節.介音 + 'ɻ'],
    [音節.韻核 === 'o' && '一等 非 (冬模韻 或 幫組)', 'w'],
    ['', 音節.介音],
  ]);
  if (音節.聲母 === 'j' && ['j', 'ɥ'].includes(音節.介音)) 音節.聲母 = '';

  if (音節.韻核.includes('ˤ')) {
    音節.替換('韻核', 'eˤ', 'æ');
    音節.替換('韻核', 'oˤ', 'æ');
    音節.替換('韻核', 'aˤ', 'a');
  } else {
    音節.韻核 = when([
      ['三等', [
        ['蒸韻', 'i'],
        ['莊組 眞臻欣韻 開口', 'i˞ '],
        ['微韻', 'ɨ'],
        ['幽韻', 'ÿ'],
        [音節.韻核 === 'e' || '支清韻', 'ɛ'],
        ['魚韻', 'ə'],
        [音節.韻核 === 'o' || '虞韻', 'ɔ'],
        ['陽韻', 'ɐ'],
        ['嚴凡韻 幫組', 'ɞ'],
        ['', 音節.韻核],
      ]],
      ['侯韻 非 幫組', 'ɘu'],
      ['豪韻', 'ɑ'],
      ['覃韻 或 咍灰韻 開口', 'ɐ'],
      ['咍灰韻', 'ɔ̞'],
      [音節.韻核 === 'ə', 'ɘ'],
      ['', 音節.韻核],
    ]);
  }

  音節.韻尾 = when([
    ['通江攝', is`舒聲` ? 'ŋʷ' : 'kʷ'],
    ['宕攝', is`舒聲` ? 'ŋ' : 'k'],
    ['梗攝', is`舒聲` ? 'ɲ' : 'c'],
    ['支魚虞幽韻', ''],
    ['', 音節.韻尾],
  ]);

  [['j', 'i'], ['j̈', 'ɨ']].forEach(e => {
    if (音節.韻核[0] === e[1] && (音節.聲母 || 音節.介音[0] === 'ɻ')) 音節.替換('介音', e[0], '');
  });
}

function get音節() {
  const 音節 = {
    聲母: get聲母(),
    介音: get介音(),
    聲調: get聲調(),
    替換(propertyName, from, to) { this[propertyName] = this[propertyName].replace(from, to); }
  };
  [音節.韻核, 音節.韻尾] = get韻基();

  if (!選項.顯示純音位形式) 音位to音值(音節);
  調整聲母(音節);
  調整鈍C介音(音節);
  if (isJ新版 || isJ原版) 音值toJ(音節);
  if (is朗讀音) 音值to朗讀音(音節);
  調整二等元音(音節);

  let 聲調記號插入位置 = ['̩', '͇', '̳', 'ɘu'].some(e => 音節.韻核.includes(e)) ? 2 : 1;
  音節.韻基 = 音節.韻核 + 音節.韻尾;
  音節.帶調韻基 = 選項.聲調記號.includes('ˊ') ?
    音節.韻核.slice(0, 聲調記號插入位置) + 音節.聲調 + 音節.韻核.slice(聲調記號插入位置) + 音節.韻尾 :
    音節.韻核 + 音節.韻尾 + 音節.聲調;
  音節.韻母 = 音節.介音 + 音節.韻基;
  音節.帶調韻母 = 音節.介音 + 音節.帶調韻基;
  return 音節;
}

if (!音韻地位) return get選項列表();
調整選項();
調整音韻地位();
const 音節 = get音節();
return 音節.聲母 + 音節.帶調韻母;
});

_defaultDerivers.unt = _schemata.unt();

/**
 * unt 切韻擬音
 *
 * 包含 4 個版本：
 *
 * - 2019：切韻朗讀音
 *   https://zhuanlan.zhihu.com/p/58227457
 *
 * - 2020：切韻擬音 J（原版）
 *   https://zhuanlan.zhihu.com/p/305516512 & https://zhuanlan.zhihu.com/p/313005024
 *
 * - 2020：切韻擬音 J（2022 新版）
 *
 * - 2022：切韻擬音 L
 *   暫無專門文章，可參閱 https://zhuanlan.zhihu.com/p/541263851
 *
 * 前兩個版本已過時，僅作爲歷史存檔，不建議使用，默認不啟用。如需啟用請勾選專業模式
 *
 * 切韻擬音 J 和 L 的對比見 https://phesoca.com/comparison-of-reconstructions
 *
 * @author unt
 * @param {Qieyun.音韻地位} 音韻地位 切韻音系音韻地位
 * @param {string=} 字頭 字頭（可選）
 * @param {undefined=} 舊選項參數 已棄用，請用 `<方案名>.方案(選項)(音韻地位, 字頭?)` 代替
 * @return {string} 音韻地位對應的unt 切韻擬音
 * @property {推導方案<string>} schema 推導方案，可用於指定推導選項、獲取方案詳細資訊等
 */
export function unt(音韻地位, 字頭, 舊選項參數) {
  return _perform("unt", _defaultDerivers.unt, 音韻地位, 字頭, 舊選項參數);
}
/** @type {推導方案<string>} */
unt.schema = _schemata.unt

_schemata.msoeg_v8 = new 推導方案(function (選項, 音韻地位, 字頭) {
const is = (...x) => 音韻地位.屬於(...x);
const when = (...x) => 音韻地位.判斷(...x);
const 顯示高級選項 = 選項.顯示高級選項 ?? false;

if (!音韻地位) return [
  ['章組', [2, '齦後音', '齦腭音']],
  ['捲舌元音記號', [1, // 知乎文章用下加點，韻鑒用 r 音鉤
    'r音鉤（帶空隙）◌˞ ',
    'r音鉤（無空隙）◌˞',
    '下加點 ◌̣'
  ]],
  ['通江宕攝韻尾', [3, 'ŋ/k', 'ŋʷ/kʷ', 'ɴ/q']], // 知乎文章用 ŋʷ/kʷ，韻鑒用 ɴ/q
  ['顯示高級選項', 顯示高級選項],
].concat(顯示高級選項 ? [
  '高級選項',
  ['保留非三等ʶ記號', false],
  ['莊三介音', [2, '捲舌元音', 'ɻ']], // 知乎文章用捲舌元音，韻鑒用 ɻ
  ['覺韻', [2, '中元音', '低元音']],
  ['庚三清', [2, '中元音', '低元音']],
  ['宕攝入聲附加', [2, '無', '⁽ʷ⁾', 'ʷ']],
] : []);

if (!顯示高級選項) {
  選項.保留非三等ʶ記號 = false;
  選項.莊三介音 = 'ɻ';
  選項.覺韻 = '低元音';
  選項.庚三清 = '低元音';
  選項.宕攝入聲附加 = '⁽ʷ⁾';
}

function get聲母_默認拼寫() {
  // 五十一聲類 + 俟母
  const 普通聲母字典 = {
    幫: 'p', 滂: 'pʰ', 並: 'b', 明: 'm',
    知: 'ʈ', 徹: 'ʈʰ', 澄: 'ɖ', 孃: 'ɳ', 來: 'ɭ',
    見: 'k', 溪: 'kʰ', 羣: 'ɡ', 疑: 'ŋ', 云: 'w',
    影: 'ʔ', 曉: 'x',
    精: 'ts', 清: 'tsʰ', 從: 'dz', 心: 's', 邪: 'z',
    莊: 'tʂ', 初: 'tʂʰ', 崇: 'dʐ', 生: 'ʂ', 俟: 'ʐ',
    章: 'tç', 昌: 'tçʰ', 常: 'dʝ', 書: 'ç', 船: 'ʝ', 日: 'ɲ', 以: 'j',
  };
  const 小舌化聲母字典 = {
    幫: 'pʶ', 滂: 'pʶʰ', 並: 'bʶ', 明: 'mʶ',
    端: 'tʶ', 透: 'tʶʰ', 定: 'dʶ', 泥: 'nʶ', 來: 'lʶ',
    見: 'q', 溪: 'qʰ', 疑: 'ɴ',
    影: 'ʔʶ', 曉: 'χ', 匣: 'ʁ',
    精: 'tsʶ', 清: 'tsʶʰ', 從: 'dzʶ', 心: 'sʶ',
  };
  if (is('庚韻 二等') && 字頭 === '打') return 'tʶ';
  if (is('庚韻 二等') && 字頭 === '冷') return 'lʶ';
  if (is('云母 開口 非 侵鹽韻')) return 'ɰ';
  if (is('以母 合口 或 以母 東鍾虞韻')) return 'ɥ';
  if (is('三等 或 來母 二等')) return 普通聲母字典[音韻地位.母] || 小舌化聲母字典[音韻地位.母];
  return 小舌化聲母字典[音韻地位.母] || 普通聲母字典[音韻地位.母];
}

function get聲母() {
  let 聲母 = get聲母_默認拼寫();
  if (選項.章組 === '齦後音') {
    聲母 = 聲母.replace('ç', 'ʃ').replace('ʝ', 'ʒ');
  }
  if (!選項.保留非三等ʶ記號) {
    聲母 = 聲母.replace('ʶ', '');
  }
  return 聲母;
}

function get韻母() {
  // 爲了方便推導，對韻類稍作調整
  const 韻 = when([
    ['臻韻 或 莊組 欣韻', '眞'],
    ['文韻', '欣'],
    ['灰韻', '咍'],
    ['魂韻', '痕'],
    ['凡韻', '嚴'],
    ['東韻 三等', '終'],
    ['麻韻 三等', '遮'],
    ['歌韻 三等', '迦'],
    ['庚清韻 三等', 選項.庚三清 === '中元音' ? '淸' : '清'],
    ['庚韻 二等', ['打', '冷'].includes(字頭) ? '打' : '庚'],
    ['江韻 入聲', 選項.覺韻 === '中元音' ? '江' : '覺'],
    ['', 音韻地位.韻],
  ]);
  const 推導韻到元音 = [
    //  ŋ u ŋʷi n m
    ['　　幽　脂眞侵', 'i'],
    ['之蒸　　微欣　', 'ɯ'],
    ['　　尤終　　　', 'u'],
    ['　　侯東　　　', 'ᵒu'],

    ['支　　　　　　', 'ie'], // 知乎原文支韻在 -i 列
    ['魚　　　　　　', 'ɯɤ'],
    ['虞　　鍾　　　', 'uo'], // 知乎原文虞韻在 -u 列
    ['模　　　　　　', 'o'], // 知乎原文模韻在 -u 列

    ['　淸宵　祭仙鹽', 'iɛ'], // 用“淸”表示庚三清的中元音變體
    ['　青蕭　齊先添', 'ɛ'],
    ['佳耕　　皆山咸', 'ɜ̣'],
    ['　　　江　　　', 'ɞ̣'],
    ['　　宵　廢元嚴', 'ɯʌ'],
    ['　登　　咍痕　', 'ʌ'],
    ['　　　冬　　覃', 'ɔ'],

    ['遮清　　　　　', 'ia'],
    ['　打　　　　　', 'a'],
    ['麻庚肴　夬刪銜', 'ạ'],
    ['　　　覺　　　', 'ɑ̣'],
    ['迦　　陽　　　', 'ɯɑ'],
    ['歌　豪唐泰寒談', 'ɑ'],
  ];
  const 韻尾列表 = ['', 'ŋ/k', 'u', 選項.通江宕攝韻尾, 'i', 'n/t', 'm/p'];

  let 匹配行 = 推導韻到元音.find(e => e[0].includes(韻));
  let 元音 = 匹配行[1];
  let 韻尾 = 韻尾列表[匹配行[0].indexOf(韻)].split('/')[+is('入聲')];
  if (韻 === '覺' && !韻尾.includes('ʷ')) {
    韻尾 += 'ʷ';
  }
  if (is('宕攝 入聲') && !韻尾.includes('ʷ')) {
    韻尾 += 選項.宕攝入聲附加.replace('無', '');
  }

  // 處理三等介音
  if (元音 === 'ɯ' && is('銳音 或 蒸韻 (重紐B類 或 幫組 或 合口)')) {
    // 銳音之蒸韻歸 A 類，唇音性蒸韻下面歸 B 類
    元音 = 'i' + 元音;
  }
  if (元音.includes('i') && is('重紐B類 或 云母 或 鈍音 庚蒸韻')) {
    // 其中，支宵侵的重紐三等開口歸 C 類
    元音 = 元音.replace('i', is('支宵韻 或 侵韻 見影組 非 云母') ? 'ɯ' : 'ị');
  }
  if (is('莊組 三等')) {
    元音 = 元音[0] + '̣' + 元音.slice(1);
    if (!is('支魚韻')) {
      // 介音變爲等同於二等的 ɨ̣（支魚的第一部分不是介音）
      元音 = 元音[0].replace('ɯ', 'ɨ').replace('i', 'ɨ') + 元音.slice(1);
    }
    if (選項.莊三介音 !== '捲舌元音') {
      // 介音是 ɨ̣ 則改寫爲 ɻ，否則前加 ɻ。支韻的 ị 也改寫爲 ɻ
      元音 = 選項.莊三介音 + 元音.replace('ị', '').replace('ɨ̣', '').replace('̣', '');
    }
  }

  // 處理開合介音
  if (元音[0] === 'ɯ') {
    if (is('合口') || is('幫組 非 支宵侵韻')) {
      元音 = 元音.replace('ɯ', 'u');
    }
  } else if (is('合口 非 云以母')) {
    元音 = 'ʷ' + 元音;
    元音 = 元音.replace('ʷɻ', 'ɻʷ');
  }

  元音 = 元音.replace('̣', 選項.捲舌元音記號.split('◌')[1]);
  韻尾 = 韻尾.replace(元音.slice(-1), ''); // 刪去重複字母 ii、uu
  return 元音 + 韻尾;
}

function get聲調() {
  return { 上: 'ˀ', 去: 'ʰ' }[音韻地位.聲] || '';
}

return get聲母() + get韻母() + get聲調();
});

_defaultDerivers.msoeg_v8 = _schemata.msoeg_v8();

/**
 * msoeg 中古擬音 V8
 *
 * https://zhuanlan.zhihu.com/p/145409852
 *
 * @author unt
 * @param {Qieyun.音韻地位} 音韻地位 切韻音系音韻地位
 * @param {string=} 字頭 字頭（可選）
 * @param {undefined=} 舊選項參數 已棄用，請用 `<方案名>.方案(選項)(音韻地位, 字頭?)` 代替
 * @return {string} 音韻地位對應的msoeg 中古擬音 V8
 * @property {推導方案<string>} schema 推導方案，可用於指定推導選項、獲取方案詳細資訊等
 */
export function msoeg_v8(音韻地位, 字頭, 舊選項參數) {
  return _perform("msoeg_v8", _defaultDerivers.msoeg_v8, 音韻地位, 字頭, 舊選項參數);
}
/** @type {推導方案<string>} */
msoeg_v8.schema = _schemata.msoeg_v8

_schemata.mid_tang = new 推導方案(function (選項, 音韻地位, 字頭) {
const is = (...x) => 音韻地位.屬於(...x);
const when = (...x) => 音韻地位.判斷(...x);

const has預置風格 = !選項.預置風格 || 選項.預置風格 !== '無';
const is盛唐 = 選項.預置風格?.includes('盛唐') ?? false;
const is中唐 = 選項.預置風格?.includes('中唐') ?? true;
const is慧琳 = 選項.預置風格?.includes('慧琳') ?? true;
const is韻圖 = 選項.預置風格?.includes('韻圖') ?? false;
const 輕唇化條件 = '幫組 東鍾微虞廢文元歌陽尤凡韻 三等 非 重紐A類';

const 非組字典 = {
  'pf': { 幫: 'pf', 滂: 'pfʰ', 並: 'bv', 明: 'mʋ' },
  'f': { 幫: 'f', 滂: 'fʰ', 並: 'v', 明: 'ɱ' }, // 微母脫鼻比非母脫塞晚很多
};
const 常船母字典 = {
  '常dʑ 船ʑ': [],
  '全ʑ': [['常船母', 'ʑ'], ['俟母', 'dʐ']], // 如果不分常船，則俟母也不獨立
  '全dʑ': [['常船母', 'dʑ'], ['俟母', 'dʐ']],
  '平dʑ 仄ʑ': [['常船母', [['平聲', 'dʑ'], ['', 'ʑ']]], ['俟母', 'dʐ']],
};

if (!音韻地位) return [
  ['預置風格', [3,
    '無',
    '盛唐（保守）',
    '中唐·慧琳反切',
    '中唐·韻圖',
  ]],

  '聲',
  ['非組', [1].concat(Object.keys(非組字典))],
  ['莊章組', has預置風格 ? null : [1,
    '莊tʂ二等 章tɕ四等',
    '莊tɕ二等 章tɕ三等', // 韻圖風格
  ]],
  ['常船母', [1].concat(Object.keys(常船母字典).slice(is慧琳 ? 1 : 0))
    .map(e => is慧琳 ? e + ' ' : e) // 加空格是爲了在切換預置風格時刷新選中的內容
  ],
  // TODO: 鼻音塞化（暫時不實現）

  '韻',
  ['二等介音', [1, '省略', 'ʕ']], // 如需其他寫法請自行修改
  ['入聲韻尾', [1, 'p t k', 'p ɾ k', 'β ɾ ɣ']],
  ['部分蟹攝二等入假攝', true],
  ['部分流攝脣音入遇攝', true],
  // ['遇攝用半高元音', true], // 模韻介於 u、o 之間，默認寫 u
  ['BC韻合流', has預置風格 ? null : true],
  ['清青合併', has預置風格 ? null : true],
  ['咍泰合併', has預置風格 ? null : true], // 盛唐可能還沒合併

  '調',
  ['聲調', [1, '五度符號', '附加符號', '調類數字']],
  ['全濁上歸去', has預置風格 && !is韻圖 ? null : true],
  '', // 在 v0.1 推導器中使「恢復預設值」按鈕位於下一行
];

選項.常船母 = 選項.常船母.trim();
if (has預置風格) {
  選項.BC韻合流 = is中唐;
  選項.清青合併 = is中唐;
  選項.咍泰合併 = is中唐;
  選項.莊章組 = is韻圖 ? '莊tɕ二等 章tɕ三等' : '莊tʂ二等 章tɕ四等';
  if (!is韻圖) 選項.全濁上歸去 = is中唐;
}

function 調整音韻地位() {
  function 調整(表達式, 調整屬性) { if (is(表達式)) 音韻地位 = 音韻地位.調整(調整屬性); }
  // 輕唇化例外
  調整('明母 尤韻', { 等: '一', 韻: '侯' });
  調整('明母 東韻', { 等: '一' });

  // [慧琳反切體現的, 唐代用韻體現的, 據今音推測的]
  const 蟹攝二等入假攝字 = ['崖咼(呙)扠涯搋派差絓畫(画)罣罷(罢)', '佳鼃娃解釵(钗)卦柴', '哇洼蛙灑蝸話(话)掛挂查叉杈衩'].join('');
  const 流攝脣音入遇攝字 = ['浮戊母罦罘蜉矛茂覆懋拇某負(负)阜', '謀(谋)部畝(亩)畮婦(妇)不否桴富牟缶', '復複(复)副牡'].join('');
  if (選項.部分蟹攝二等入假攝 && 蟹攝二等入假攝字.includes(字頭)) 調整('蟹攝 二等', { 韻: '麻' });
  if (選項.部分流攝脣音入遇攝 && 流攝脣音入遇攝字.includes(字頭)) 調整('幫組 尤侯韻', { 韻: is`尤韻` ? '虞' : '模' });

  if (選項.全濁上歸去) 調整('全濁 上聲', { 聲: '去' });
}

function get聲母() {
  let 聲母 = when([
    [輕唇化條件, 非組字典[選項.非組][音韻地位.母]],
    ...常船母字典[選項.常船母],
    ['', {
      // 儘管全濁聲母在盛中唐在音值上已經弛化乃至清化，但是仍可使用濁音符號
      // 鼻音塞化也不需要寫出來，而且它只是長安音的特色
      幫: 'p', 滂: 'pʰ', 並: 'b', 明: 'm',
      端: 't', 透: 'tʰ', 定: 'd', 泥: 'n', 來: 'l',
      知: 'ʈ', 徹: 'ʈʰ', 澄: 'ɖ', 孃: 'ɳ',
      精: 'ts', 清: 'tsʰ', 從: 'dz', 心: 's', 邪: 'z',
      莊: 'tʂ', 初: 'tʂʰ', 崇: 'dʐ', 生: 'ʂ', 俟: 'ʐ',
      章: 'tɕ', 昌: 'tɕʰ', 常: 'dʑ', 書: 'ɕ', 船: 'ʑ', 日: 'ɲ', 以: 'j',
      見: 'k', 溪: 'kʰ', 羣: 'ɡ', 疑: 'ŋ',
      影: 'ʔ', 曉: 'x', 匣: 'ɣ', 云: '',
    }[音韻地位.母]],
  ]);
  if (!選項.莊章組.includes('ʂ')) {
    聲母 = 聲母.replace('ʂ', 'ɕ').replace('ʐ', 'ʑ');
    聲母 = 聲母.replace('j', ''); // 以云也合爲喻母
  }
  return 聲母;
}

function get韻母() {
  let 韻圖等 = when([
    ['精組 止攝 開口', '一'],
    ['莊組', '二'],
    ['章組 或 日母', 選項.莊章組.includes('四') ? '四' : '三'],
    ['非 三等', 音韻地位.等], // 切韻一二四等到韻圖不變

    [輕唇化條件, '輕'], // 輕唇算作一個獨立的等
    ['以母 或 重紐A類 或 鈍音 麻清韻 或 見影組 幽韻', '四'], // 鈍音切韻三 A
    ['端精組', '四'],
    ['', '三'] // 其他切韻三等到韻圖不變
  ]);

  let 韻母列表 = when([
    // 分開合的韻，用豎線分隔開口韻母和合口韻母，之後篩選
    // 輕唇韻母同下一行的韻母去掉介音
    ['遇攝', {
      輕: 'u',
      一: 'u',
      二: 'ʕɯ|ʕu',
      三: 'ɨɯ|ʉu',
      四: 'iɯ|yu'
    }],
    ['果假攝', {
      輕: 'uɒ',
      一: 'ɑ|uɒ',
      二: 'ʕa|ʕua',
      三: 'ɨɑ|ʉɒ',
      四: 'ia|ya'
    }],

    ['曾攝', {
      一: 'əŋ|uəŋ',
      二: 'ʕəŋ|ʕuəŋ',
      三: 'ɨŋ|ʉɨŋ',
      四: 'iɨŋ|yɨŋ',
    }],
    ['梗攝', {
      二: 'ʕɛŋ|ʕuɛŋ',
      三: 'ɨɛŋ|ʉɛŋ', // 清青合併前，庚三和庚二是相同的元音
      四: 選項.清青合併 || !is`青韻` ? 'iɛŋ|yɛŋ' : 'ieŋ|yeŋ',
    }],

    ['通攝', {
      一: 'oŋ',
      二: 'ʕoŋ',
      輕: is`東韻` ? 'uŋ' : 'oŋ',
      三: is`東韻` ? 'ɨuŋ' : 'ʉoŋ',
      四: is`東韻` ? 'iuŋ' : 'yoŋ',
    }],
    ['江攝', {
      二: 'ʕœŋ',
    }],
    ['宕攝', {
      輕: 'ɑŋ',
      一: 'ɑŋ|uɑŋ',
      二: 'ʕɑŋ|ʕuɑŋ',
      三: 'ɨɑŋ|ʉɑŋ',
      四: 'iɑŋ|yɑŋ',
    }],

    ['止攝', {
      一: 'ɹ̩',
      二: 'ʕɹ̩|ʕuɹ̩',
      輕: 選項.BC韻合流 ? 'ʉi' : 'ʉj',
      三: 選項.BC韻合流 || !is`微韻` ? 'ɨi|ʉi' : 'ɨj|ʉj',
      四: 'i|yi',
    }],
    ['蟹攝', {
      一: 選項.咍泰合併 ? 'ɑj|uɔj' : is`泰韻` ? 'ɑj|uɑj' : 'ʌj|uɔj',
      二: 'ʕaj|ʕuaj',
      三: 選項.BC韻合流 && is`見影組` || is`廢韻` ? 'ɨɜj|ʉɜj' : 'ɨɛj|ʉɛj',
      輕: 'iɛj',
      四: 'iɛj|yɛj',
    }],

    ['臻攝 非 元韻', {
      一: 'ən|un',
      二: 'ʕən|ʕun',
      輕: 'ʉn',
      三: 選項.BC韻合流 && is`見影組` || is`欣文韻` ? 'ɨn|ʉn' : 'ɨin|ʉin',
      四: 'in|yn',
    }],
    ['山攝 或 元韻', {
      一: 'ɑn|uɒn',
      二: 'ʕan|ʕuan',
      輕: 'ɜn',
      三: 選項.BC韻合流 && is`見影組` || is`元韻` ? 'ɨɜn|ʉɜn' : 'ɨɛn|ʉɛn',
      四: 'iɛn|yɛn',
    }],

    ['流攝', {
      輕: 'əw',
      一: 'əw',
      二: 'ʕəw',
      三: 'ɨw',
      四: 'iw',
    }],
    ['效攝', {
      一: 'ʌw',
      二: 'ʕaw',
      三: 'ɨɛw',
      四: 'iɛw',
    }],

    ['深攝', {
      二: 'ʕəm',
      三: 選項.BC韻合流 && is`見影組` ? 'ɨm' : 'ɨim',
      四: 'im',
    }],
    ['咸攝', {
      一: 'ɑm',
      二: 'ʕam',
      輕: 'ɜm',
      三: 選項.BC韻合流 && is`見影組` || is`嚴凡韻` ? 'ɨɜm' : 'ɨɛm',
      四: 'iɛm',
    }],
  ])[韻圖等].split('|');

  let 開合index = when([
    [韻母列表.length === 1, 0],
    ['虞韻 或 幫組 一等 非 曾宕攝', 1],
    ['開口 或 幫組', 0],
    ['合口', 1],
  ]);

  let 韻母 = 韻母列表[開合index];
  if (is`入聲`) {
    選項.入聲韻尾.split(' ').forEach((e, i) => { 韻母 = 韻母.replace('mnŋ'[i], e); });
  }

  韻母 = 韻母.replace('ʕ', 選項.二等介音.replace('省略', ''));

  if (選項.遇攝用半高元音 && ['ɯ', 'u'].includes(韻母.slice(-1))) {
    韻母 = 韻母.replace('ɯ', 'ɤ').replace('u', 'o');
  }
  return 韻母;
}

function get聲調() {
  const is陰 = is`全清 或 次清 或 次濁 上聲`;
  return {
    五度符號: is陰 ?
      { 平: '˦˨', 上: '˦˥', 去: '˧˨˦', 入: '˦' } :
      { 平: '˨˩', 上: '˨˦', 去: '˨˨˦', 入: '˨' },
    附加符號: { 平: '̀', 上: '́', 去: '̌', 入: '' },
    調類數字: { 平: '¹', 上: '²', 去: '³', 入: '⁴' },
  }[選項.聲調][音韻地位.聲];
}

function get音節() {
  const 音節 = {
    聲母: get聲母(),
    韻母: get韻母(),
    聲調: get聲調(),
  };
  const 韻尾列表 = [...'ŋjnwm', ...選項.入聲韻尾.split(' ')];
  音節.帶調韻母 = 選項.聲調 === '附加符號' && 韻尾列表.includes(音節.韻母.slice(-1)) ?
    音節.韻母.slice(0, -1) + 音節.聲調 + 音節.韻母.slice(-1) :
    音節.韻母 + 音節.聲調;
  return 音節;
}

調整音韻地位();
const 音節 = get音節();
return 音節.聲母 + 音節.帶調韻母;
});

_defaultDerivers.mid_tang = _schemata.mid_tang();

/**
 * 推導盛中唐擬音
 *
 * 中古漢語音變匯總：https://zhuanlan.zhihu.com/p/403479390
 * 本擬音是根據南北朝以來的音變，從《切韻》音系推出的。盛中唐期間的一些音變，以及擬音和記音方面的多種可能，均以選項形式列出
 *
 * 預置風格說明：
 * - 盛唐(保守)：區分BC韻、清青、咍泰；章組歸在四等，區分常船；濁上獨立
 * - 中唐 慧琳 ：不分BC韻、清青、咍泰；章組歸在四等，不分常船；濁上歸去
 * - 中唐 韻圖 ：不分BC韻、清青、咍泰；莊章合爲照組，區分常船
 *
 * 擬音依據包括：
 * - 韻圖
 * - 玄應、慧琳音義和各家反切
 * - 詩歌碑誌銘文用韻
 * - 日、韓、越漢字音
 * - 梵語、藏語、突厥語、粟特語、回鶻語譯音
 * - 關於語音不正的記載和笑話
 *
 * @author unt
 * @param {Qieyun.音韻地位} 音韻地位 切韻音系音韻地位
 * @param {string=} 字頭 字頭（可選）
 * @param {undefined=} 舊選項參數 已棄用，請用 `<方案名>.方案(選項)(音韻地位, 字頭?)` 代替
 * @return {string} 音韻地位對應的推導盛中唐擬音
 * @property {推導方案<string>} schema 推導方案，可用於指定推導選項、獲取方案詳細資訊等
 */
export function mid_tang(音韻地位, 字頭, 舊選項參數) {
  return _perform("mid_tang", _defaultDerivers.mid_tang, 音韻地位, 字頭, 舊選項參數);
}
/** @type {推導方案<string>} */
mid_tang.schema = _schemata.mid_tang

_schemata.chiangxhua = new 推導方案(function (選項, 音韻地位, 字頭) {
const is = (...x) => 音韻地位.屬於(...x);
const when = (...x) => 音韻地位.判斷(...x);
const 顯示高級選項 = 選項.顯示高級選項 ?? false;

if (!音韻地位) return [
  ['顯示', [4,
    '音位，入聲用陰聲尾（僅作爲原圖轉寫，不推薦）',
    '音位，入聲用入聲尾',
    '音值，保留二等介音，知照組用齦腭音',
    '音值，省略二等介音，知照組用齦腭音',
    '音值，省略二等介音，知照組用捲舌音',
    '音值，省略二等介音，知照組二等捲舌、三等齦腭',
  ]],
  ['入聲尾', [1,
    '默認',
    'ʋ ɹ ɣ',
    'β ɾ ɣ',
    'b d ɡ',
    'p t k',
  ]],
  ['聲調', [1, '附加符號', '調類數字']],
  ['全濁上歸去', true], // 圖中未體現全濁上字的聲調，按口語則已歸去聲，按《蒙古字韻》風格則仍算上聲
  ['影喻上聲合併', false], // 圖中無影喻上聲對立的空間，可能口語已合併，但同期反切沒有相混的情況
  ['顯示高級選項', 顯示高級選項],
].concat(顯示高級選項 ? [
  '高級選項',
  ['部分蟹攝二等入假攝', false],
  ['部分流攝脣音入遇攝', false],
  ['次濁上省略喉化', false],
  ['全濁平省略送氣', false],
  ['山攝二等脣音歸合口', false], // （僅作爲原圖轉寫）圖中“八”在合口，但二等不應該在合口。《韻鏡》《七音略》脣音刪舒、山入在合口，刪入、山舒在開口
] : []);

function 調整音韻地位() {
  function 調整(表達式, 調整屬性) { if (is(表達式)) 音韻地位 = 音韻地位.調整(調整屬性); }
  // 輕唇化例外
  調整('明母 尤韻', { 等: '一', 韻: '侯' });
  調整('明母 東韻', { 等: '一' });

  // [慧琳反切體現的, 唐代用韻體現的, 據今音推測的]
  const 蟹攝二等入假攝字 = ['崖咼(呙)扠涯搋派差絓畫(画)罣罷(罢)', '佳鼃娃解釵(钗)卦柴', '哇洼蛙灑蝸話(话)掛挂查叉杈衩'].join('');
  const 流攝脣音入遇攝字 = ['浮戊母罦罘蜉矛茂覆懋拇某負(负)阜', '謀(谋)部畝(亩)畮婦(妇)不否桴富牟缶', '復複(复)副牡'].join('');
  if (選項.部分蟹攝二等入假攝 && 蟹攝二等入假攝字.includes(字頭)) 調整('蟹攝 二等', { 韻: '麻' });
  if (選項.部分流攝脣音入遇攝 && 流攝脣音入遇攝字.includes(字頭)) 調整('幫組 尤侯韻', { 韻: is`尤韻` ? '虞' : '模' });
}

function get聲母() {
  return when([
    ['云母 通攝 平聲', 'ɣ'], // 云母“雄”在二音
    ['匣母 肴韻 平聲', ''], // 匣母“爻”在三音，但一等無字，說明“完”不是零聲母
    ['崇母 止攝 仄聲', 'ʑ'], // 崇母“士”在十音。另船母船小韻和繩小韻中“乘”聲符字今讀塞擦音，但圖中未體現，這裏不考慮
    [選項.影喻上聲合併 && '云以母 上聲', 'ʔ'], // 可能的情況
    // 以上爲特例

    ['東鍾微虞廢文元歌陽尤凡韻 三等 非 重紐A類', [ // “非 重紐A類”用於過濾𩦠小韻，“歌韻”用於包含縛小韻
      ['幫滂母', 'f'], ['並母', 'v'], ['明母 上聲', 'ˀʋ'], ['明母', 'ʋ'], // 四音
    ]],
    ['曉母', 'x'], ['匣母', 'ɣ'], ['疑母 上聲', 'ˀŋ'], ['疑母', 'ŋ'], // 二音
    ['心母', 's'], ['邪母', 'z'], // 九音。這一組的後兩個聲母（同部位的近音）有音無字
    ['生書母', 'ɕ'], ['常母 仄聲 或 俟船母', 'ʑ'], ['日母 上聲', 'ˀɹ'], ['日母', 'ɹ'], // 十音

    ['幫母', 'p'], ['並母 仄聲', 'b'], ['滂母', 'pʰ'], ['並母', 'bʰ'], // 五音
    ['端母', 't'], ['定母 仄聲', 'd'], ['透母', 'tʰ'], ['定母', 'dʰ'], // 六音
    ['見母', 'k'], ['羣母 仄聲', 'ɡ'], ['溪母', 'kʰ'], ['羣母', 'ɡʰ'], // 一音
    ['知母', 'tɹ'], ['澄母 仄聲', 'dɹ'], ['徹母', 'tɹʰ'], ['澄母', 'dɹʰ'], // 十二音
    ['精母', 'ts'], ['從母 仄聲', 'dz'], ['清母', 'tsʰ'], ['從母', 'dzʰ'], // 八音
    ['莊章母', 'tɕ'], ['崇母 仄聲', 'dʑ'], ['初昌母', 'tɕʰ'], ['崇常母', 'dʑʰ'], // 十一音

    ['泥孃母 上聲', 'ˀn'], ['泥孃母', 'n'], ['來母 上聲', 'ˀl'], ['來母', 'l'], // 七音
    ['影母', 'ʔ'], ['云以母', is`上聲` ? 'ˀ' : ''], ['明母 上聲', 'ˀm'], ['明母', 'm'], // 三音
  ], '無聲母規則', true);
}

function get等() {
  return when([
    ['以母 或 重紐A類 或 鈍音 麻清幽韻 三等', '四'], // 幽韻按韻圖而非反切，一律歸四等
    ['幫組 三等', [ // 輕脣化的韻
      ['微廢韻', '四'],
      ['元凡韻', '二'],
      ['東鍾虞文歌陽尤韻', '一'],
    ]],
    ['銳音', [
      ['精組 止攝 開口', '一'], // 中唐發生 /siə/ > /sə/，變一等
      ['莊組', '二'],
      ['四等', '三'], // 圖中銳音一律無四等
      ['蟹山咸攝 一等 開口 非 (泥母 蟹攝)', '二'], // 圖中這類韻列在二等，但“乃”（泥開一咍上）仍在一等
    ]],
    ['', 音韻地位.等],
  ], '無等規則', true);
}

function get開合() {
  return when([
    ['流深咸攝', '開'], // 深咸攝舒入聲都視爲開口，不依圖中定義
    ['幫組', [
      ['通宕攝', '開'],
      ['一等 或 虞文歌韻', '合'], // 其餘一等（包括輕脣變一等的）歸合口
      ['山攝 二等 或 元韻', 選項.山攝二等脣音歸合口 ? '合' : '開'],
    ]],
    ['鍾虞模韻', '合'],
    ['江韻 銳音', '合'],
    ['', 音韻地位.呼 ?? '開'],
  ], '無開合規則', true);
}

function get介音(等, 開合) {
  return {
    一: { 開: '', 合: 'u' },
    二: { 開: 'ʕ', 合: 'ʕu' },
    三: { 開: 'i', 合: 'iu' },
    四: { 開: 'ʲi', 合: 'ʲiu' },
  }[等][開合];
}

function get韻基() {
  return when([
    ['蟹攝 (一二等 或 莊組)', 'aj'], // 一聲下(僅舒聲)

    ['止蟹攝 合口 非 莊組', 'əj'], // 五聲下(舒)
    ['曾梗攝', is`入聲` ? 'əɣ' : 'əŋ'], // 五聲下(入) & 二聲下

    ['止蟹攝', 'ə'], // 五聲上(舒)
    ['臻攝 非 元韻', is`入聲` ? 'əɾ' : 'ən'], // 五聲上(入) & 三聲下

    ['果假攝', 'a'], // 一聲上(舒)
    ['山攝 或 元韻', is`入聲` ? 'aɾ' : 'an'],// 一聲上(入) & 三聲上

    ['遇攝', 'ɯ'], // 六聲下(僅舒聲)

    ['流攝', 'əw'], // 四聲下(舒)
    ['通攝', is`入聲` ? 'əwɣ' : 'əwŋ'], // 四聲下(入) & 六聲上

    ['效攝', 'aw'], // 四聲上(舒)
    ['宕江攝', is`入聲` ? 'aɣ' : 'aŋ'], // 四聲上(入) & 二聲上

    ['深攝', is`入聲` ? 'əβ' : 'əm'], // 七聲上
    ['咸攝', is`入聲` ? 'aβ' : 'am'], // 七聲下
  ], '無韻基規則');
}

function get聲調() {
  return {
    平: { 附加符號: '̀', 調類數字: '¹' },
    上: { 附加符號: '́', 調類數字: '²' },
    去: { 附加符號: '̌', 調類數字: '³' },
    入: { 附加符號: '', 調類數字: '⁴' }, // 入聲已由韻尾表明，無需附加符號
  }[選項.全濁上歸去 && is`全濁 上聲` ? '去' : 音韻地位.聲][選項.聲調];
}

function 音位to音值(音節) {
  function 替換韻核(from, tos, condition = true) {
    if (!condition) return;
    音節.韻核 = 音節.韻核.replace(from, 音節.介音.includes('u') ? tos.pop() : tos[0]);
  }

  const is收ŋ韻 = ['ŋ', 'ɣ'].includes(音節.韻尾);
  替換韻核('a', ['ɑ']); // 先把 /a/ 重置成一等的音值 [ɑ]
  if (音節.介音.includes('ʕ')) {
    // 二等
    替換韻核('ɯ', ['ɯ', '']);
    替換韻核('ə', ['ɛ'], is收ŋ韻);
    替換韻核('ɑ', ['a']);
  } else if (音節.介音.includes('i')) {
    // 三四等
    替換韻核('ɯ', ['ɯ', 'u']);
    替換韻核('ə', ['']); // 韻核和介音相同，省略韻核
    替換韻核('ɑ', ['æ', 'ɐ'], !音節.韻尾); // 北宋時麻二麻三未有明確分開的跡象，暫擬作 [iæ]
    替換韻核('ɑ', ['ɛ', 'ɜ'], !is收ŋ韻); // 宕三保留在 [ɑ]
  } else {
    // 一等
    替換韻核('ɯ', ['ɯ', '']);
    替換韻核('ə', ['ə', ''], !is收ŋ韻); // [uəŋ] 不寫作 [uŋ]，以免與通攝混淆
    替換韻核('ɑ', ['ʌ', 'ɔ'], !音節.韻尾);
    替換韻核('ɑ', ['ɑ', 'ɔ'], !is收ŋ韻);
    替換韻核('ɔ', ['o'], 音節.韻尾 === 'j'); // /uɑj/ 離 [i] 更近
  }
  if (is`精莊章組` && 音節.韻核 === 'ə' && !音節.韻尾) {
    // 噝音後的開音節 /ə/ 變 [ɹ̩]（實際僅精組開口和莊組有字）
    音節.介音.includes('u') ? 音節.韻尾 = 'ɹ' : 音節.韻核 = 'ɹ̩';
  }
  if (選項.顯示.includes('知照組用捲舌音') ||
    選項.顯示.includes('知照組二等捲舌') && 音節.介音.includes('ʕ')) {
    [...'ɕʑɹ'].forEach((e, i) => { 音節.聲母 = 音節.聲母.replace(e, 'ʂʐɻ'[i]); });
    if (音節.介音.includes('ʕ')) {
      音節.韻核 = 音節.韻核.replace('ɹ̩', 'ɻ̍');
      音節.韻尾 = 音節.韻尾.replace('ɹ', 'ɻ');
    }
  }
  if (選項.顯示.includes('省略二等介音')) 音節.介音 = 音節.介音.replace('ʕ', '');
  音節.介音 = 音節.介音.replace('iu', 'ʉ');
  if (!音節.韻核) 音節.韻尾 = 音節.韻尾.replace('j', 'i'); // /iuəj/ 實際上是 [i] 的合口
  音節.韻尾 = 音節.韻尾.replace('wŋ', 'uŋ');
  音節.韻尾 = 音節.韻尾.replace('wɣ', 'uɣ');
}

function 後處理(音節) {
  if (音節.介音.includes('ʲ')) {
    音節.聲母 += ['', 'ˀ'].includes(音節.聲母) ? 'j' : 'ʲ';
    音節.聲母 = 音節.聲母.replace('ʰʲ', 'ʲʰ');
    音節.介音 = 音節.介音.replace('ʲ', '');
  }
  if (選項.次濁上省略喉化) 音節.聲母 = 音節.聲母.replace('ˀ', '');
  if (選項.全濁平省略送氣 && is`全濁`) 音節.聲母 = 音節.聲母.replace('ʰ', '');
  if (is`入聲`) {
    let 入聲尾 = 選項.入聲尾;
    if (入聲尾 === '默認') 入聲尾 = 選項.顯示.includes('音值') ? 'β ɾ ɣ' : 'p t k';
    if (選項.顯示.includes('入聲用陰聲尾')) 入聲尾 = 'ˀ ʋˀ jˀ wˀ';
    const 韻尾to = 入聲尾.split(' ');
    const 韻尾from = 韻尾to.length === 3 ? [...'βɾɣ'] : ['ɾ', 'β', is`曾梗攝` && 'ɣ', /.?ɣ/];
    韻尾from.forEach((e, i) => { 音節.韻尾 = 音節.韻尾.replace(e, 韻尾to[i]); });
  }
}

function get音節() {
  const 韻基 = get韻基();
  const 音節 = {
    聲母: get聲母(),
    介音: get介音(get等(), get開合()),
    韻核: 韻基[0],
    韻尾: 韻基.substring(1),
    聲調: get聲調(),
  };
  if (選項.顯示.includes('音值')) 音位to音值(音節);
  後處理(音節);
  音節.韻母 = 音節.介音 + 音節.韻核 + 音節.韻尾;
  音節.帶調韻母 = 音節.介音 + 音節.韻核 + (選項.聲調 === '調類數字' ? 音節.韻尾 + 音節.聲調 : 音節.聲調 + 音節.韻尾);
  return 音節;
}

調整音韻地位();
const 音節 = get音節();
return 音節.聲母 + 音節.帶調韻母;
});

_defaultDerivers.chiangxhua = _schemata.chiangxhua();

/**
 * 推導《聲音唱和圖》擬音
 *
 * https://zhuanlan.zhihu.com/p/498778513
 *
 * 默認顯示音值，採用北宋早期音系通常拼寫風格
 * 默認入聲尾音位用 /-p -t -k/，音值用 [-β -ɾ -ɣ]
 * 如需顯示原圖的音位轉寫，請在「顯示」選擇「音位，入聲用陰聲尾」
 *
 * 代碼注釋中的等均指韻圖等（聲音唱和圖等），而不是切韻等
 *
 * @author unt
 * @param {Qieyun.音韻地位} 音韻地位 切韻音系音韻地位
 * @param {string=} 字頭 字頭（可選）
 * @param {undefined=} 舊選項參數 已棄用，請用 `<方案名>.方案(選項)(音韻地位, 字頭?)` 代替
 * @return {string} 音韻地位對應的推導《聲音唱和圖》擬音
 * @property {推導方案<string>} schema 推導方案，可用於指定推導選項、獲取方案詳細資訊等
 */
export function chiangxhua(音韻地位, 字頭, 舊選項參數) {
  return _perform("chiangxhua", _defaultDerivers.chiangxhua, 音韻地位, 字頭, 舊選項參數);
}
/** @type {推導方案<string>} */
chiangxhua.schema = _schemata.chiangxhua

_schemata.fanwan = new 推導方案(function (選項, 音韻地位, 字頭) {
if (!音韻地位) return [['$legacy', true]];

const is = (x) => 音韻地位.屬於(x);

function 聲母規則() {
  if (is('幫母')) {
    if (is('東韻 三等 或 鍾微虞廢文元陽尤凡韻')) return 'f';
    return 'b';
  }
  if (is('滂母')) {
    if (is('東韻 三等 或 鍾微虞廢文元陽尤凡韻')) return 'f';
    return 'p';
  }
  if (is('並母')) {
    if (is('東韻 三等 或 鍾微虞廢文元陽尤凡韻')) return 'f';
    return is('平聲') ? 'p' : 'b';
  }
  if (is('明母')) return 'm';

  if (is('端母')) return 'd';
  if (is('透母')) return 't';
  if (is('定母')) return is('平聲') ? 't' : 'd';
  if (is('泥母')) return 'n';
  if (is('來母')) return 'l';

  if (is('知母')) return 'zh';
  if (is('徹母')) return 'ch';
  if (is('澄母')) return is('平聲') ? 'ch' : 'zh';
  if (is('孃母')) return 'n';

  if (is('精母')) return 'z';
  if (is('清母')) return 'c';
  if (is('從母')) return is('平聲') ? 'c' : 'z';
  if (is('心母')) return 's';
  if (is('邪母')) return is('平聲') ? 'c' : 'z'; // 塞擦音多於擦音

  if (is('莊母 二等')) return 'zh';
  if (is('初母 二等')) return 'ch';
  if (is('崇母 二等')) return is('平聲') ? 'ch' : 'zh';
  if (is('生母 二等')) return 'sh';
  if (is('俟母 二等')) return is('平聲') ? 'ch' : 'zh';

  // 莊組三等平翹音規律不明確
  if (is('莊母')) return 'z';
  if (is('初母')) return 'c';
  if (is('崇母')) return is('平聲') ? 'c' : 'z';
  if (is('生母')) return 's';
  if (is('俟母')) return is('平聲') ? 'c' : 'z';

  if (is('章母')) return 'zh';
  if (is('昌母')) return 'ch';
  if (is('常母')) return 'sh'; // 擦音多於塞擦音
  if (is('書母')) return 'sh';
  if (is('船母')) return 'sh';
  if (is('日母')) return 'nj';

  if (is('見母')) return 'g';
  if (is('溪母')) return 'h'; // 多數擦化
  if (is('羣母')) return is('平聲') ? 'k' : 'g';
  if (is('疑母')) return 'ng'; // ng 拼細音時為 nj，詳後

  if (is('曉母')) return 'h';
  if (is('匣母')) {
    if (is('合口 或 模韻')) return 'j'; // 非 yu 前為 w，詳後
    return 'h';
  }
  if (is('影云以母')) {
    if (is('三四等')) return 'j'; // 非 yu 前為 w，詳後
    return '';
  }

  throw new Error('無聲母規則');
}

function 韻母規則() {
  // 通攝
  if (is('東冬鍾韻')) return 'ung';

  // 江攝
  if (is('江韻 幫組')) return 'ong';
  if (is('江韻 舌齒音')) return 'oeng';
  if (is('江韻 牙喉音')) return 'ong';

  // 止攝
  if (is('支脂之微韻 幫組')) return 'i';
  if (is('支脂之微韻 開口')) return 'i'; // i 在 z/c/s 前為 ii，詳後
  if (is('支脂之微韻 合口 舌齒音')) return 'eoi';
  if (is('支脂之微韻 合口 牙喉音')) return 'ai';

  // 遇攝
  if (is('魚虞韻 幫組')) return 'u';
  if (is('魚虞韻 莊組')) return 'o';
  if (is('魚虞韻')) return 'yu';
  if (is('模韻 疑母')) return '';
  if (is('模韻')) return 'u';

  // 蟹攝
  if (is('齊韻')) return 'ai';
  if (is('祭韻 幫組')) return 'ai';
  if (is('祭韻 開口')) return 'ai';
  if (is('祭韻 合口 舌齒音')) return 'eoi';
  if (is('祭韻 合口 以母')) return 'eoi';
  if (is('祭韻 合口 牙喉音')) return 'ai';
  if (is('泰韻 幫組')) return 'ui';
  if (is('泰韻 開口 端組')) return 'aai';
  if (is('泰韻 開口 來母')) return 'aai';
  if (is('泰韻 開口 精組')) return 'oi';
  if (is('泰韻 開口 牙喉音')) return 'oi';
  if (is('泰韻 合口 舌齒音')) return 'eoi';
  if (is('泰韻 合口 疑母')) return 'oi';
  if (is('泰韻 合口 牙喉音')) return 'ui';
  if (is('佳皆夬韻')) return 'aai';
  if (is('灰韻 疑母')) return 'oi';
  if (is('灰韻')) return 'ui';
  if (is('咍韻')) return 'oi';
  if (is('廢韻')) return 'ai';

  // 臻攝
  if (is('眞韻 幫組')) return 'an';
  if (is('眞韻 開口')) return 'an';
  if (is('眞韻 合口 舌齒音')) return 'eon';
  if (is('眞韻 合口 牙喉音')) return 'an';
  if (is('臻文欣韻')) return 'an';
  if (is('元韻 幫組')) return 'aan';
  if (is('元韻 開口')) return 'in';
  if (is('元韻 合口')) return 'yun';
  if (is('魂韻 幫組')) return 'un';
  if (is('魂韻 端組')) return 'eon';
  if (is('魂韻 來母')) return 'eon';
  if (is('魂韻 精組')) return 'yun';
  if (is('魂韻 牙喉音')) return 'an';
  if (is('痕韻')) return 'an';

  // 山攝
  if (is('寒韻 幫組')) return 'un';
  if (is('寒韻 開口 舌齒音')) return 'aan';
  if (is('寒韻 開口 牙喉音')) return 'on';
  if (is('寒韻 合口 舌齒音')) return 'yun';
  if (is('寒韻 合口 牙喉音')) return 'un';
  if (is('刪山韻')) return 'aan';
  if (is('仙先韻 幫組')) return 'in';
  if (is('仙先韻 開口')) return 'in';
  if (is('仙先韻 合口')) return 'yun';

  // 效攝
  if (is('蕭宵韻')) return 'iu';
  if (is('肴韻')) return 'aau';
  if (is('豪韻')) return 'u';

  // 果攝
  if (is('歌韻 一等')) return 'o';
  if (is('歌韻 三等')) return 'e';

  // 假攝
  if (is('麻韻 二等')) return 'aa';
  if (is('麻韻 三等')) return 'e';

  // 宕攝
  if (is('陽韻 幫組')) return 'ong';
  if (is('陽韻 開口 莊組')) return 'ong';
  if (is('陽韻 開口')) return 'oeng';
  if (is('陽韻 合口')) return 'ong';
  if (is('唐韻')) return 'ong';

  // 梗攝
  if (is('庚韻 二等')) return 'ang';
  if (is('庚韻 三等 莊組')) return 'ang';
  if (is('庚韻 三等')) return 'ing';
  if (is('耕韻')) return 'ang';
  if (is('清青韻')) return 'ing';

  // 曾攝
  if (is('蒸韻')) return 'ing';
  if (is('登韻')) return 'ang';

  // 流攝
  if (is('尤侯幽韻')) return 'au';

  // 深攝
  if (is('侵韻')) return 'am'; // m 韻尾在聲母為脣音時為 n，詳後，下同

  // 咸攝
  if (is('覃談韻 幫組')) return 'aam';
  if (is('覃談韻 舌齒音')) return 'aam';
  if (is('覃談韻 牙喉音')) return 'om';
  if (is('鹽添嚴韻')) return 'im';
  if (is('咸銜凡韻')) return 'aam';

  throw new Error('無韻母規則');
}

function 聲調規則() {
  if (is('全清 或 次清')) {
    if (is('平聲')) return '1'; // 陰平
    if (is('上聲')) return '2'; // 陰上
    if (is('去聲')) return '3'; // 陰去
    if (is('入聲')) return '1'; // 陰入
  } else {
    if (is('平聲')) return '4'; // 陽平
    if (is('全濁 上聲')) return '6'; // 陽去，全濁上變去
    if (is('上聲')) return '5'; // 陽上
    if (is('去聲')) return '6'; // 陽去
    if (is('入聲')) return '6'; // 陽入
  }
  throw new Error('無聲調規則');
}

let 聲母 = 聲母規則();
let 韻母 = 韻母規則();
let 聲調 = 聲調規則();

// i 在 z/c/s 前為 ii
if (['z', 'c', 's'].includes(聲母) && 韻母 === 'i') 韻母 = 'ii';

// ng 拼細音時為 nj
const is細音 = ['eo', 'i', 'oe', 'u', 'yu'].some((x) => 韻母.startsWith(x));
if (聲母 === 'ng' && is細音) 聲母 = 'nj';

if (is('合口 或 模韻') && !['eo', 'oe', 'yu'].some((x) => 韻母.startsWith(x))) { // 合口字
  if (聲母 === 'g') 聲母 = 'gw';
  else if (聲母 === 'k') 聲母 = 'kw';
  else if (聲母 === 'h' && !韻母.startsWith('i')) 聲母 = 'f';
  else if (聲母 === 'j') 聲母 = 'w';
  else if (聲母 === '') 聲母 = 'w';
}

// m 韻尾在聲母為脣音時為 n
if (is('幫組') && 韻母.endsWith('m')) 韻母 = 韻母.slice(0, -1) + 'n';

if (is('入聲')) {
  if (韻母.endsWith('m')) 韻母 = 韻母.slice(0, -1) + 'p';
  else if (韻母.endsWith('n')) 韻母 = 韻母.slice(0, -1) + 't';
  else if (韻母.endsWith('ng')) 韻母 = 韻母.slice(0, -2) + 'k';
}

return 聲母 + 韻母 + 聲調;
});

_defaultDerivers.fanwan = _schemata.fanwan();

/**
 * 推導《分韻撮要》擬音
 *
 * https://ayaka.shn.hk/fanwan/
 *
 * @author Ayaka
 * @param {Qieyun.音韻地位} 音韻地位 切韻音系音韻地位
 * @param {string=} 字頭 字頭（可選）
 * @param {undefined=} 舊選項參數 已棄用，請用 `<方案名>.方案(選項)(音韻地位, 字頭?)` 代替
 * @return {string} 音韻地位對應的推導《分韻撮要》擬音
 * @property {推導方案<string>} schema 推導方案，可用於指定推導選項、獲取方案詳細資訊等
 */
export function fanwan(音韻地位, 字頭, 舊選項參數) {
  return _perform("fanwan", _defaultDerivers.fanwan, 音韻地位, 字頭, 舊選項參數);
}
/** @type {推導方案<string>} */
fanwan.schema = _schemata.fanwan

_schemata.putonghua = new 推導方案(function (選項, 音韻地位, 字頭) {
/** @type { 音韻地位['屬於'] } */
const is = (...x) => 音韻地位.屬於(...x);
/** @type { 音韻地位['判斷'] } */
const when = (...x) => 音韻地位.判斷(...x);

if (!音韻地位) return [
  ['標調方式', [2, '數字', '附標']],
];

const 聲母規則 = () => when([
  ['幫滂並母 輕脣韻', 'f'],
  ['幫母', 'b'],
  ['滂母', 'p'],
  ['並母', [['平聲', 'p'], ['', 'b']]],
  ['明母', [['微虞文元陽凡韻', 'w'], ['', 'm']]],

  ['端母', 'd'],
  ['透母', 't'],
  ['定母', [['平聲', 't'], ['', 'd']]],
  ['泥母', 'n'],
  ['來母', 'l'],

  ['知母', 'zh'],
  ['徹母', 'ch'],
  ['澄母', [['平聲', 'ch'], ['', 'zh']]],
  ['孃母', 'n'],

  ['精母', 'z'],
  ['清母', 'c'],
  ['從母', [['平聲', 'c'], ['', 'z']]],
  ['心母', 's'],
  ['邪母', 's'],

  ['莊母', 'zh'],
  ['初母', 'ch'],
  ['崇母', [['平聲', 'ch'], ['', 'sh']]],
  ['生母', 'sh'],
  ['俟母', 'sh'],

  ['章母', 'zh'],
  ['昌母', 'ch'],
  ['常母', [['平聲', 'ch'], ['', 'sh']]],
  ['書母', 'sh'],
  ['船母', 'sh'],
  ['日母', 'r'],

  ['見母', 'g'],
  ['溪母', 'k'],
  ['羣母', [['平聲', 'k'], ['', 'g']]],
  ['曉匣母', 'h'],
  ['疑影云以母', ''],
], '無聲母規則');

const 舒聲韻母規則 = () => when([
  // 通攝
  ['通攝', [['三等 牙喉音', 'iong'], ['', 'ueng']]],

  // 江攝
  ['江韻', [['牙喉音', 'iang'], ['', 'uang']]],

  // 止攝
  ['止攝 合口', [['莊組', 'uai'], ['', 'uei']]],
  ['止攝', [['牙喉音', 'i'], ['', 'er']]],

  // 遇攝
  ['魚虞韻', 'ü'],
  ['模韻', 'u'],

  // 蟹攝
  ['祭韻 合口 莊組', 'uai'],
  ['齊祭廢韻', [['合口', 'uei'], ['', 'i']]],
  ['泰灰韻', [['開口', 'ai'], ['', 'uei']]],
  ['佳韻 牙喉音', [['合口', 'ua'], ['', 'ia']]],
  ['皆夬韻 牙喉音', [['合口', 'uai'], ['', 'ie']]],
  ['佳皆夬咍韻', [['合口', 'uai'], ['', 'ai']]],

  // 臻攝
  ['眞欣韻', [['合口', 'ün'], ['', 'in']]],
  ['臻韻 或 痕韻 牙喉音', 'en'],
  ['文韻 牙喉音', 'ün'],
  ['痕魂文韻', 'uen'],
  ['元韻', [['合口', 'üan'], ['', 'ian']]],

  // 山攝
  ['寒刪山韻 合口', 'uan'],
  ['刪山韻 牙喉音', 'ian'],
  ['寒刪山韻', 'an'],
  ['仙先韻', [['合口', 'üan'], ['', 'ian']]],

  // 效攝
  ['蕭宵韻 或 肴韻 牙喉音', 'iao'],
  ['豪肴韻', 'ao'],

  // 果攝
  ['歌韻 一等', [['開口 牙喉音', 'e'], ['', 'uo']]],
  ['歌韻 三等', [['開口', 'ie'], ['', 'üe']]],

  // 假攝
  ['麻韻 二等', [['合口', 'ua'], ['牙喉音', 'ia'], ['', 'a']]],
  ['麻韻 三等', 'ie'],

  // 宕攝
  ['唐韻 開口', 'ang'],
  ['陽韻 開口 非 莊組', 'iang'],
  ['唐陽韻', 'uang'],

  // 梗攝
  ['梗攝 二等', [['合口', 'ueng'], ['', 'eng']]],
  ['梗攝', [['合口', 'iong'], ['', 'ing']]],

  // 曾攝
  ['登韻', [['合口', 'ueng'], ['', 'eng']]],
  ['蒸韻', 'ing'],

  // 流攝
  ['侯韻', 'ou'],
  ['尤韻', [['幫組', 'ou'], ['', 'iou']]],
  ['幽韻', [['幫組', 'iao'], ['', 'iou']]],

  // 深攝
  ['侵韻', 'in'],

  // 咸攝
  ['鹽添韻 或 嚴咸銜韻 牙喉音', 'ian'],
  ['覃談嚴咸銜韻', 'an'],
  ['凡韻', 'uan'],
], '無韻母規則');

const 入聲韻母規則 = () => when([
  // 通攝
  ['通攝', 'u'],

  // 江攝
  ['江韻', [['牙喉音', 'üe'], ['', 'uo']]],

  // 臻攝
  ['眞韻 合口', [['莊組', 'uai'], ['', 'ü']]],
  ['眞欣韻', 'i'],
  ['臻痕韻', 'e'],
  ['魂韻', [['幫組', 'o'], ['', 'u']]],
  ['文韻', 'ü'],
  ['元韻', [['開口', 'ie'], ['牙喉音', 'üe'], ['', 'a']]],

  // 山攝
  ['寒韻', [['非 開口', 'uo'], ['牙喉音', 'e'], ['', 'a']]],
  ['刪山韻', [['合口', 'ua'], ['牙喉音', 'ia'], ['', 'a']]],
  ['仙先韻', [['合口', 'üe'], ['', 'ie']]],

  // 宕攝
  ['唐韻', [['開口 牙喉音', 'e'], ['', 'uo']]],
  ['陽韻', [['幫組', 'o'], ['', 'üe']]],

  // 梗攝
  ['梗攝 二等', [['開口', 'e'], ['', 'uo']]],
  ['梗攝', [['合口', 'ü'], ['', 'i']]],

  // 曾攝
  ['登韻', [['開口', 'e'], ['', 'uo']]],
  ['蒸韻', [['合口', 'ü'], ['莊組', 'e'], ['', 'i']]],

  // 深攝
  ['侵韻', [['莊組', 'e'], ['', 'i']]],

  // 咸攝
  ['覃談韻', [['牙喉音', 'e'], ['', 'a']]],
  ['鹽添嚴韻', 'ie'],
  ['咸銜凡韻', [['牙喉音', 'ia'], ['', 'a']]],
], '無韻母規則');

const 聲調規則 = () => when([
  ['清音', [
    ['平聲', '1'],
    ['上聲', '3'],
    ['去聲', '4'],
    ['入聲', ''],
  ]],
  ['濁音', [
    ['平聲', '2'],
    ['上聲', [['全濁', '4'], ['次濁', '3']]],
    ['去聲', '4'],
    ['入聲', [['全濁', '2'], ['次濁', '4']]],
  ]],
], '無聲調規則');

let 聲母 = 聲母規則();
let 韻母 = is`舒聲` ? 舒聲韻母規則() : 入聲韻母規則();
let 聲調 = 聲調規則();

if (['i', 'ü'].includes(韻母[0])) 聲母 = {
  g: 'j', k: 'q', h: 'x',
  z: 'j', c: 'q', s: 'x',
}[聲母] || 聲母;

if (韻母 === 'er') {
  if (聲母 === 'r') 聲母 = '';
  else 韻母 = 'i';
}

if (['n', 'l'].includes(聲母) && ['ua', 'uai', 'uang', 'uei'].includes(韻母)) 韻母 = 韻母.slice(1);
if (韻母[0] === 'ü' && !(['n', 'l'].includes(聲母) && ['ü', 'üe'].includes(韻母))) {
  if (!聲母) 聲母 = 'y';
  韻母 = 'u' + 韻母.slice(1);
}

if (['zh', 'ch', 'sh', 'r'].includes(聲母)) {
  if (韻母[0] === 'i') {
    if (韻母[1] === 'n') 韻母 = 'e' + 韻母.slice(1);
    else if (韻母[1]) 韻母 = 韻母.slice(1);
  }
  if (韻母 === 'ue') 韻母 = 'uo';
}

if (['b', 'p', 'm', 'f', 'w'].includes(聲母) && 韻母[0] === 'u' && 韻母[1]) 韻母 = 韻母.slice(1);
if (['f', 'w'].includes(聲母) && 韻母[0] === 'i') 韻母 = 韻母.slice(1) || 'ei';

if (!聲母) {
  if (韻母[0] === 'i') 聲母 = 'y';
  if (韻母[0] === 'u') 聲母 = 'w';
  if (聲母 && 韻母[1] && 韻母[1] !== 'n') 韻母 = 韻母.slice(1);
}

韻母 = { iou: 'iu', uei: 'ui', uen: 'un', ueng: 'ong' }[韻母] || 韻母;

if (選項.標調方式 === '數字') return 聲母 + 韻母 + 聲調;
return 聲母 + (聲調 ? 韻母.replace(/.*a|.*[eo]|.*[iuü]/, '$&' + ' ̄́̌̀'[聲調]) : 韻母);
});

_defaultDerivers.putonghua = _schemata.putonghua();

/**
 * 推導普通話
 *
 * @author graphemecluster
 * @param {Qieyun.音韻地位} 音韻地位 切韻音系音韻地位
 * @param {string=} 字頭 字頭（可選）
 * @param {undefined=} 舊選項參數 已棄用，請用 `<方案名>.方案(選項)(音韻地位, 字頭?)` 代替
 * @return {string} 音韻地位對應的推導普通話
 * @property {推導方案<string>} schema 推導方案，可用於指定推導選項、獲取方案詳細資訊等
 */
export function putonghua(音韻地位, 字頭, 舊選項參數) {
  return _perform("putonghua", _defaultDerivers.putonghua, 音韻地位, 字頭, 舊選項參數);
}
/** @type {推導方案<string>} */
putonghua.schema = _schemata.putonghua

_schemata.gwongzau = new 推導方案(function (選項, 音韻地位, 字頭) {
if (!音韻地位) return [['$legacy', true]];

const is = (x) => 音韻地位.屬於(x);

function 聲母規則() {
  if (is('幫母')) {
    if (is('東韻 三等 或 鍾微虞廢文元陽尤凡韻')) return 'f';
    return 'b';
  }
  if (is('滂母')) {
    if (is('東韻 三等 或 鍾微虞廢文元陽尤凡韻')) return 'f';
    return 'p';
  }
  if (is('並母')) {
    if (is('東韻 三等 或 鍾微虞廢文元陽尤凡韻')) return 'f';
    if (is('平聲')) return 'p';
    return 'b';
  }
  if (is('明母')) return 'm';

  if (is('端母')) return 'd';
  if (is('透母')) return 't';
  if (is('定母')) return is('平聲') ? 't' : 'd';
  if (is('泥母')) return 'n';
  if (is('來母')) return 'l';

  if (is('知母')) return 'z';
  if (is('徹母')) return 'c';
  if (is('澄母')) return is('平聲') ? 'c' : 'z';
  if (is('孃母')) return 'n';

  if (is('精母')) return 'z';
  if (is('清母')) return 'c';
  if (is('從母')) return is('平聲') ? 'c' : 'z';
  if (is('心母')) return 's';
  if (is('邪母')) return is('平聲') ? 'c' : 'z'; // 塞擦音多於擦音

  if (is('莊母')) return 'z';
  if (is('初母')) return 'c';
  if (is('崇母')) return is('平聲') ? 'c' : 'z';
  if (is('生母')) return 's';
  if (is('俟母')) return is('平聲') ? 'c' : 'z';

  if (is('章母')) return 'z';
  if (is('昌母')) return 'c';
  if (is('常母')) return 's'; // 擦音多於塞擦音
  if (is('書母')) return 's';
  if (is('船母')) return 's';
  if (is('日母')) return 'j';

  if (is('見母')) return 'g';
  if (is('溪母')) return 'h'; // 多數擦化
  if (is('羣母')) return is('平聲') ? 'k' : 'g';
  if (is('疑母')) return 'ng'; // ng 拼細音時為 j，詳後

  if (is('曉母')) return 'h';
  if (is('匣母')) {
    if (is('合口 或 模韻')) return 'j'; // 非 yu 前為 w，詳後
    return 'h';
  }
  if (is('影云以母')) {
    if (is('三四等')) return 'j'; // 非 yu 前為 w，詳後
    return '';
  }

  throw new Error('無聲母規則');
}

function 韻母規則() {
  // 通攝
  if (is('東冬鍾韻')) return 'ung';

  // 江攝
  if (is('江韻 幫組')) return 'ong';
  if (is('江韻 舌齒音')) return 'oeng';
  if (is('江韻 牙喉音')) return 'ong';

  // 止攝
  if (is('支脂之微韻 幫組')) return 'ei';
  if (is('支脂之微韻 開口 舌齒音 端組')) return 'ei';
  if (is('支脂之微韻 開口 舌齒音 來母')) return 'ei';
  if (is('支脂之微韻 開口 舌齒音 孃母')) return 'ei';
  if (is('支脂之微韻 開口 舌齒音')) return 'i';
  if (is('支脂之微韻 開口 牙喉音 疑母')) return 'i';
  if (is('支脂之微韻 開口 牙喉音 影母')) return 'i';
  if (is('支脂之微韻 開口 牙喉音 云母')) return 'i';
  if (is('支脂之微韻 開口 牙喉音 以母')) return 'i';
  if (is('支脂之微韻 開口 牙喉音')) return 'ei';
  if (is('支脂之微韻 合口 舌齒音')) return 'eoi';
  if (is('支脂之微韻 合口 牙喉音')) return 'ai';

  // 遇攝
  if (is('魚虞韻 幫組 幫滂並母')) return 'u';
  if (is('魚虞韻 幫組 明母')) return 'ou';
  if (is('魚虞韻 舌齒音 端組')) return 'eoi';
  if (is('魚虞韻 舌齒音 來母')) return 'eoi';
  if (is('魚虞韻 舌齒音 孃母')) return 'eoi';
  if (is('魚虞韻 舌齒音 精組')) return 'eoi';
  if (is('魚虞韻 舌齒音 莊組')) return 'o';
  if (is('魚虞韻 舌齒音')) return 'yu';
  if (is('魚虞韻 牙喉音 見溪羣母')) return 'eoi';
  if (is('魚虞韻 牙喉音 曉匣母')) return 'eoi';
  if (is('魚虞韻 牙喉音')) return 'yu';
  if (is('模韻 脣音')) return 'ou';
  if (is('模韻 舌齒音')) return 'ou';
  if (is('模韻 牙喉音 疑母')) return '';
  if (is('模韻 牙喉音')) return 'u';

  // 蟹攝
  if (is('齊韻')) return 'ai';
  if (is('祭韻 幫組')) return 'ai';
  if (is('祭韻 開口')) return 'ai';
  if (is('祭韻 合口 舌齒音')) return 'eoi';
  if (is('祭韻 合口 以母')) return 'eoi';
  if (is('祭韻 合口 牙喉音')) return 'ai';
  if (is('泰韻 幫組')) return 'ui';
  if (is('泰韻 開口 舌齒音 精組')) return 'oi';
  if (is('泰韻 開口 舌齒音')) return 'aai';
  if (is('泰韻 開口 牙喉音')) return 'oi';
  if (is('泰韻 合口 舌齒音')) return 'eoi';
  if (is('泰韻 合口 牙喉音 疑母')) return 'oi';
  if (is('泰韻 合口 牙喉音')) return 'ui';
  if (is('佳皆夬韻 幫組')) return 'aai';
  if (is('佳皆夬韻 開口')) return 'aai';
  if (is('佳皆夬韻 合口 舌齒音')) return 'eoi';
  if (is('佳皆夬韻 合口')) return 'aai';
  if (is('灰韻 舌齒音')) return 'eoi';
  if (is('灰韻 疑母')) return 'oi';
  if (is('灰韻')) return 'ui';
  if (is('咍韻')) return 'oi';
  if (is('廢韻')) return 'ai';

  // 臻攝
  if (is('眞韻 幫組')) return 'an';
  if (is('眞韻 開口')) return 'an';
  if (is('眞韻 合口 舌齒音')) return 'eon';
  if (is('眞韻 合口 牙喉音')) return 'an';
  if (is('臻文欣韻')) return 'an';
  if (is('元韻 幫組')) return 'aan';
  if (is('元韻 開口')) return 'in';
  if (is('元韻 合口')) return 'yun';
  if (is('魂韻 幫組')) return 'un';
  if (is('魂韻 端組')) return 'eon';
  if (is('魂韻 來母')) return 'eon';
  if (is('魂韻 精組')) return 'yun';
  if (is('魂韻 牙喉音')) return 'an';
  if (is('痕韻')) return 'an';

  // 山攝
  if (is('寒韻 幫組')) return 'un';
  if (is('寒韻 開口 舌齒音')) return 'aan';
  if (is('寒韻 開口 牙喉音')) return 'on';
  if (is('寒韻 合口 舌齒音')) return 'yun';
  if (is('寒韻 合口 牙喉音')) return 'un';
  if (is('刪山韻')) return 'aan';
  if (is('仙先韻 幫組')) return 'in';
  if (is('仙先韻 開口')) return 'in';
  if (is('仙先韻 合口')) return 'yun';

  // 效攝
  if (is('蕭宵韻')) return 'iu';
  if (is('肴韻')) return 'aau';
  if (is('豪韻')) return 'ou';

  // 果攝
  if (is('歌韻 一等')) return 'o';
  if (is('歌韻 三等 脣音')) return 'e';
  if (is('歌韻 三等 開口')) return 'e';
  if (is('歌韻 三等 合口')) return 'oe';

  // 假攝
  if (is('麻韻 二等')) return 'aa';
  if (is('麻韻 三等')) return 'e';

  // 宕攝
  if (is('陽韻 幫組')) return 'ong';
  if (is('陽韻 開口 莊組')) return 'ong';
  if (is('陽韻 開口')) return 'oeng';
  if (is('陽韻 合口')) return 'ong';
  if (is('唐韻')) return 'ong';

  // 梗攝
  if (is('庚韻 二等')) return 'ang';
  if (is('庚韻 三等 莊組')) return 'ang';
  if (is('庚韻 三等')) return 'ing';
  if (is('耕韻')) return 'ang';
  if (is('清青韻')) return 'ing';

  // 曾攝
  if (is('蒸韻')) return 'ing';
  if (is('登韻')) return 'ang';

  // 流攝
  if (is('尤侯幽韻')) return 'au';

  // 深攝
  if (is('侵韻')) return 'am'; // m 韻尾在聲母為脣音時為 n，詳後，下同

  // 咸攝
  if (is('覃談韻 幫組')) return 'aam';
  if (is('覃談韻 舌齒音')) return 'aam';
  if (is('覃談韻 牙喉音')) return 'om'; // -om 併入 -am，詳後
  if (is('鹽添嚴韻')) return 'im';
  if (is('咸銜凡韻')) return 'aam';

  throw new Error('無韻母規則');
}

function 聲調規則() {
  if (is('全清 或 次清')) {
    if (is('平聲')) return '1'; // 陰平
    if (is('上聲')) return '2'; // 陰上
    if (is('去聲')) return '3'; // 陰去
    if (is('入聲')) return '1'; // 陰入。長元音為 3，詳後
  } else {
    if (is('平聲')) return '4'; // 陽平
    if (is('全濁 上聲')) return '6'; // 陽去，全濁上變去
    if (is('上聲')) return '5'; // 陽上
    if (is('去聲')) return '6'; // 陽去
    if (is('入聲')) return '6'; // 陽入
  }
  throw new Error('無聲調規則');
}

function is長元音(韻母) {
  if (['aam', 'aan', 'im', 'in', 'om', 'on', 'ong', 'oeng', 'un', 'yun'].includes(韻母)) return true;
  if (['am', 'an', 'ang', 'eon', 'ing', 'ung'].includes(韻母)) return false;
  throw new Error('無法判斷元音長短：' + 韻母);
}

let 聲母 = 聲母規則();
let 韻母 = 韻母規則();
let 聲調 = 聲調規則();

// ng 拼細音時為 j
const is細音 = ['eo', 'i', 'oe', 'u', 'yu'].some((x) => 韻母.startsWith(x));
if (聲母 === 'ng' && is細音) 聲母 = 'j';

// 陰入分化
if (is('入聲') && 聲調 === '1' && is長元音(韻母)) 聲調 = '3';

if (is('合口 或 模韻') && !['eo', 'oe', 'yu'].some((x) => 韻母.startsWith(x))) { // 合口字
  if (聲母 === 'g' && !韻母.startsWith('u')) 聲母 = 'gw';
  else if (聲母 === 'k' && !韻母.startsWith('u')) 聲母 = 'kw';
  else if (聲母 === 'h' && !韻母.startsWith('i')) 聲母 = 'f';
  else if (聲母 === 'j') 聲母 = 'w';
  else if (聲母 === '') 聲母 = 'w';
}

// -om 併入 -am
if (韻母 === 'om') 韻母 = 'am';

// m 韻尾在聲母為脣音時為 n
if (is('幫組') && 韻母.endsWith('m')) 韻母 = 韻母.slice(0, -1) + 'n';

if (is('入聲')) {
  if (韻母.endsWith('m')) 韻母 = 韻母.slice(0, -1) + 'p';
  else if (韻母.endsWith('n')) 韻母 = 韻母.slice(0, -1) + 't';
  else if (韻母.endsWith('ng')) 韻母 = 韻母.slice(0, -2) + 'k';
}

return 聲母 + 韻母 + 聲調;
});

_defaultDerivers.gwongzau = _schemata.gwongzau();

/**
 * 推導廣州音
 *
 * https://ayaka.shn.hk/teoi/
 *
 * @author Ayaka
 * @param {Qieyun.音韻地位} 音韻地位 切韻音系音韻地位
 * @param {string=} 字頭 字頭（可選）
 * @param {undefined=} 舊選項參數 已棄用，請用 `<方案名>.方案(選項)(音韻地位, 字頭?)` 代替
 * @return {string} 音韻地位對應的推導廣州音
 * @property {推導方案<string>} schema 推導方案，可用於指定推導選項、獲取方案詳細資訊等
 */
export function gwongzau(音韻地位, 字頭, 舊選項參數) {
  return _perform("gwongzau", _defaultDerivers.gwongzau, 音韻地位, 字頭, 舊選項參數);
}
/** @type {推導方案<string>} */
gwongzau.schema = _schemata.gwongzau

_schemata.zaonhe = new 推導方案(function (選項, 音韻地位, 字頭) {
const is = (x) => 音韻地位.屬於(x);

if (!音韻地位)
  return [
    ['$legacy', true],
    ['文白讀', [3, '文上白下', '僅白讀', '僅文讀', '主流層']],
    ['標調方式', [3, '數字調值', '折線', '附標', '八調序號', '弗標']],
    ['分尖團', [1, '分尖團', '區分⟨情､琴⟩', '區分⟨徐､齊⟩']],
    ['區分⟨衣､煙⟩', true],
    ['區分⟨來､蘭⟩', true],
    ['區分⟨袜､麦⟩', true],
    ['區分⟨打､黨⟩', true],
    ['區分⟨肉､月⟩', true],
    ['區分⟨國､骨⟩', true],
    ['區分⟨于､園⟩', true],
    ['區分⟨干､官⟩', true],
    ['區分⟨困､孔⟩', true],
    ['區分⟨羣､窮⟩', true],
  ];

const 分衣煙 = 選項['區分⟨衣､煙⟩'];
const 分來蘭 = 選項['區分⟨來､蘭⟩'];
const 分袜麦 = 選項['區分⟨袜､麦⟩'];
const 分打黨 = 選項['區分⟨打､黨⟩'];
const 分肉月 = 選項['區分⟨肉､月⟩'];
const 分國骨 = 選項['區分⟨國､骨⟩'];
const 分于園 = 選項['區分⟨于､園⟩'];
const 分干官 = 選項['區分⟨干､官⟩'];
const 分困恐 = 選項['區分⟨困､孔⟩'];
const 分羣窮 = 選項['區分⟨羣､窮⟩'];

const 元音 = 'iyɨʉɯuɪʏʊeøɘɵɤoəɛœɜɞʌɔæɐaɶäɑɒ';
const 元音Re = new RegExp('[' + 元音 + ']');
const 閉前元音 = 'iyɪʏ';
const 元音附標 = '̃̈';
const 顎化分尖團 = {
  n: 'ɲ',
  k: 'tɕ',
  kʰ: 'tɕʰ',
  h: 'ɕ',
  ŋ: 'ɲ',
  g: 'dʑ',
};
const 顎化分情琴 = {
  ts: 'tɕ',
  tsʰ: 'tɕʰ',
  s: 'ɕ',
  z: 'ʑ',
};
const 顎化弗分尖團 = {
  ʑ: 'dʑ',
};
const 數字標調 = {
  陰平: '⁵³',
  陰去: '³³⁴',
  陽去: '²³',
  陰入: '⁵⁵',
  陽入: '¹²',
};
const 折線標調 = {
  陰平: '˥˧',
  陰去: '˧˧˦',
  陽去: '˨˧',
  陰入: '˥',
  陽入: '˩˨',
};
const 附標標調 = {
  陰平: '᷇',
  陰去: '̄',
  陽去: '̌',
  陰入: '̋',
  陽入: '᷅',
};
const 序號標調 = {
  陰平: '¹',
  陰去: '⁵',
  陽去: '⁶',
  陰入: '⁷',
  陽入: '⁸',
};

function 聲母規則(文讀) {
  // 顎化規則由獨立個 function 來做
  // '▽' 標識弗是主流層
  // 表一（of「上海市区方言志」）
  if (is('幫滂母 東鍾微虞廢文元陽尤凡韻 三等')) return 'f'; // 理論白讀 /p/
  if (is('並母 東鍾微虞廢文元陽尤凡韻 三等')) return 文讀 ? 'v' : '▽▽b'; // 「防」等
  if (is('明母 鍾微虞廢文元陽凡韻')) return 文讀 ? 'v' : '▽▽m'; // 「蚊」等
  if (is('幫母')) return 'p';
  if (is('滂母')) return 'pʰ';
  if (is('並母')) return 'b';
  if (is('明母')) return 'm';
  // 表二
  if (is('端母')) return 't';
  if (is('透母')) return 'tʰ';
  if (is('定母')) return 'd';
  if (is('泥孃母')) return 'n';
  if (is('來母')) return 'l';
  // 表三
  if (is('精知莊章母')) return 'ts';
  if (is('清徹初昌母')) return 'tsʰ';
  if (is('心生書母')) return 's';
  if (is('日母')) return 文讀 ? (is('止攝 開口 三等') ? '▽▽ɦ' : 'z') : 'n'; // 止開三部分在表五
  if (is('從邪澄崇俟常船母')) return 'z';
  // 表四
  if (is('見母')) return 'k';
  if (is('溪母')) return 'kʰ';
  if (is('曉母')) return 'h';
  if (is('日母')) return 'n';
  if (is('羣母')) return 'g';
  // 表五
  if (is('疑母 開合中立 遇攝 一等 上聲')) return 文讀 ? '▽▽ʔ' : 'ŋ'; // 「五」。表中無
  if (is('疑母 開合中立')) return 文讀 ? '▽▽ɦ' : 'ŋ'; // 「吾」等
  if (is('疑母 合口 果攝 一等')) return 文讀 ? '▽▽ɦ' : 'ŋ'; // 「臥」
  if (is('疑母 合口 麻泰韻')) return 文讀 ? '▽▽ɦ' : 'ŋ'; // 「外」「瓦」
  if (is('疑母 合口')) return 文讀 ? 'ɦ' : '▽▽ŋ'; // 「魏」等
  if (is('疑母 開口 二三四等')) return 文讀 ? '▽▽ɦ' : 'ŋ'; // 「言」等
  if (is('疑母 開口 一等')) return 'ŋ';
  if (is('以母 脂韻 合口')) return 'v'; // 表中無
  // if (is('以母 上聲 通攝')) return 'ʔ'; // 似無規律。弗收
  // if (is('以母 上聲') && '以苡已勇蛹涌恿甬俑踊慂𧻹悀埇𧗴'.includes(字頭)) return 'ʔ';
  if (is('云母 上去聲 梗攝')) return 'ʔ'; // 表中無
  if (is('匣云以母')) return 'ɦ';
  if (is('影母')) return 'ʔ';
  throw new Error('無聲母規則');
}

function 韻母規則(文讀) {
  // 特殊韻母
  if (is('日母 支脂之微韻 開口') && 文讀) return 'əɭ';
  // if (is('明母 侯韻') && !文讀) return '̩';
  // if (is('明母 虞韻') && !文讀) return '̩';
  // if ((is('疑母 遇攝 開口 三等') || is('疑母 遇攝 開合中立 一等')) && !文讀) return '̩';

  // 通攝
  if (is('東冬鍾韻 舒聲 三等 牙音 疑母')) return 'ioŋ';
  if (is('東冬鍾韻 舒聲 三等 牙音')) return 文讀 ? 'oŋ' : 'ioŋ▽'; // 「龔」等
  if (is('東冬鍾韻 舒聲 三等 日母')) return 文讀 ? 'oŋ▽' : 'ioŋ'; // 「茸」等
  if (is('東冬鍾韻 舒聲 三等 孃母')) return 'ioŋ';
  if (is('東冬鍾韻 舒聲 三等 喉音')) return 'ioŋ';
  if (is('東冬鍾韻 舒聲')) return 'oŋ'; // 脣音白讀僅「夢」。待考
  if (is('東冬鍾韻 入聲 三等 牙喉音')) return 分肉月 ? 'ioʔ' : 'yɪʔ';
  if (is('東冬鍾韻 入聲 三等 日母'))
    return 文讀 ? 'oʔ▽' : 分肉月 ? 'ioʔ' : 'yɪʔ'; // 「肉」等
  if (is('東冬鍾韻 入聲 三等 孃母')) return 分肉月 ? 'ioʔ' : 'yɪʔ';
  if (is('東冬鍾韻 入聲')) return 'oʔ';

  // 江攝
  if (is('江韻 舒聲 牙音')) return 文讀 ? (分打黨 ? 'iã▽' : 'iɑ̃▽') : 'ɑ̃'; // 「江」等
  if (is('江韻 舒聲')) return 'ɑ̃';
  if (is('江韻 入聲 疑母')) return 文讀 ? (分袜麦 ? 'iɑʔ▽' : 'iɐʔ▽') : 'oʔ'; // 「樂」等。疑母文讀同匣母
  // if (is('江韻 入聲 牙音')) return 文讀 ? 分肉月 ? 'ioʔ▽' : 'yɪʔ▽' : 'oʔ'; // 「確」等。箇個同「上海市区方言志」音系
  if (is('江韻 入聲 牙音')) return 文讀 ? (分袜麦 ? 'iɑʔ▽' : 'iɐʔ▽') : 'oʔ'; // 「確」等。箇個同「上海土白集字」音系
  if (is('江韻 入聲 喉音')) return 文讀 ? (分袜麦 ? 'iɑʔ▽' : 'iɐʔ▽') : 'oʔ'; // 「學」等
  if (is('江韻 入聲')) return 'oʔ';

  // 止攝
  if (is('支脂之微韻 脣音')) return 'i';
  if (is('支脂之微韻 開口 舌齒音 端組')) return 'i';
  if (is('支脂之微韻 開口 舌齒音 來孃日母')) return 'i';
  if (is('支脂之微韻 開口 舌齒音')) return 'z̩';
  if (is('支脂之微韻 開口 牙喉音')) return 'i';
  if (is('支脂之微韻 合口 舌齒音 來孃母')) return 'e';
  if (is('支脂之微韻 合口 舌齒音 日母')) return 文讀 ? 'ø' : 'y▽'; // 「蕊」
  if (is('支脂之微韻 合口 舌齒音')) return 文讀 ? 'ø' : 'z̩▽'; // 「吹」等
  if (is('支脂之微韻 合口 牙音 疑母')) return 'ue';
  if (is('支脂之微韻 合口 牙音')) return 文讀 ? 'ue' : 'y▽'; // 「跪」等
  if (is('脂韻 合口 以母')) return 'i';
  if (is('支脂之微韻 合口 喉音')) return 'ue';

  // 遇攝
  if (is('魚虞韻 脣音')) return 'u';
  if (is('魚韻 舌齒音 端組')) return 文讀 ? 'y' : 'i▽';
  if (is('魚虞韻 舌齒音 端組')) return 'y';
  if (is('魚韻 舌齒音 來母')) return 文讀 ? 'y' : 'i▽'; // 「呂」等
  if (is('魚虞韻 舌齒音 來母')) return 'y';
  if (is('魚韻 舌齒音 孃母')) return 文讀 ? 'y' : 'i▽'; // 「女」（老派）
  if (is('魚虞韻 舌齒音 孃母')) return 'y';
  if (is('魚韻 舌齒音 精組')) return 文讀 ? 'y' : 'i▽'; // 「絮」等
  if (is('魚虞韻 舌齒音 精組')) return 'y';
  if (is('魚韻 舌齒音 莊組')) return 文讀 ? 'u' : 'z̩▽'; // 「鋤」等
  if (is('魚虞韻 舌齒音 莊組')) return 'u';
  if (is('魚韻 舌齒音 日母')) return 文讀 ? 'z̩' : 'i▽'; // 白讀類推無實例
  if (is('魚虞韻 舌齒音 日母')) return 文讀 ? 'z̩' : 'y▽'; // 白讀類推無實例
  if (is('魚虞韻 舌齒音')) return 'z̩';
  if (is('魚虞韻 牙喉音')) return 'y'; // 白讀 /e/。「許」「鋸」。白讀 /i/。「去」「渠」。屬於特殊存古，且弗唯一，故弗用
  if (is('模韻')) return 'u';

  // 蟹攝
  if (is('齊韻')) return 'i';
  if (is('祭韻 脣音')) return 'i';
  if (is('祭韻 開口 舌齒音 來母')) return 'i';
  if (is('祭韻 開口 舌齒音 精莊組')) return 'i';
  if (is('祭韻 開口 舌齒音')) return 'z̩';
  if (is('祭韻 開口 牙喉音')) return 'i';
  if (is('祭韻 合口 舌齒音 日母')) return 文讀 ? 'ø' : 'i▽'; // 白讀類推無實例
  if (is('祭韻 合口 舌齒音')) return 'ø';
  if (is('祭韻 合口 牙喉音')) return 'ue';
  if (is('泰韻 脣音')) return 'e';
  if (is('泰韻 開口 舌齒音')) return 'a';
  if (is('泰韻 開口 牙喉音')) return 'e';
  if (is('泰韻 合口 舌齒音 來母')) return 'e';
  if (is('泰韻 合口 舌齒音 精組')) return 'ø';
  if (is('泰韻 合口 舌齒音')) return 'e';
  if (is('泰韻 合口 牙喉音 疑母')) return 文讀 ? 'ue▽' : 'a'; // 「外」
  if (is('泰韻 合口 牙喉音')) return 'ue';
  if (is('佳皆夬韻 脣音')) return 'a';
  if (is('佳皆夬韻 開口 舌齒音 孃母')) return 'i';
  if (is('佳皆夬韻 開口 舌齒音 來母')) return 'e';
  if (is('佳皆夬韻 開口 舌齒音')) return 'a';
  if (is('佳皆夬韻 開口 牙喉音 匣母')) return 文讀 ? 'ie' : 'a▽'; // 「械」等。理論上還有 /ia/ 個白讀
  if (is('佳皆夬韻 開口 牙喉音')) return 文讀 ? 'ia' : 'a▽'; // 「解」等
  if (is('佳皆夬韻 合口 舌齒音')) return 'ø';
  if (is('佳皆夬韻 合口 牙喉音 疑母')) return 文讀 ? 'ue' : 'a▽'; // 字典弗曾收「聵」
  if (is('佳皆夬韻 合口 牙喉音')) return 文讀 ? 'ua' : 'o▽'; // 「卦」等
  if (is('灰韻 脣音')) return 'e';
  if (is('灰韻 舌齒音 精組')) return 'ø';
  if (is('灰韻 舌齒音')) return 'e';
  if (is('灰韻 牙喉音')) return 'ue';
  if (is('咍韻')) return 'e';
  if (is('廢韻 牙喉音')) return 'ue';
  if (is('廢韻')) return 'i';

  // 臻攝
  if (is('眞韻 舒聲 脣音')) return 'ɪɲ';
  if (is('眞韻 舒聲 開口 舌齒音 知章莊組')) return 'əŋ';
  if (is('眞韻 舒聲 開口 舌齒音 日母')) return 文讀 ? 'əŋ▽' : 'ɪɲ'; // 「人」等
  if (is('眞韻 舒聲 開口 舌齒音')) return 'ɪɲ';
  if (is('眞韻 舒聲 開口 牙喉音')) return 'ɪɲ';
  if (is('眞韻 舒聲 合口 舌齒音 精組')) return 'ɪɲ';
  if (is('眞韻 舒聲 合口 舌齒音 日母')) return 文讀 ? 'əŋ▽' : 'ɪɲ'; // 「閏」等
  if (is('眞韻 舒聲 合口 舌齒音')) return 'əŋ';
  if (is('眞韻 舒聲 合口 牙喉音')) return 分羣窮 ? 'yɪɲ' : 'ioŋ';
  if (is('眞韻 入聲 脣音')) return 'iɪʔ';
  if (is('眞韻 入聲 開口 舌齒音 知章莊組')) return 'əʔ';
  if (is('眞韻 入聲 開口 舌齒音 日母')) return 文讀 ? 'əʔ▽' : 'iɪʔ'; // 「日」等
  if (is('眞韻 入聲 開口 舌齒音')) return 'iɪʔ';
  if (is('眞韻 入聲 開口 牙喉音')) return 'iɪʔ';
  if (is('眞韻 入聲 合口 舌齒音 精組')) return 'iɪʔ';
  if (is('眞韻 入聲 合口 舌齒音 來母')) return 'iɪʔ';
  if (is('眞韻 入聲 合口 舌齒音 日母')) return 文讀 ? 'əʔ▽' : 'iɪʔ';
  if (is('眞韻 入聲 合口 舌齒音')) return 'əʔ';
  if (is('眞韻 入聲 合口 牙喉音')) return 'yɪʔ';
  if (is('臻文欣韻 舒聲 脣音')) return 'əŋ';
  if (is('臻文欣韻 舒聲 開口')) return 'ɪɲ';
  if (is('臻文欣韻 舒聲 合口')) return 分羣窮 ? 'yɪɲ' : 'ioŋ';
  if (is('臻文欣韻 入聲 脣音')) return 'əʔ';
  if (is('臻文欣韻 入聲 開口')) return 'iɪʔ';
  if (is('臻文欣韻 入聲 合口')) return 'yɪʔ';
  if (is('元韻 舒聲 脣音')) return 分來蘭 ? 'æ' : 'e';
  if (is('元韻 舒聲 開口')) return 分衣煙 ? 'iɪ' : 'i';
  if (is('元韻 舒聲 合口')) return 分于園 ? 'yø' : 'y';
  if (is('元韻 入聲 脣音')) return 'ɐʔ';
  if (is('元韻 入聲 開口')) return 'iɪʔ';
  if (is('元韻 入聲 合口')) return 'yɪʔ';
  if (is('魂韻 舒聲 牙喉音 疑母'))
    return 文讀 ? (分困恐 ? 'uəŋ' : 'oŋ') : 'əŋ▽'; // 「諢」。白讀類推無實例
  if (is('魂韻 舒聲 牙喉音')) return 分困恐 ? 'uəŋ' : 'oŋ';
  if (is('魂韻 舒聲')) return 'əŋ';
  if (is('魂韻 入聲 牙喉音 疑母'))
    return 文讀 ? (分國骨 ? 'uəʔ▽' : 'oʔ▽') : 'əʔ'; // 「兀」
  if (is('魂韻 入聲 牙喉音')) return 分國骨 ? 'uəʔ' : 'oʔ';
  if (is('魂韻 入聲')) return 'əʔ';
  if (is('痕韻 舒聲')) return 'əŋ';
  if (is('痕韻 入聲')) return 'iɪʔ';

  // 山攝
  if (is('寒韻 舒聲 脣音')) return 'ø';
  if (is('寒韻 舒聲 開口 舌齒音')) return 分來蘭 ? 'æ' : 'e';
  if (is('寒韻 舒聲 開口 牙喉音')) return 'ø';
  if (is('寒韻 舒聲 合口 舌齒音')) return 'ø';
  if (is('寒韻 舒聲 合口 牙喉音')) return 分干官 ? 'uø' : 'ø';
  if (is('寒韻 入聲 脣音')) return 'əʔ';
  if (is('寒韻 入聲 開口 舌齒音')) return 'ɐʔ';
  if (is('寒韻 入聲 開口 牙喉音')) return 'əʔ';
  if (is('寒韻 入聲 合口 舌齒音')) return 'əʔ';
  if (is('寒韻 入聲 合口 牙喉音 見曉影母')) return 'uɐʔ';
  if (is('寒韻 入聲 合口 牙喉音 疑母'))
    return 文讀 ? (分國骨 ? 'uəʔ▽' : 'oʔ▽') : 'əʔ'; // 字典弗曾收「枂」
  if (is('寒韻 入聲 合口 牙喉音')) return 分國骨 ? 'uəʔ' : 'oʔ';
  if (is('刪山韻 舒聲 脣音')) return 分來蘭 ? 'æ' : 'e';
  if (is('刪山韻 舒聲 開口 舌齒音')) return 分來蘭 ? 'æ' : 'e';
  if (is('刪山韻 舒聲 合口 舌齒音')) return 'ø';
  if (is('刪山韻 舒聲 開口 牙喉音'))
    return 文讀 ? (分衣煙 ? 'iɪ▽' : 'i▽') : 分來蘭 ? 'æ' : 'e'; // 「間」等
  if (is('刪山韻 舒聲 合口 牙喉音')) return 分來蘭 ? 'uæ' : 'ue';
  if (is('刪山韻 入聲 脣音')) return 'ɐʔ';
  if (is('刪山韻 入聲 開口 舌齒音 日母')) return 文讀 ? 'əʔ▽' : 'iɪʔ'; // 字典弗曾收「𩭿」
  if (is('刪山韻 入聲 開口 舌齒音')) return 'ɐʔ';
  if (is('刪山韻 入聲 開口 牙喉音 匣母')) return 文讀 ? 'iɐʔ▽' : 'ɐʔ'; // 「轄」
  if (is('刪山韻 入聲 開口 牙喉音 疑母')) return 文讀 ? 'iɐʔ▽' : 'ɐʔ'; // 「齾」
  if (is('刪山韻 入聲 開口 牙喉音')) return 'ɐʔ';
  if (is('刪山韻 入聲 合口 舌齒音')) return 'əʔ';
  if (is('刪山韻 入聲 合口 牙喉音 疑母')) return 文讀 ? 'uɐʔ▽' : 'ɐʔ'; // 字典弗曾收「刖」
  if (is('刪山韻 入聲 合口 牙喉音')) return 'uɐʔ';
  if (is('仙先韻 舒聲 脣音')) return 分衣煙 ? 'iɪ' : 'i';
  if (is('仙先韻 舒聲 開口 知章組')) return 'ø';
  if (is('仙先韻 舒聲 開口 日母')) return 文讀 ? 'ø' : 分衣煙 ? 'iɪ▽' : 'i▽'; // 「燃」等
  if (is('仙先韻 舒聲 開口')) return 分衣煙 ? 'iɪ' : 'i';
  if (is('仙先韻 舒聲 合口 舌齒音 精組')) return 分衣煙 ? 'iɪ' : 'i'; // 白讀 /æ/ 或 /ø/ 只有「全」特殊存古。故弗列
  if (is('仙先韻 舒聲 合口 舌齒音 來母')) return 分衣煙 ? 'iɪ' : 'i';
  if (is('仙先韻 舒聲 合口 舌齒音 日母'))
    return 文讀 ? 'ø▽' : 分于園 ? 'yø' : 'y'; // 「軟」等
  if (is('仙先韻 舒聲 合口 舌齒音')) return 'ø';
  if (is('仙先韻 舒聲 合口 牙喉音')) return 分于園 ? 'yø' : 'y';
  if (is('仙先韻 入聲 脣音')) return 'iɪʔ';
  if (is('仙先韻 入聲 開口 舌齒音 知莊章組')) return 'əʔ';
  if (is('仙先韻 入聲 開口 舌齒音 日母')) return 文讀 ? 'əʔ▽' : 'iɪʔ'; // 「熱」等
  if (is('仙先韻 入聲 開口 舌齒音')) return 'iɪʔ';
  if (is('仙先韻 入聲 開口 牙喉音')) return 'iɪʔ';
  if (is('仙先韻 入聲 合口 舌齒音 知莊章組')) return 'əʔ';
  if (is('仙先韻 入聲 合口 舌齒音 日母')) return 文讀 ? 'əʔ▽' : 'iɪʔ'; // 「爇」等
  if (is('仙先韻 入聲 合口 舌齒音')) return 'iɪʔ';
  if (is('仙先韻 入聲 合口 牙喉音')) return 'yɪʔ';

  // 效攝
  if (is('蕭宵韻 知章組')) return 'ɔ';
  if (is('蕭宵韻')) return 'iɔ';
  if (is('肴韻 脣音')) return 'ɔ';
  if (is('肴韻 舌齒音')) return 'ɔ';
  if (is('肴韻 牙喉音')) return 文讀 ? 'iɔ▽' : 'ɔ'; // 「交」等
  if (is('豪韻')) return 'ɔ';

  // 果攝
  if (is('歌韻 一等 開口 端組')) return 文讀 ? 'u' : 'a▽'; // 「大」等
  if (is('歌韻 一等 開口 喉音')) return 文讀 ? 'u' : 'a▽'; // 「何」等
  if (is('歌韻 一等')) return 'u';
  if (is('歌韻 三等 開口')) return 文讀 ? 'ia▽' : 'a'; // 「茄」等
  if (is('歌韻 三等 合口')) return 文讀 ? 'ia▽' : 分于園 ? 'io' : 'y'; // 「瘸」等
  if (is('歌韻 三等 脣音')) return 'u'; // 「縛」

  // 假攝
  if (is('麻韻 二等 脣音')) return 文讀 ? 'a▽' : 'o'; // 「馬」等
  if (is('麻韻 二等 舌齒音')) return 文讀 ? 'a▽' : 'o'; // 「差」等
  if (is('麻韻 二等 開口 牙音')) return 文讀 ? 'ia▽' : 'a'; // 「家」等
  if (is('麻韻 二等 開口 喉音')) return 文讀 ? 'ia▽' : 'o'; // 「下」等
  if (is('麻韻 二等 合口 牙喉音')) return 文讀 ? 'ua▽' : 'o'; // 「花」等。脆麻花～
  if (is('麻韻 三等 脣音')) return 文讀 ? 'i▽' : 'ia'; // 「乜」
  if (is('麻韻 三等 舌齒音 精知組')) return 文讀 ? 'i▽' : 'ia'; // 「姐」等
  if (is('麻韻 三等 舌齒音 日母')) return 文讀 ? 'a' : 'ia▽'; // 「惹」等
  if (is('麻韻 三等 舌齒音 章組')) return 文讀 ? 'o' : 'a▽'; // 「射」等
  if (is('麻韻 三等 喉音')) return 文讀 ? 'ie▽' : 'ia'; // 「也」等

  // 宕攝
  if (is('陽韻 舒聲 脣音')) return 'ɑ̃';
  if (is('陽韻 舒聲 舌齒音 舌音 來孃母')) return 分打黨 ? 'iã' : 'iɑ̃';
  if (is('陽韻 舒聲 舌齒音 舌音')) return 分打黨 ? 'ã' : 'ɑ̃';
  if (is('陽韻 舒聲 舌齒音 齒音 精組')) return 分打黨 ? 'iã' : 'iɑ̃';
  if (is('陽韻 舒聲 舌齒音 齒音 日母'))
    return 文讀 ? 'ɑ̃▽' : 分打黨 ? 'iã' : 'iɑ̃'; // 「壤」等
  if (is('陽韻 舒聲 舌齒音')) return 'ɑ̃';
  if (is('陽韻 舒聲 開口 牙喉音')) return 分打黨 ? 'iã' : 'iɑ̃';
  if (is('陽韻 舒聲 合口 牙喉音 云母'))
    return 文讀 ? 'uɑ̃' : 分打黨 ? 'iã▽' : 'iɑ̃▽'; // 「旺」等
  if (is('陽韻 舒聲 合口 牙喉音')) return 'uɑ̃';
  if (is('陽韻 入聲 脣音')) return 'oʔ';
  if (is('陽韻 入聲 舌齒音 來孃母')) return 分袜麦 ? 'iɑʔ' : 'iɐʔ';
  if (is('陽韻 入聲 舌齒音 精組')) return 分袜麦 ? 'iɑʔ' : 'iɐʔ';
  if (is('陽韻 入聲 舌齒音 日母'))
    return 文讀 ? (分袜麦 ? 'ɑʔ' : 'ɐʔ') : 分袜麦 ? 'iɑʔ▽' : 'iɐʔ▽'; // 「箬」等
  if (is('陽韻 入聲 舌齒音')) return 分袜麦 ? 'ɑʔ' : 'ɐʔ';
  if (is('陽韻 入聲 開口 牙喉音')) return 分袜麦 ? 'iɑʔ' : 'iɐʔ';
  if (is('陽韻 入聲 合口 牙喉音')) return 分肉月 ? 'ioʔ' : 'yɪʔ';
  if (is('唐韻 舒聲 脣音')) return 'ɑ̃';
  if (is('唐韻 舒聲 舌齒音')) return 'ɑ̃';
  if (is('唐韻 舒聲 開口 牙喉音')) return 'ɑ̃';
  if (is('唐韻 舒聲 合口 牙喉音')) return 'uɑ̃';
  if (is('唐韻 入聲')) return 'oʔ';

  // 梗攝
  if (is('庚韻 舒聲 二等 脣音')) return 文讀 ? 'əŋ▽' : 分打黨 ? 'ã' : 'ɑ̃'; // 「猛」等
  if (is('庚韻 舒聲 二等 舌齒音')) return 文讀 ? 'əŋ▽' : 分打黨 ? 'ã' : 'ɑ̃'; // 「澄」等
  if (is('庚韻 舒聲 二等 開口 牙喉音'))
    return 文讀 ? 'əŋ▽' : 分打黨 ? 'ã' : 'ɑ̃'; // 「更」等
  if (is('庚韻 舒聲 二等 合口 牙喉音')) return 'uɑ̃';
  if (is('庚韻 舒聲 三等 脣音')) return 'ɪɲ';
  if (is('庚韻 舒聲 三等 齒音')) return 文讀 ? 'əŋ▽' : 分打黨 ? 'ã' : 'ɑ̃'; // 「省」等
  if (is('庚韻 舒聲 三等 開口 牙喉音'))
    return 文讀 ? 'ɪɲ' : 分打黨 ? 'iã▽' : 'iɑ̃▽'; // 「映」等
  if (is('庚韻 舒聲 三等 合口 牙喉音')) return 'ioŋ';
  if (is('庚韻 入聲 二等 脣音')) return 文讀 ? 'əʔ▽' : 分袜麦 ? 'ɑʔ' : 'ɐʔ'; // 「白」等。收於「上海土白集字」
  if (is('庚韻 入聲 二等 舌齒音')) return 文讀 ? 'əʔ▽' : 分袜麦 ? 'ɑʔ' : 'ɐʔ'; // 「澤」等
  if (is('庚韻 入聲 二等 開口 牙喉音'))
    return 文讀 ? 'əʔ▽' : 分袜麦 ? 'ɑʔ' : 'ɐʔ'; // 「客」等。收於「上海土白集字」
  if (is('庚韻 入聲 二等 合口 牙喉音')) return 'oʔ';
  if (is('庚韻 入聲 三等')) return 'iɪʔ';
  if (is('耕韻 舒聲 脣音')) return 文讀 ? 'əŋ▽' : 分打黨 ? 'ã' : 'ɑ̃'; // 「萌」等
  if (is('耕韻 舒聲 舌齒音')) return 文讀 ? 'əŋ▽' : 分打黨 ? 'ã' : 'ɑ̃'; // 「爭」等
  if (is('耕韻 舒聲 開口 牙喉音')) return 文讀 ? 'ɪɲ' : 分打黨 ? 'ã▽' : 'ɑ̃▽'; // 「櫻」等
  if (is('耕韻 舒聲 合口 喉音')) return 'oŋ';
  if (is('耕韻 入聲 脣音')) return 文讀 ? 'əʔ▽' : 分袜麦 ? 'ɑʔ' : 'ɐʔ'; // 「脈」等。收於「上海土白集字」
  if (is('耕韻 入聲 舌齒音')) return 文讀 ? 'əʔ▽' : 分袜麦 ? 'ɑʔ' : 'ɐʔ'; // 「責」等
  if (is('耕韻 入聲 開口 牙喉音')) return 文讀 ? 'əʔ▽' : 分袜麦 ? 'ɑʔ' : 'ɐʔ'; // 「革」等
  if (is('耕韻 入聲 合口 牙喉音')) return 'oʔ';
  if (is('清青韻 舒聲 脣音')) return 'ɪɲ';
  if (is('清青韻 舒聲 舌齒音 知章組'))
    return 文讀 ? 'əŋ' : 分打黨 ? 'ã▽' : 'ɑ̃▽'; // 「聲」等
  if (is('清青韻 舒聲 舌齒音')) return 'ɪɲ';
  if (is('清青韻 舒聲 開口 牙喉音'))
    return 文讀 ? 'ɪɲ' : 分打黨 ? 'iã▽' : 'iɑ̃▽'; // 理論白讀
  if (is('清青韻 舒聲 合口 牙喉音')) return 'ioŋ'; // 有異讀 /ɪɲ/
  if (is('清青韻 入聲 脣音')) return 'iɪʔ';
  if (is('清青韻 入聲 舌齒音 知章組'))
    return 文讀 ? 'əʔ▽' : 分袜麦 ? 'ɑʔ' : 'ɐʔ'; // 「適」等
  if (is('清青韻 入聲 舌齒音')) return 'iɪʔ';
  if (is('清青韻 入聲 開口 牙喉音')) return 'iɪʔ';
  if (is('清青韻 入聲 合口 牙喉音')) return 分肉月 ? 'ioʔ' : 'yɪʔ';

  // 曾攝
  if (is('蒸韻 舒聲 脣音')) return 'ɪɲ';
  if (is('蒸韻 舒聲 舌齒音 來母')) return 'ɪɲ';
  if (is('蒸韻 舒聲 舌齒音 日母')) return 文讀 ? 'əŋ▽' : 'ɪɲ'; // 「仍」等
  if (is('蒸韻 舒聲 舌齒音')) return 文讀 ? 'əŋ' : 分打黨 ? 'ã▽' : 'ɑ̃▽'; // 「剩」等
  if (is('蒸韻 舒聲 牙喉音')) return 'ɪɲ';
  if (is('蒸韻 入聲 脣音')) return 'iɪʔ';
  if (is('蒸韻 入聲 舌齒音 精組')) return 'iɪʔ';
  if (is('蒸韻 入聲 舌齒音 來孃母')) return 'iɪʔ';
  if (is('蒸韻 入聲 舌齒音 日母')) return 文讀 ? 'əʔ▽' : 'iɪʔ';
  if (is('蒸韻 入聲 舌齒音')) return 'əʔ';
  if (is('蒸韻 入聲 開口 牙喉音')) return 'iɪʔ';
  if (is('蒸韻 入聲 合口 喉音')) return 分肉月 ? 'ioʔ' : 'yɪʔ';
  if (is('登韻 舒聲 脣音 明母')) return 文讀 ? 'oŋ' : 'ɑ̃▽'; // 「懵」
  if (is('登韻 舒聲 脣音')) return 文讀 ? 'əŋ▽' : 分打黨 ? 'ã' : 'ɑ̃'; // 「崩」等
  if (is('登韻 舒聲 開口')) return 'əŋ'; // 無白讀例
  if (is('登韻 舒聲 合口')) return 'oŋ';
  if (is('登韻 入聲 脣音 明母')) return 'əʔ';
  if (is('登韻 入聲 脣音')) return 'oʔ';
  if (is('登韻 入聲 開口')) return 'əʔ';
  if (is('登韻 入聲 合口')) return 'oʔ';

  // 流攝
  if (is('尤侯韻 尤韻 脣音 去聲')) return 'u';
  if (is('尤侯韻 脣音')) return 'ɤ';
  if (is('尤侯韻 尤韻 舌齒音 精組')) return 'iɤ';
  if (is('尤侯韻 尤韻 舌齒音 來孃母')) return 'iɤ';
  if (is('尤侯韻 尤韻 舌齒音 日母')) return 文讀 ? 'ɤ' : 'iɤ▽'; // 「柔」等
  if (is('尤侯韻 舌齒音')) return 'ɤ';
  if (is('尤侯韻 尤韻 牙喉音')) return 'iɤ';
  if (is('尤侯韻 牙喉音')) return 'ɤ';
  if (is('幽韻 脣音 明母')) return 文讀 ? 'iɤ▽' : 'iɔ'; // 「繆」等
  if (is('幽韻 脣音')) return 'iɔ';
  if (is('幽韻 舌齒音 精組')) return 'iɤ';
  if (is('幽韻 舌齒音 來母')) return 'iɤ';
  if (is('幽韻 尤韻 舌齒音 日母')) return 文讀 ? 'ɤ' : 'iɤ▽';
  if (is('幽韻 舌齒音')) return 'ɤ';
  if (is('幽韻 牙喉音')) return 'iɤ';

  // 深攝
  if (is('侵韻 舒聲 脣音')) return 'ɪɲ';
  if (is('侵韻 舒聲 舌齒音 精組')) return 'ɪɲ';
  if (is('侵韻 舒聲 舌齒音 來孃母')) return 'ɪɲ';
  if (is('侵韻 舒聲 舌齒音 日母')) return 文讀 ? 'əŋ' : 'ɪɲ▽'; // 「任」等
  if (is('侵韻 舒聲 舌齒音')) return 'əŋ';
  if (is('侵韻 舒聲 牙喉音')) return 'ɪɲ';
  if (is('侵韻 入聲 脣音')) return 'iɪʔ';
  if (is('侵韻 入聲 舌齒音 精組')) return 'iɪʔ';
  if (is('侵韻 入聲 舌齒音 來孃母')) return 'iɪʔ';
  if (is('侵韻 入聲 舌齒音 日母')) return 文讀 ? 'əʔ' : 'iɪʔ▽'; // 「入」等
  if (is('侵韻 入聲 舌齒音')) return 'əʔ';
  if (is('侵韻 入聲 牙喉音')) return 'iɪʔ';

  // 咸攝
  if (is('覃韻 舒聲 脣音')) return 'ø';
  if (is('覃韻 舒聲 舌齒音 來母')) return 分來蘭 ? 'æ' : 'e';
  if (is('覃韻 舒聲 舌齒音 端定母')) return 分來蘭 ? 'æ' : 'e'; // 也有讀 /ø/ 個口音，但是弗在「上海市區方言志」
  if (is('覃韻 舒聲 舌齒音')) return 'ø';
  if (is('覃韻 舒聲 牙喉音 溪母')) return 分來蘭 ? 'æ' : 'e'; // 也有讀 /ø/ 個口音，但是弗在「上海市區方言志」
  if (is('覃韻 舒聲 牙喉音')) return 'ø';
  if (is('覃韻 入聲')) return 文讀 ? 'əʔ▽' : 'ɐʔ'; // 「答」等
  if (is('談韻 舒聲 脣音')) return 分來蘭 ? 'æ' : 'e'; // 生僻字無資料。推測
  if (is('談韻 舒聲 舌齒音')) return 分來蘭 ? 'æ' : 'e';
  if (is('談韻 舒聲 牙喉音')) return 'ø';
  if (is('談韻 入聲 牙喉音')) return 'əʔ';
  if (is('談韻 入聲')) return 'ɐʔ';
  if (is('鹽添嚴韻 舒聲 舌齒音 知組 孃母')) return 分衣煙 ? 'iɪ' : 'i';
  if (is('鹽添嚴韻 舒聲 舌齒音 知組')) return 'ø';
  if (is('鹽添嚴韻 舒聲 舌齒音 莊章組')) return 'ø';
  if (is('鹽添嚴韻 舒聲 舌齒音 日母'))
    return 文讀 ? 'ø' : 分衣煙 ? 'iɪ▽' : 'i▽'; // 「染」等
  if (is('鹽添嚴韻 舒聲')) return 分衣煙 ? 'iɪ' : 'i';
  if (is('鹽添嚴韻 入聲 舌齒音 知組 孃母')) return 'iɪʔ';
  if (is('鹽添嚴韻 入聲 舌齒音 知組')) return 'əʔ';
  if (is('鹽添嚴韻 入聲 舌齒音 莊章組')) return 'əʔ';
  if (is('鹽添嚴韻 入聲 舌齒音 日母')) return 文讀 ? 'əʔ▽' : 'iɪʔ'; // 無文讀例
  if (is('鹽添嚴韻 入聲')) return 'iɪʔ';
  if (is('咸銜凡韻 舒聲 脣音')) return 分來蘭 ? 'æ' : 'e';
  if (is('咸銜凡韻 舒聲 舌齒音 來母')) return 分衣煙 ? 'iɪ' : 'i';
  if (is('咸銜凡韻 舒聲 舌齒音 孃母')) return 'ø';
  if (is('咸銜凡韻 舒聲 舌齒音')) return 分來蘭 ? 'æ' : 'e';
  if (is('咸銜凡韻 舒聲 牙喉音'))
    return 文讀 ? (分衣煙 ? 'iɪ▽' : 'i▽') : 分來蘭 ? 'æ' : 'e'; // 「減」等
  if (is('咸銜凡韻 入聲 牙喉音')) return 文讀 ? 'iɐʔ▽' : 'ɐʔ'; // 「甲」等
  if (is('咸銜凡韻 入聲')) return 'ɐʔ';

  throw new Error('無韻母規則');
}

function 顎化規則(音節) {
  if (音節.match(元音Re)) {
    let 第一個元音 = 音節.match(元音Re)[0];
    if (閉前元音.includes(第一個元音)) {
      for (let 聲母 in 顎化分尖團) 音節 = 音節.replace(聲母, 顎化分尖團[聲母]);
      if (選項.分尖團 !== '分尖團') {
        for (let 聲母 in 顎化分情琴)
          音節 = 音節.replace(聲母, 顎化分情琴[聲母]);
      }
      if (選項.分尖團 === '區分⟨徐､齊⟩') {
        if (is('從崇常母')) {
          for (let 聲母 in 顎化弗分尖團)
            音節 = 音節.replace(聲母, 顎化弗分尖團[聲母]);
        }
      }
    }
  }
  return 音節;
}

function 主流層選擇規則(音們) {
  let 音們_非主流度們 = [];
  音們.forEach((音) => {
    音們_非主流度們.push({ 音: 音, 非主流度: (音.match(/▽/g) || []).length });
  });
  return 音們_非主流度們.reduce(function (prev, curr) {
    return prev.非主流度 < curr.非主流度 ? prev : curr;
  }).音;
}

function 聲調規則(音節) {
  let 聲調;
  if (is('云母 上去聲 梗攝')) 聲調 = '陰去'; // 由於聲母例外
  // else if (is('以母 上聲 通攝')) 聲調 = '陰去'; // 由於聲母例外
  else if (is('疑母 上聲 模韻')) 聲調 = '陰平'; // 由於聲母例外
  else if (is('全清 或 次清') && is('平聲')) 聲調 = '陰平';
  else if (is('全清 或 次清') && is('上去聲')) 聲調 = '陰去';
  else if (is('全清 或 次清') && is('入聲')) 聲調 = '陰入';
  else if (is('全濁 或 次濁') && is('入聲')) 聲調 = '陽入';
  else if (is('全濁 或 次濁') && is('舒聲')) 聲調 = '陽去';
  else throw new Error('無聲調規則');

  if (選項.標調方式 === '附標') {
    let 標調位置;
    if (音節.match(元音Re)) {
      let 第一個元音 = 音節.match(元音Re)[0];
      標調位置 = 音節.indexOf(第一個元音);
      if (元音.includes(音節[標調位置 + 1])) 標調位置 += 1; // 弗要標在介音上
      if (元音附標.includes(音節[標調位置 + 1])) 標調位置 += 1; // 弗要標在附標下頭
    } else {
      標調位置 = 音節.indexOf('̩');
    }
    標調位置 += 1;
    return 音節.slice(0, 標調位置) + 附標標調[聲調] + 音節.slice(標調位置);
  } else if (選項.標調方式 === '數字調值') {
    return 音節 + 數字標調[聲調];
  } else if (選項.標調方式 === '折線') {
    return 音節 + 折線標調[聲調];
  } else if (選項.標調方式 === '八調序號') {
    return 音節 + 序號標調[聲調];
  } else {
    return 音節;
  }
}

function finalise(音節) {
  音節 = 音節.replace(/▽/g, '');
  音節 = 顎化規則(音節);
  音節 = 聲調規則(音節);
  return 音節;
}

let 文讀聲母 = 聲母規則(true);
let 白讀聲母 = 聲母規則(false);

let 文讀韻母 = 韻母規則(true);
let 白讀韻母 = 韻母規則(false);

let 文讀音 = 文讀聲母 + 文讀韻母;
let 白讀音 = 白讀聲母 + 白讀韻母;

let 結果;
if (選項.文白讀 === '主流層') {
  結果 = 主流層選擇規則([文讀音, 白讀音]);
  結果 = finalise(結果);
} else {
  if (選項.文白讀 === '僅白讀') 結果 = finalise(白讀音);
  else if (選項.文白讀 === '僅文讀') 結果 = finalise(文讀音);
  else if (文讀音 === 白讀音) 結果 = finalise(文讀音);
  else 結果 = finalise(文讀音) + '\n' + finalise(白讀音);
}

return 結果;
});

_defaultDerivers.zaonhe = _schemata.zaonhe();

/**
 * 推導上海話
 *
 * https://zhuanlan.zhihu.com/p/386456940
 *
 * ——適改「上海市区方言志」音系
 *
 * 在墶「上海市区方言志」個基礎上向增加着一些理論白讀層搭着理論文讀層（參考「上海土白集字」），但是確保至少一個層次是「上海市区方言志」音系。
 * 提供了一些選項，其中「上海市区方言志」弗分尖團、弗分衣煙、弗分來蘭、弗分襪麥、弗分打黨、弗分肉月。
 *
 * @author Nyoeghau
 * @param {Qieyun.音韻地位} 音韻地位 切韻音系音韻地位
 * @param {string=} 字頭 字頭（可選）
 * @param {undefined=} 舊選項參數 已棄用，請用 `<方案名>.方案(選項)(音韻地位, 字頭?)` 代替
 * @return {string} 音韻地位對應的推導上海話
 * @property {推導方案<string>} schema 推導方案，可用於指定推導選項、獲取方案詳細資訊等
 */
export function zaonhe(音韻地位, 字頭, 舊選項參數) {
  return _perform("zaonhe", _defaultDerivers.zaonhe, 音韻地位, 字頭, 舊選項參數);
}
/** @type {推導方案<string>} */
zaonhe.schema = _schemata.zaonhe

_schemata.langjin = new 推導方案(function (選項, 音韻地位, 字頭) {
const is = (x) => 音韻地位.屬於(x);

if (!音韻地位) return [
  ['$legacy', true],
  ['標調方式', [1, '數字次序', '韻母附標']],  
];

const 次序標調 = {
  '陰平': '¹',
  '陽平': '²',
  '上聲': '³',
  '去聲': '⁴',
  '入聲': '⁵',
};
const 附標標調 = {
  '陰平': '̄',
  '陽平': '́',
  '上聲': '̌',
  '去聲': '̀',
  '入聲': '̂',
};

const 元音 = 'iuüaeoyär';
const 元音Re = new RegExp("[" + 元音 + "]");
const 元音附標 = '̃̈';

function 聲母規則() {
  if (is('幫母')) return is('東韻 三等 或 鍾微虞廢文元陽尤凡韻') ? 'f' : 'b';
  if (is('滂母')) return is('東韻 三等 或 鍾微虞廢文元陽尤凡韻') ? 'f' : 'p';
  if (is('並母')) return is('東韻 三等 或 鍾微虞廢文元陽尤凡韻') ? 'f' : is('平聲') ? 'p' : 'b';
  if (is('明母')) return is('三等 凡微文陽虞元韻') ? '' : 'm'; // 等價於三等合口
  if (is('端母')) return 'd';
  if (is('透母')) return 't';
  if (is('定母')) return is('平聲') ? 't' : 'd';
  if (is('泥來孃母')) return 'l';
  if (is('知母')) return is('麻韻 三等') ? 'd' : is('庚耕韻') ? 'z' : 'zh'; // 知組平翹律
  if (is('徹母')) return is('庚耕韻') ? 'c' : 'ch'; // 知組平翹律
  if (is('澄母')) { 
    if (is('庚耕韻')) return is('平聲') ? 'c' : 'z'; // 平送氣仄不送氣
    // 剩下翹舌
    return is('平聲') ? 'ch' : 'zh'; // 平送氣仄不送氣
  }
  if (is('精母')) return 'z';
  if (is('清母')) return 'c';
  if (is('從母')) return is('二等') ? 'ch' : is('平聲') ? 'c' : 'z'; // 平送氣仄不送氣
  if (is('心母')) return 's';
  if (is('邪母')) return is('平聲 尤之韻') ? 'c' : 's';
  if (is('莊母')) return is('宕假效江攝 或 止攝 合口 或 蟹咸山攝 二等') ? 'zh' : 'z'; // 莊組平翹律
  if (is('初母')) return is('宕假效江攝 或 止攝 合口 或 蟹咸山攝 二等') ? 'ch' : 'c'; // 莊組平翹律
  if (is('崇母')) {
    if (is('宕假效江攝 或 止攝 合口 或 蟹咸山攝 二等')) return is('平聲') ? 'ch' : 'zh';
    // 剩下平舌
    return is('之韻') ? 's' : is('平聲') ? 'c' : 'z'; // 平送氣仄不送氣
  }
  if (is('生母')) return is('宕假效江攝 或 止攝 合口 或 蟹咸山攝 二等') ? 'sh' : 's'; // 莊組平翹律
  if (is('俟母')) return is('平聲') ? 'c' : 's'; // 平送氣仄不送氣
  if (is('章母')) return 'zh';
  if (is('昌母')) return 'ch';
  if (is('常母')) return is('曾攝 入聲') ? 'zh' : is('平聲 齊侵清仙鹽陽尤魚虞眞蒸支鍾韻 或 一等') ? 'ch' : 'sh';
  if (is('船書母')) return is('平聲 通攝 或 平聲 合口 山臻攝') ? 'ch' : 'sh'; // 章組擦音分化律
  if (is('日母')) return is('四等') ? 'l' : is('支之脂韻 開口 或 眞侵韻 入聲') ? '' : 'r';  
  let 不顎化 = '一等 或 二等 合口 或 二等 庚耕韻 或 三等 合口 祭微陽支脂凡廢韻 舒聲 或 三等 通攝 舒聲 或 四等 合口 齊韻'; // 見溪羣曉匣母不顎化條件
  if (is('見母')) return is(不顎化) ? 'g' : 'j';
  if (is('溪母')) return is('二等 皆韻 或 二等 江韻 入聲') ? 'k' : is(不顎化) ? 'k' : 'q';
  if (is('羣母')) {  
    if (is('宵韻 重紐A類')) return 'q';
    if (is('平聲')) return is('三等 合口 山陽脂韻') ? 'k' : 'q'; // 平送氣
    // 剩下仄聲不送氣
    return is(不顎化) ? 'g' : 'j';
  }
  if (is('疑母')) {
    if(is('之韻 上聲')) return 'l';
    return is('一二等') ? '' : is('尤蒸齊韻 平聲 或 先仙陽庚韻 入聲') ? 'l' : '';
  }
  if (is('匣母')) return is('開口 耕韻 舒聲') ? 'x' : is(不顎化) ? 'h' : 'x';  
  if (is('曉母')) return is('三等 開口 或 三等 通攝') ? 'x' : is(不顎化) ? 'h' : 'x';
  if (is('以母')) return is('合口 祭韻') ? 'r' : '';
  if (is('影母')) return '';
  if (is('云母')) return is('舒聲 通攝') ? 'x' : '';
  throw new Error('無聲母規則');
}

function 韻母規則() {
  // 通攝
  if (is('東韻 入聲')) return is('三等 見溪羣曉匣疑以影云母') ? 'ü' : 'u'; 
  if (is('東韻 舒聲')) return is('三等 疑以影母') ? 'iong' : is('幫組') ? 'en' : 'ong';
  if (is('冬韻')) return is('入聲') ? 'u' : is('幫組') ? 'en' : 'ong';
  if (is('鍾韻 入聲')) return is('見溪羣曉匣疑以影云母') ? 'ü' : 'u'; 
  if (is('鍾韻 舒聲')) return is('疑以影母') ? 'iong' : is('幫組') ? 'en' : 'ong';

  // 江攝
  if (is('江韻 入聲')) return is('疑母') ? 'io' : 'o'; 
  if (is('江韻 舒聲')) return is('徹澄崇初生知母') ? 'uang' : is('疑母') ? 'iang' : 'ang'; 

  // 止攝 
  if (is('支脂之韻 日母 開口')) return 'er'; 
  if (is('支脂之韻 崇初從精清生俟邪心莊母 開口')) return 'y'; // 平舌音
  if (is('支脂之韻 昌常徹澄船書章知母 開口')) return 'r'; // 翹舌音
  if (is('支脂韻 莊組 合口')) return 'uä';
  if (is('支韻 重紐B類') && is('幫母 或 並母 上去聲')) return 'ei';
  if (is('脂韻 重紐B類') && is('並滂母 或 幫母 平聲')) return 'ei';
  if (is('脂韻 明母')) return 'ei';
  if (is('微韻 幫並滂母')) return 'ei';
  if (is('支脂之微韻 幫並滂母')) return 'i';
  if (is('支韻 明母')) return 'i';
  if (is('微韻 明母')) return 'uei';
  if (is('支脂之微韻 開口')) return 'i';
  if (is('支脂微韻 合口')) return 'uei';

  // 遇攝
  if (is('模韻')) return is('明母') ? 'o' : 'u'; 
  if (is('魚虞韻')) return is('從見精來孃清羣溪曉邪心疑以影云母') ? 'ü' : 'u';  

  // 蟹攝
  if (is('齊韻')) return is('合口 或 常母') ? 'uei' : 'i'; 
  if (is('祭韻 明母')) return 'ei';
  if (is('廢祭灰韻 合口')) return 'uei'; 
  if (is('廢韻 幫組')) return 'ei';
  if (is('祭韻 幫組')) return 'i';
  if (is('廢祭韻 開口')) return is('章知組') ? 'r' : is('莊組') ? 'y' : 'i';  
  if (is('佳韻 合口')) return is('見溪匣曉影母') ? 'ua' : 'uä';
  if (is('皆夬韻 合口')) return 'uä';
  if (is('佳皆夬韻 幫組'))  return 'ä';
  if (is('佳皆夬韻 開口'))  return is('疑母') ? 'iä' : 'ä';
  if (is('灰廢韻 幫組')) return 'ei';
  if (is('灰韻 開口')) return is('以母') ? 'iä' : 'ä'; 
  if (is('咍韻')) return is('合口') ? 'uei' : is('以母') ? 'iä' : 'ä';
  if (is('泰韻 合口')) return is('見溪疑母') ? 'uä' : 'uei'; 
  if (is('泰韻 幫組')) return 'ei';
  if (is('泰韻 開口')) return is('以母') ? 'iä' : 'ä';  

  // 臻攝
  if (is('文韻 入聲 幫組')) return 'u';
  if (is('臻欣文眞韻 入聲 合口')) return is('知莊章組') ? 'u' : 'ü'; 
  if (is('臻欣文眞韻 入聲 幫組')) return 'i';
  if (is('臻欣文眞韻 入聲 開口')) return is('莊組') ? 'ä' : is('章組 或 知徹澄日母') ? 'r' : 'i';
  if (is('臻欣文眞韻 舒聲 開口')) return is('莊章組 或 日知徹澄母') ? 'en' : 'in';
  if (is('眞韻 舒聲 幫組')) return 'in'; 
  if (is('文眞韻 舒聲 合口')) return is('來明日書章知昌常徹澄船母') ? 'uen' : 'üin';            
  if (is('眞臻欣文韻 舒聲 幫組')) return is ('明母') ? 'uen' : 'en'; 
  if (is('眞臻欣韻 舒聲 合口')) return is('滂幫並母') ? 'en' : is('來明日書章知昌常徹澄船母') ? 'uen' : 'üin';
  if (is('魂痕韻 入聲')) return is('幫組 或 開口') ? 'o' : 'u';
  if (is('魂痕韻 舒聲 幫組')) return 'en';
  if (is('魂痕韻 舒聲 開口')) return is('端組') ? 'uen' : 'en';
  if (is('魂痕韻 舒聲 合口')) return 'uen';

  // 山攝
  if (is('先韻 舒聲 合口')) return 'üän';
  if (is('先韻 舒聲 開口 或 先韻 舒聲 幫組')) return is('崇母') ? 'uang' : is('見溪羣曉匣母') ? 'än' : 'iän';
  if (is('元韻 入聲 幫組')) return is('明母') ? 'ua' : 'a';
  if (is('元仙先韻 入聲 合口')) return is('日母 或 知莊章組') ? 'o' : 'üe'; 
  if (is('仙先韻 入聲 幫組')) return 'ie';
  if (is('元仙先韻 入聲 開口')) return is('日母 或 知莊章組') ? 'ä' : is('見溪羣曉匣母') ? 'e' : 'ie';
  if (is('仙韻 舒聲 幫組')) return 'iän';
  if (is('元韻 舒聲 幫組')) return is('明母') ? 'uang' : 'ang';
  if (is('元仙韻 舒聲 合口')) return is('日來母 或 知莊章組') ? 'uang' : 'üän';
  if (is('元仙韻 舒聲 開口')) return is('日知徹澄母 或 莊章組') ? 'ang' : is('見溪羣曉匣母') ? 'än' : 'iän';
  if (is('刪山韻 入聲')) return is('合口') ? 'ua' : is('疑影母') ? 'ia' : 'a';
  if (is('刪山韻 舒聲 幫組')) return 'ang'; 
  if (is('刪山韻 舒聲 開口')) return is('影疑母') ? 'iän' : is('見溪羣曉匣母') ? 'än' : 'ang';
  if (is('刪山韻 舒聲 合口')) return 'uang';
  if (is('寒韻 入聲 開口')) return is('見溪羣曉匣疑影母') ? 'o' : 'a';
  if (is('寒韻 入聲 幫組')) return 'o';
  if (is('寒韻 入聲 合口')) return is('見組') ? 'uä' : 'o';
  if (is('寒韻 舒聲')) return is('開口 或 幫組') ? 'ang' : 'uang';
  
  // 效攝
  if (is('蕭宵韻')) return is('見溪羣曉匣日母 或 知章組') ? 'ao' : 'iao';
  if (is('肴韻')) return is('疑母') ? 'iao' : 'ao';
  if (is('豪韻')) return 'ao';
  
  // 果攝
  if (is('歌韻')) return is('一等') ? 'o' : is('開口') ? 'e' : 'üe';
  
  // 假攝
  if (is('麻韻 二等')) return is('合口') ? 'ua' : is('疑影母') ? 'ia' : 'a';
  if (is('麻韻 三等')) return is('日母 或 章組') ? 'e' : 'ie';

  // 宕攝
  if (is('陽韻 入聲')) return is('心疑以影云來孃母 或 精組') ? 'io' : 'o';
  if (is('唐陽韻 舒聲 合口')) return 'uang';
  if (is('唐陽韻 舒聲 滂幫並母')) return 'ang';
  if (is('陽韻 舒聲 明母')) return 'uang';
  if (is('陽韻 舒聲 開口')) return is('來孃疑以影母 或 精組') ? 'iang' : is('莊組') ? 'uang' : 'ang';
  if (is('唐韻 入聲')) return is('合口 見組') ? 'uä' : 'o';
  if (is('唐韻 舒聲 明母')) return 'ang';
  if (is('唐韻 舒聲 開口'))return 'ang';

  // 梗攝
  if (is('庚韻 入聲 二等')) return is('合口') ? 'uä' : 'ä';
  if (is('庚韻 入聲 三等')) return is('莊組') ? 'y' : is('合口') ? 'ü' : 'i';
  if (is('庚韻 舒聲 二等')) return is('合口') ? 'uen' : 'en';
  if (is('庚韻 舒聲 三等 合口')) return is('心以影母') ?  'in' : is('云影母') ? 'iong' : 'ong';
  if (is('庚韻 舒聲 三等 開口')) return is('知莊章組') ? 'en' : 'in';
  if (is('庚韻 舒聲 三等 幫組')) return 'in';
  if (is('青韻 舒聲 合口')) return is('云影母') ? 'iong' : 'ong';
  if (is('青韻 舒聲 開口')) return 'in';
  if (is('清青韻 入聲 幫組')) return 'i'; 
  if (is('清青韻 入聲 合口')) return 'ü';
  if (is('清青韻 入聲 開口')) return is('莊組') ? 'y' : is('知章組') ? 'r' : 'i';
  if (is('清韻 舒聲 合口 心以影母')) return 'in';
  if (is('清韻 舒聲 合口 羣溪曉母')) return 'ong';
  if (is('清韻 舒聲 合口 云影母')) return 'iong';
  if (is('清青韻 舒聲 幫組')) return 'in';
  if (is('清韻 舒聲 開口')) return is('知莊章組') ? 'en' : 'in';
  if (is('耕韻 入聲 合口')) return 'uä';
  if (is('耕韻 入聲 開口 或 耕韻 入聲 幫組')) return 'ä';       
  if (is('耕韻 舒聲 幫組')) return 'en';
  if (is('耕韻 舒聲 合口')) return 'ong' ; 
  if (is('耕韻 舒聲 開口')) return is('匣影母') ? 'in' : 'en';

  // 曾攝
  if (is('蒸韻 入聲 幫組')) return 'i'; 
  if (is('蒸韻 入聲 合口')) return 'ü';
  if (is('蒸韻 入聲 開口')) return is('莊組') ? 'ä' : is('知徹澄母 或 章組') ? 'r' : 'i';  
  if (is('蒸韻 舒聲 幫組')) return 'in'; 
  if (is('蒸韻 舒聲 開口')) return is('見組 或 來曉以影母') ? 'in' : 'en';
  if (is('登韻 入聲 幫組')) return 'ä'; 
  if (is('登韻 入聲 合口')) return 'uä';
  if (is('登韻 入聲 開口')) return 'ä';
  if (is('登韻 舒聲 幫組')) return 'en'; 
  if (is('登蒸韻 舒聲 合口')) return 'ong'; 
  if (is('登韻 舒聲 開口')) return 'en';

  // 流攝
  if (is('幽韻')) return is('幫組') ? 'iao' : is('見溪羣曉生母') ? 'ou' : 'iou';
  if (is('尤韻')) return is('滂幫並母') ? 'u' : is('精組 或 疑以影云孃來母') ? 'iou' : 'ou';
  if (is('侯韻')) return 'ou';
  
  // 深攝
  if (is('侵韻 入聲')) return is('莊組') ? 'ä' : is('章組 或 日知徹澄母') ? 'r' : 'i';
  if (is('侵韻 舒聲')) return is('章莊組 或 日知徹澄母') ? 'en' : 'in';

  // 咸攝
  if (is('添韻 入聲')) return is('見溪羣曉匣母') ? 'e' : 'ie';
  if (is('添韻 舒聲')) return is('見溪羣曉匣母') ? 'än' : 'iän';
  if (is('鹽嚴凡韻 入聲 幫組')) return 'a';
  if (is('鹽嚴凡韻 入聲 合口')) return is('徹孃母') ? 'ua' : 'a';
  if (is('鹽嚴凡韻 入聲 開口')) return is('莊章組 或 日知徹澄母') ? 'ä' : is('見溪羣曉匣母') ? 'e' : 'ie';
  if (is('鹽韻 舒聲 幫組')) return 'iän';
  if (is('嚴凡韻 舒聲 幫組')) return is('明母') ? 'uang' : 'ang';
  if (is('鹽嚴凡韻 舒聲 合口')) return 'uang';
  if (is('鹽嚴凡韻 舒聲 開口')) return is('日知徹澄母 或 莊章組') ? 'ang' : is('見溪羣曉匣母') ? 'än' : 'iän';
  if (is('咸銜韻 入聲')) return is('疑影母') ? 'ia' : 'a';
  if (is('咸銜韻 舒聲')) return is('來影疑母') ? 'iän' : is('見溪羣曉匣母') ? 'än' : 'ang';
  if (is('覃談韻 入聲')) return is('見組 或 匣曉影母') ? 'o' : 'a';     
  if (is('覃談韻 舒聲')) return is('開口 或 幫組') ? 'ang' : 'uang';

  throw new Error('無韻母規則');
}

function 聲調規則(音節) {
  let 聲調;
  if (is('平聲')) 聲調 = is('全清 或 次清') ? '陰平' : '陽平';
  else if (is('上聲')) 聲調 = is('全濁') ? '去聲' : '上聲';   
  else if (is('去聲')) 聲調 = '去聲';   
  else if (is('入聲')) 聲調 = '入聲';
  else throw new Error('無聲調規則');
  if (選項.標調方式 === '韻母附標') {
    let 標調位置;
    if (音節.match(元音Re)) {
      let 第一個元音 = 音節.match(元音Re)[0];
      標調位置 = 音節.indexOf(第一個元音);
      if (元音.includes(音節[標調位置 + 1])) 標調位置 += 1; // 不要標在介音高頭
      if (元音附標.includes(音節[標調位置 + 1])) 標調位置 += 1; // 不要標在附標下頭
      if (音節.includes('a')) 標調位置 = 音節.indexOf('a');
      else if (音節.includes('o')) 標調位置 = 音節.indexOf('o');
      else if (音節.includes('e')) 標調位置 = 音節.indexOf('e');
    } else {
      標調位置 = 音節.indexOf('̩');
    }
    標調位置 += 1;
    return 音節.slice(0, 標調位置) + 附標標調[聲調] + 音節.slice(標調位置);
  } 
  else {
    return 音節 + 次序標調[聲調];
  }
}

let 聲母 = 聲母規則();
let 韻母 = 韻母規則();
return 聲調規則(聲母 + 韻母);
});

_defaultDerivers.langjin = _schemata.langjin();

/**
 * 推導南京話
 *
 * https://zhuanlan.zhihu.com/p/391166351
 *
 * 中古音與南京音的對應表：https://github.com/uliloewi/lang2jin1/blob/master/Guangyun_Langjin_pulish_Alphabetic.2.0.csv
 * 南京音本是清末以前標準官話的基礎音系，和中古音有嚴格的對應關係，故有上表。本程序展示此對應關係。
 * 南京話拼音方案：https://zh.wikipedia.org/wiki/%E5%8D%97%E4%BA%AC%E8%A9%B1%E6%8B%89%E4%B8%81%E5%8C%96%E6%96%B9%E6%A1%88#%E8%BC%B8%E5%85%A5%E6%B3%95%E6%96%B9%E6%A1%88
 *
 * @author uliloewi
 * @param {Qieyun.音韻地位} 音韻地位 切韻音系音韻地位
 * @param {string=} 字頭 字頭（可選）
 * @param {undefined=} 舊選項參數 已棄用，請用 `<方案名>.方案(選項)(音韻地位, 字頭?)` 代替
 * @return {string} 音韻地位對應的推導南京話
 * @property {推導方案<string>} schema 推導方案，可用於指定推導選項、獲取方案詳細資訊等
 */
export function langjin(音韻地位, 字頭, 舊選項參數) {
  return _perform("langjin", _defaultDerivers.langjin, 音韻地位, 字頭, 舊選項參數);
}
/** @type {推導方案<string>} */
langjin.schema = _schemata.langjin

_schemata.taibu = new 推導方案(function (選項, 音韻地位, 字頭) {
const is = (x) => 音韻地位.屬於(x);

// 推導選項
if (!音韻地位) return [
  ['$legacy', true],
  ['文白讀', [3, '文上白下', '僅白讀', '僅文讀']],
  ['音標', [1, '客家語拼音方案 (臺灣客拼)', '國際音標 (IPA)']],
  ['標調方式', [1, '數字調號', '符號調號', '數字調值', '折線調值']]
];

// 一些常量
const 平舌 = ['z', 'c', 's'];
const 翹舌 = ['zh', 'ch', 'sh', 'rh'];
const 平舌或翹舌 = 平舌.concat(翹舌);
const 輕脣 = ['f', 'v'];
const 翹舌或輕脣 = 翹舌.concat(輕脣);

function 聲母規則(白讀) {
  if (is('幫母')) {
    if (is('東韻 三等 或 鍾微虞廢文元陽尤凡韻') && !白讀) return 'f';
    return 'b';
  }
  if (is('滂母')) {
    if (is('東韻 三等 或 鍾微虞廢文元陽尤凡韻') && !白讀) return 'f';
    return 'p';
  }
  if (is('並母')) {
    if (is('東韻 三等 或 鍾微虞廢文元陽尤凡韻') && !白讀) return 'f';
    return 'p';
  }
  if (is('明母')) {
    if (is('東韻 三等 或 鍾微虞廢文元陽尤凡韻') && !白讀) return 'v';
    return 'm';
  }

  if (is('端母')) return 'd';
  if (is('透母')) return 't';
  if (is('定母')) return 't';
  if (is('泥母')) return 'n';
  if (is('來母')) return 'l';

  if (is('知母')) return is('二等') ? 'z' : 'zh';
  if (is('徹母')) return is('二等') ? 'c' : 'ch';
  if (is('澄母')) return is('二等') ? 'c' : 'ch';
  if (is('孃母')) return 'n';

  if (is('精母')) return 'z';
  if (is('清母')) return 'c';
  if (is('從母')) return 'c';
  if (is('心母')) return 's';
  if (is('邪母')) {
    if (is('支脂之微韻 去聲')) return 'c';
    if (is('魚虞模東冬鍾侵韻 平聲')) return 'c';
    if (is('侯尤幽麻韻')) return 'c';
    return 's';
  }

  if (is('莊母')) return 'z';
  if (is('初母')) return 'c';
  if (is('崇母')) return 'c';
  if (is('生母')) return 's';
  if (is('俟母')) return 'c';

  if (is('章母')) return 'zh';
  if (is('昌母')) return 'ch';
  if (is('常母')) return 'sh';
  if (is('書母')) return 'sh';
  if (is('船母')) return 'sh';
  if (is('日母')) return 'ng';

  if (is('見母')) return 'g';
  if (is('溪母')) return 'k';
  if (is('羣母')) return 'k';
  if (is('疑母')) return 'ng';

  if (is('曉母')) {
    if (is('合口')) return 'f';
    if (is('三四等')) return 'x';
    return 'h';
  }
  if (is('匣母')) {
    if (is('合口 或 東模韻')) return 'f';
    if (is('開口 四等')) return 'x';
    return 'h';
  }
  if (is('影母')) {
    if (is('合口')) return 'v';
    if (is('三四等')) return 'rh';
    return '';
  }
  if (is('云母')) {
    if (is('東韻')) return 'x';
    if (is('支脂之微唐陽韻 或 魚虞模韻 平聲')) return 'v';
    return 'rh';
  }
  if (is('以母')) {
    if (is('支脂之微韻 合口')) return 'v';
    return 'rh';
  }

  throw new Error('無聲母規則');
}

function 韻母規則(白讀) {
  // 通攝
  if (is('東冬鍾韻 幫組')) return 'ung';
  if (is('東冬鍾韻 一等')) return 'ung';
  if (is('東冬鍾韻 三等')) return 'iung'; // (>ung/翹舌_)

  // 江攝
  if (is('江韻')) return 'ong'; // 例外: 窗雙 ung

  // 宕攝
  if (is('唐韻 幫組')) return 'ong';
  if (is('唐韻 開口')) return 'ong';
  if (is('唐韻 合口 見影組')) return 'uong'; // (>ong/輕脣_)
  if (is('陽韻 幫組')) return 白讀 ? 'iong' : 'ong';
  if (is('陽韻 莊組')) return 'ong';
  if (is('陽韻 開口')) return 'iong'; // (>ong/翹舌_)
  if (is('陽韻 合口 見影組')) return 'uong'; // (>ong/輕脣_)

  // 止攝
  if (is('支脂韻 幫組')) return 'i'; // 例外: 美費 ui
  if (is('微韻 幫組')) return 'ui';
  if (is('支脂之微韻 開口')) return 'i'; // (>ii/平翘舌_) 例外: 知姊死四 i
  if (is('支脂之微韻 合口')) return 'ui'; // 例外: 炊吹衰睡 oi

  // 遇攝
  if (is('虞韻 幫組')) return 'u';
  if (is('魚虞韻 莊組')) return 'ii';
  if (is('魚虞韻')) {
    if (is('知章組')) return 'u';
    return 'i'; // (>ii/翘舌_)
  }
  if (is('模韻')) return 'u'; // 例外: 蜈梧吾吳午五伍 零韻母

  // 蟹攝
  if (is('咍韻')) return 白讀 ? 'oi' : 'ai';
  if (is('泰韻 幫組')) return 'ui';
  if (is('泰韻 開口')) return 'ai'; // 例外: 蓋害 oi
  if (is('泰韻 合口')) {
    if (is('見組')) return 'uai';
    return 'ui';
  }
  if (is('灰廢韻')) return 白讀 ? 'oi' : 'ui';
  if (is('佳韻 幫組')) return 'ai';
  if (is('佳韻 開口')) {
    if (is('影組')) return 'ei';
    return 'ai';
  }
  if (is('佳韻 合口')) return 'ua'; // (>a/輕脣_)
  if (is('皆夬韻 幫組')) return 'ai';
  if (is('皆夬韻 開口')) return 'ai';
  if (is('皆夬韻 合口')) return 'uai'; // (>ai/輕脣_)
  if (is('齊祭韻 幫組')) return 'i';
  if (is('祭韻 開口')) return 'i'; // (>ii/翹舌_)
  if (is('祭韻 合口')) return 白讀 ? 'oi' : 'ui'; // 例外: 脆 cioi5 歲 sei5
  if (is('齊韻 開口')) {
    if (is('影組')) return 'i';
    return 白讀 ? 'ei' : 'i'; // 例外: 梯 toi1
  }
  if (is('齊韻 合口')) return 'ui';

  // 臻攝
  if (is('痕韻')) return 'en'; // 例外: 吞 tun1; 痕 fin2
  if (is('魂韻')) return 'un';
  if (is('眞韻 幫組')) return 'in'; // 例外: 閩敏憫 en; 密蜜 ed
  if (is('文韻 幫組')) return 'un';
  if (is('眞臻文欣韻 見組')) return 白讀 ? 'iun' : 'in';
  if (is('眞臻文欣韻 開口')) return 'in'; // 例外: 韌 ngiun5 瑟 sed7
  if (is('眞臻文欣韻 合口')) {
    if (is('精組')) return 'un';
    return 'iun'; // (>un/翹舌_)
  }

  // 山攝
  if (is('寒韻 幫組')) return 'an';
  if (is('寒韻 開口')) {
    if (is('見影組')) return 'on';
    return 'an'; // 例外: 餐 on
  } else if (is('寒韻 合口')) {
    if (is('見影組')) return 'uan'; // (>an/輕脣_)
    return 'on';
  }
  if (is('刪山韻 幫組')) return 'an'; // 例外: 慢 een; 八捌 eed
  if (is('刪山韻 開口')) {
    if (is('見影組')) return 'ien'; // 例外: 限 een; 瞎 eed
    return 'an';
  } else if (is('刪山韻 合口')) {
    if (is('見影組')) return 'uan'; // (>an/輕脣_)
    return 'on';
  }
  if (is('仙韻 幫組')) return 'ien';
  if (is('仙韻 開口')) return 'ien'; // (>een/翹舌_)
  if (is('仙韻 合口')) {
    if (is('見影組')) return 'ien'; // (>een/翹舌_) 例外: 圓 vien2
    if (is('精莊組 入聲')) return 'ied';
    return 'ion'; // (>on/翹舌_)
  }
  if (is('先韻')) {
    if (is('幫組')) return 白讀 ? 'een' : 'ien';
    if (is('端組')) return 'een';
    if (is('來母')) return 'een';
    return 'ien'; // (>een/翹舌_)
  }
  if (is('元韻')) {
    if (is('幫組')) return 'an'; // 例外: 飯 on
    if (is('見影組')) return 'ien'; // (>een/翹舌_) 例外: 冤園遠 ien
  }

  // 效攝
  if (is('豪韻')) return 'ou'; // 例外: 熬靠考袍 au
  if (is('肴韻')) return 'au'; // (>au/翹舌_)
  if (is('宵韻')) return 'iau'; // (>au/翹舌_)
  if (is('蕭韻')) {
    if (is('端組')) return 'eeu';
    if (is('來母')) return 'eeu';
    return 'iau'; // (>au/翹舌_)
  }

  // 果攝
  if (is('歌韻 一等')) return 'ou';
  if (is('歌韻 三等')) return 'iau';

  // 假攝
  if (is('麻韻 幫組')) return 'a';
  if (is('麻韻 二等 開口')) return 'a';
  if (is('麻韻 二等 合口')) return 'ua'; // (>a/輕脣_)
  if (is('麻韻 三等')) return 'ia'; // (>a/翹舌_)

  // 梗攝
  if (is('庚韻 二等 幫組')) return 白讀 ? 'ang' : 'en';
  if (is('庚韻 二等 開口')) return 白讀 ? 'ang' : 'en';
  if (is('庚韻 二等 合口')) return 白讀 ? 'uang' : 'uen'; // (>白ang|文en/輕脣_)
  if (is('庚韻 三等 幫組')) return 白讀 ? 'iang' : 'in'; // 例外: 盟 en
  if (is('庚韻 三等 開口')) return 白讀 ? 'iang' : 'in'; // (>白ang|文in/翹舌_) 例外: 省 en
  if (is('庚韻 三等 合口')) return 白讀 ? 'iung' : 'un'; // (>白ung|文un/翹舌_) 例外: 憬 in
  if (is('耕韻')) return 白讀 ? 'ang' : 'en'; // 例外: 拼莖 in
  if (is('清韻')) return 白讀 ? 'iang' : 'in'; // (>白ang|文in/翹舌_) 例外: 碧 ed
  if (is('青韻')) {
    if (is('端見組 舒聲')) return 'en';
    if (is('端見組 入聲')) return 'id';
    if (is('來母 舒聲')) return 'en';
    if (is('來母 入聲')) return 'id';
    return 白讀 ? 'iang' : 'in'; // 例外: 星 en
  }

  // 曾攝
  if (is('蒸韻')) {
    if (is('莊組')) return 'en';
    return 'in';
  }
  if (is('登韻')) return 'en'; // 例外: 國 ued

  // 流攝
  if (is('侯韻')) return 'eu';
  if (is('尤幽韻')) return 'iu';

  // 深攝
  if (is('侵韻')) {
    if (is('莊組')) return 'em';
    return 'im';
  }

  // 咸攝
  if (is('覃談咸銜凡韻')) return 'am'; // 例外: 鹹 eem; 減 iam; 夾插 iab
  if (is('鹽嚴韻')) return 'iam'; // (>am/翹舌_) 例外: 黏驗 eem; 躡 eeb
  if (is('添韻')) {
    if (is('端組')) return 'eem';
    if (is('來母')) return 'eem';
    return 'iam'; // (>am/翹舌_)
  }

  throw new Error('無韻母規則');
}

function 聲調規則(白讀) {
  if (is('平聲')) return is('全清 或 次清') ? '1' : '2';
  if (is('上聲') && 白讀) return is('次濁 或 全濁') ? '1' : '3';
  if (is('上聲') && !白讀) return is('全清 或 次清 或 次濁') ? '3' : '5';
  if (is('去聲')) return '5';
  if (is('入聲')) return is('全清 或 次清') ? '7' : '8';

  throw new Error('無聲調規則');
}

function 聲母處理(聲母, 韻母) {
  // 無聲母時自動補全 v
  if (聲母 === '' && 韻母.startsWith('u')) 聲母 = 'v';
  // 日母後接洪音爲 rh
  if (is('日母') && !韻母.startsWith('i')) 聲母 = 'rh';
  return 聲母;
}

function 韻母預先處理(韻母) {
  // 替換入聲韻尾
  if (is('入聲')) {
    if (韻母.endsWith('m')) 韻母 = 韻母.slice(0, -1) + 'b';
    else if (韻母.endsWith('n')) 韻母 = 韻母.slice(0, -1) + 'd';
    else if (韻母.endsWith('ng')) 韻母 = 韻母.slice(0, -2) + 'g';
  }
  return 韻母;
}

function 韻母善後處理(聲母, 韻母) {
  // 先處理 i 相關韻母
  if (is('支脂之微韻 開口') && 韻母 === 'i' && 平舌或翹舌.includes(聲母)) {
    韻母 = 'ii';
  }
  if (is('魚虞韻') && 韻母 === 'i' && 翹舌.includes(聲母)) 韻母 = 'ii';
  if (is('祭韻 開口') && 韻母 === 'i' && 翹舌.includes(聲母)) 韻母 = 'ii';
  // 處理一般情況
  if (韻母.startsWith('ie') && 翹舌.includes(聲母)) {
    韻母 = 'ee' + 韻母.slice(2);
  }
  if (
    (
      韻母.startsWith('ia') || 韻母.startsWith('io') || 韻母.startsWith('iu') ||
      韻母.startsWith('ua') || 韻母.startsWith('ue') || 韻母.startsWith('uo')
    ) && 翹舌或輕脣.includes(聲母)
    && 韻母 !== 'iu' // iu 韻本身除外, 如: 手 shiu3
  ) {
    韻母 = 韻母.slice(1);
  }
  return 韻母;
}

// 依選項分別推導
let 白讀音, 文讀音 = null;

function 根據規則推導讀音(白讀) {
  let 讀音 = {
    韻母: '',
    聲母: '',
    聲調: ''
  };
  const 聲母推導結果 = 聲母規則(白讀);
  const 韻母推導結果 = 韻母規則(白讀);
  讀音.韻母 = 韻母善後處理(聲母推導結果, 韻母預先處理(韻母推導結果));
  讀音.聲母 = 聲母處理(聲母推導結果, 讀音.韻母);
  讀音.聲調 = 聲調規則(白讀);
  return 讀音;
}

if (選項.文白讀 === '僅白讀') {
  白讀音 = 根據規則推導讀音(true);
} else if (選項.文白讀 === '僅文讀') {
  文讀音 = 根據規則推導讀音(false);
} else if (選項.文白讀 === '文上白下') {
  白讀音 = 根據規則推導讀音(true);
  文讀音 = 根據規則推導讀音(false);
}

// K++ 轉 IPA
function 客拼轉IPA(讀音){
  const 轉換結果 = {
    韻母: '',
    聲母: '',
    聲調: ''
  };
  const 輔音韻尾正則匹配 = `(n[ng]?|[mbdgh])$`;
  const 首元音正則匹配 = `^(ii|i|u|[aeo][reo]?)`;
  const 轉換聲母 = {
    'm': 'm', 'n': 'n', 'ng': 'ŋ',
    'b': 'p', 'd': 't', 'g': 'k',
    'p': 'pʰ', 't': 'tʰ', 'k': 'kʰ', 
    's': 's', 'sh': 'ʃ', 'x': 'ɕ',
    'z': 'ts', 'zh': 'tʃ',
    'c': 'tsʰ', 'ch': 'tʃʰ',
    'rh': 'ʒ',
    'f': 'f', 'h': 'h', 'v': 'v',
    'l': 'l', '': ''
  };
  const 轉換元音 = {
    'i': 'i', 'ii': 'ɨ', 'u': 'u',
    'oo': 'o',
    'er': 'ə',
    'e': 'ɛ', 'o': 'ɔ',
    'ee': 'æ',
    'a': 'a',
    'm': 'm̩', 'ng': 'ŋ̍'
  };
  const 轉換輔音韻尾 = {
    'm': 'm', 'n': 'n', 'ng': 'ŋ',
    'b': 'p', 'd': 't', 'g': 'k'
  };
  // 提取輔音韻尾
  let 輔音韻尾 = 讀音.韻母.match(輔音韻尾正則匹配) === null ? 
                '' :
                讀音.韻母.match(輔音韻尾正則匹配)[0];
  // 提取元音們
  let 元音們 = 輔音韻尾 !== '' ? 讀音.韻母.slice(0, -輔音韻尾.length) : 讀音.韻母;
  // 預先處理：按韻尾分別處理 u、o 元音
  if (輔音韻尾 === 'ng' || 輔音韻尾 === 'g') {
    if (元音們 === 'u') {
      元音們 = 'oo';
    } else if (元音們 === 'iu') {
      元音們 = 'ioo';
    }
  }
  if (元音們 === 'ou') {
    元音們 = 'oou';
  }
  // 將元音們逐個轉換
  let 轉換後的元音們 = [];
  while (元音們.length !== 0) {
    let 當前元音 = 元音們.match(首元音正則匹配)[0];
    轉換後的元音們.push(轉換元音[當前元音]);
    元音們 = 元音們.slice(當前元音.length);
  }
  轉換後的元音們 = 轉換後的元音們.join('');
  // 轉換韻尾
  轉換後的輔音韻尾 = 轉換輔音韻尾[輔音韻尾];
  // 善後處理：按聲母分別處理舌尖韻母
  if (轉換後的元音們 === 'ɨ') {
    if (平舌.includes(讀音.聲母)) {
      轉換後的元音們 = 'ɿ';
    } else {
      轉換後的元音們 = 'ʅ';
    }
  }
  // 將結果拼合成韻母
  轉換結果.韻母 = 轉換後的元音們 + (輔音韻尾 !== '' ? 轉換後的輔音韻尾 : '');
  // 轉換聲母
  轉換結果.聲母 = 轉換聲母[讀音.聲母];
  // 轉換聲調
  轉換結果.聲調 = 讀音.聲調;

  return 轉換結果;
}

if (選項.音標 === '國際音標 (IPA)') {
  白讀音 = 白讀音 ? 客拼轉IPA(白讀音) : null;
  文讀音 = 文讀音 ? 客拼轉IPA(文讀音) : null;
}

// 轉換標調方式
function 轉換標調方式(讀音, 標調方式){
  const 轉換結果 = 讀音;
  const 轉換爲符號調號 = {
    '1': 'ˊ',
    '2': 'ˇ',
    '3': '^',
    '5': 'ˋ',
    '7': '^',
    '8': 'ˋ'
  };
  const 轉換爲數字調值 = {
    '1': '⁴⁵',
    '2': '²²⁴',
    '3': '³¹',
    '5': '⁵¹',
    '7': '³²',
    '8': '⁵'
  };
  const 轉換爲折線調值 = {
    '1': '˦˥',
    '2': '˨˨˦',
    '3': '˧˩',
    '5': '˥˩',
    '7': '˧˨',
    '8': '˥'
  };
  if (標調方式 === '符號調號') {
    轉換結果.聲調 = 轉換爲符號調號[讀音.聲調];
  } else if (標調方式 === '數字調值') {
    轉換結果.聲調 = 轉換爲數字調值[讀音.聲調];
  } else if (標調方式 === '折線調值') {
    轉換結果.聲調 = 轉換爲折線調值[讀音.聲調];
  }

  return 轉換結果;
}

if (
    選項.標調方式 === '符號調號' ||
    選項.標調方式 === '數字調值' ||
    選項.標調方式 === '折線調值'
  ) {
  白讀音 = 白讀音 ? 轉換標調方式(白讀音, 選項.標調方式) : null;
  文讀音 = 文讀音 ? 轉換標調方式(文讀音, 選項.標調方式) : null;
}

// 返回結果
const 白讀音結果 = 白讀音 ? 白讀音.聲母 + 白讀音.韻母 + 白讀音.聲調 : null;
const 文讀音結果 = 文讀音 ? 文讀音.聲母 + 文讀音.韻母 + 文讀音.聲調 : null;
let 結果 = null;

if (選項.文白讀 === '僅白讀') {
  結果 = 白讀音結果;
} else if (選項.文白讀 === '僅文讀') {
  結果 = 文讀音結果;
} else if (選項.文白讀 === '文上白下') {
  if (文讀音結果 === 白讀音結果) {
    結果 = 文讀音結果;
  } else {
    結果 = 文讀音結果 + '\n' + 白讀音結果;
  }
}

return 結果;
});

_defaultDerivers.taibu = _schemata.taibu();

/**
 * 推導大埔話（百侯聲）
 *
 * 音系及IPA標法見：https://zhuanlan.zhihu.com/p/349914674
 * 拼音方案見：https://zhuanlan.zhihu.com/p/350459791
 * 推導規則描述：https://zhuanlan.zhihu.com/p/392372782
 * 
 * @author 以成
 * @param {Qieyun.音韻地位} 音韻地位 切韻音系音韻地位
 * @param {string=} 字頭 字頭（可選）
 * @param {undefined=} 舊選項參數 已棄用，請用 `<方案名>.方案(選項)(音韻地位, 字頭?)` 代替
 * @return {string} 音韻地位對應的推導大埔話（百侯聲）
 * @property {推導方案<string>} schema 推導方案，可用於指定推導選項、獲取方案詳細資訊等
 */
export function taibu(音韻地位, 字頭, 舊選項參數) {
  return _perform("taibu", _defaultDerivers.taibu, 音韻地位, 字頭, 舊選項參數);
}
/** @type {推導方案<string>} */
taibu.schema = _schemata.taibu

_schemata.ayaka_v8 = new 推導方案(function (選項, 音韻地位, 字頭) {
// 1. 選項

if (!音韻地位) return [
  ['$legacy', true],
  // '平假名': 地 ち
  // '片假名': 地 チ
  // '日本式羅馬字': 地 ti
  // '平文式羅馬字': 地 chi
  ['書寫系統', [3, '平假名', '片假名', '日本式羅馬字', '平文式羅馬字']],

  // true: 迥 ク𛅥ィ
  // false: 迥 クヱィ
  // 僅當開啓假名時生效
  ['ヰヱヲ小假名', true],

  // null: 宙 チウ tiu；南 ダム dam；愁 スウ suu
  // '現代日語': 宙 チュウ tyuu；南 ダン dan；愁 スウ suu
  ['音變', [1, null, '現代日語']],

  // null: 無聲調
  // '四聲': 四聲
  // '四聲（數字）': 四聲，以數字方式展示
  // '四聲（調值）': 四聲，以調值方式展示
  // '六聲（調值）': 六聲，以調值方式展示
  // '六聲（符號）': 六聲，以符號方式展示
  // '八聲': 八聲
  // '八聲（數字）': 八聲，以數字方式展示
  // '八聲（調值）': 八聲，以調值方式展示
  // 參考：尉遲治平. 日本悉曇家所傳古漢語調值.
  ['聲調', [1, null, '四聲', '四聲（數字）', '四聲（調值）', '六聲（調值）', '六聲（符號）', '八聲', '八聲（數字）', '八聲（調值）']],
];

// 2. 輔助函數

const 假名表 = {
  a:  'ア', i:  'イ', u:  'ウ', e:  'エ', o:  'オ',
  ka: 'カ', ki: 'キ', ku: 'ク', ke: 'ケ', ko: 'コ',
  ga: 'ガ', gi: 'ギ', gu: 'グ', ge: 'ゲ', go: 'ゴ',
  sa: 'サ', si: 'シ', su: 'ス', se: 'セ', so: 'ソ',
  za: 'ザ', zi: 'ジ', zu: 'ズ', ze: 'ゼ', zo: 'ゾ',
  ta: 'タ', ti: 'チ', tu: 'ツ', te: 'テ', to: 'ト',
  da: 'ダ', di: 'ヂ', du: 'ヅ', de: 'デ', do: 'ド',
  na: 'ナ', ni: 'ニ', nu: 'ヌ', ne: 'ネ', no: 'ノ',
  pa: 'ハ', pi: 'ヒ', pu: 'フ', pe: 'ヘ', po: 'ホ',
  ba: 'バ', bi: 'ビ', bu: 'ブ', be: 'ベ', bo: 'ボ',
  ma: 'マ', mi: 'ミ', mu: 'ム', me: 'メ', mo: 'モ',
  ya: 'ヤ',           yu: 'ユ',           yo: 'ヨ',
  ra: 'ラ', ri: 'リ', ru: 'ル', re: 'レ', ro: 'ロ',
  wa: 'ワ', wi: 'ヰ',           we: 'ヱ', wo: 'ヲ',
};

const 拗音表 = {
  wya: 'ヰャ', wyo: 'ヰョ',
  ya: 'ャ', yu: 'ュ', yo: 'ョ',
  wa: 'ヮ', wi: '𛅤', we: '𛅥', wo: '𛅦',
};

const 韻尾表 = {
  '': '', i: 'イ', u: 'ウ',
  m: 'ム', n: 'ン', ng: 'ゥ', // ng: 'ィ',
  p: 'フ', t: 'ツ', k: 'ク', // k: 'キ',
};

function roma2kata(s) {
  const r = /^([kgsztdnpbmyrw]?w??[yw]?)([aiueo])([ptkmngiu]*)$/g; // 將音節分為韻頭、主要元音及韻尾
  const match = r.exec(s);
  if (match == null) {
    throw new Error('無法轉換為假名：' + s);
  }
  const { 1: 韻頭, 2: 主要元音, 3: 韻尾 } = match;
  let 假名韻尾 = 韻尾表[韻尾];
  if (主要元音 === 'e') {
    if (韻尾 === 'k') 假名韻尾 = 'キ';
    if (韻尾 === 'ng') 假名韻尾 = 'ィ';
  }
  if (韻頭.length <= 1) {
    return 假名表[韻頭 + 主要元音] + 假名韻尾;
  }
  const 填充元音 = 韻頭[1] === 'w' ? 'u' : 'i'; // 韻頭[1] 只能為 w 或 y
  return 假名表[韻頭[0] + 填充元音] + 拗音表[韻頭.slice(1) + 主要元音] + 假名韻尾;
}

function kata2hira(s) {
  const diff = 'ぁ'.charCodeAt(0) - 'ァ'.charCodeAt(0);
  return [...s].map((c) => {
    switch (c) {
      case '𛅤': return '𛅐';
      case '𛅥': return '𛅑';
      case '𛅦': return '𛅒';
      default: return String.fromCharCode(c.charCodeAt(0) + diff);
    }
  }).join('');
}

function small2large(s) {
  return [...s].map((c) => {
    switch (c) {
      case '𛅤': return 'ヰ';
      case '𛅥': return 'ヱ';
      case '𛅦': return 'ヲ';
      default: return c;
    }
  }).join('');
}

// 3. 推導規則

const is = (x) => 音韻地位.屬於(x);

function 聲母規則() {
  // 幫組
  if (is('幫滂並母')) return 'p';
  if (is('明母')) return is('梗攝') && !is('庚耕青韻 入聲') ? 'm' : 'b';

  // 端組、來母
  if (is('端透定母')) return 't';
  if (is('泥母')) return is('梗攝') ? 'n' : 'd';
  if (is('來母')) return 'r';

  // 知組
  if (is('知徹澄母')) return 't';
  if (is('孃母')) return is('梗攝') ? 'n' : 'd';

  // 精組
  if (is('精清從心邪母')) return 's';

  // 莊組
  if (is('莊初崇生俟母')) return 's';

  // 章組、日母
  if (is('章昌船書常母')) return 's';
  if (is('日母')) return 'z';

  // 見組
  if (is('見溪羣母')) return 'k';
  if (is('疑母')) return 'g';

  // 影組、以母
  if (is('影母')) return '';
  if (is('曉匣母')) return 'k';
  if (is('云以母')) return '';

  throw new Error('無聲母規則');
}

function 韻母規則() {
  // 通攝
  if (is('東韻 一等')) return 'ong';
  if (is('東韻 三等 幫滂並母')) return is('舒聲') ? 'ong' : 'uk';
  if (is('東韻 三等 明母')) return 'ong';
  if (is('東韻 三等 精莊章組')) return 'yung';
  if (is('東韻 三等 舌齒音')) return is('舒聲') ? 'yung' : 'ik';
  if (is('東韻 三等 影母')) return is('舒聲') ? 'yung' : 'wik';
  if (is('東韻 三等 牙喉音')) return is('舒聲') ? 'yung' : 'ik';
  if (is('冬韻')) return 'ong';
  if (is('鍾韻 脣音')) return 'ong';
  if (is('鍾韻')) return 'yong';

  // 江攝
  if (is('江韻')) return 'ang';

  // 止攝
  if (is('支脂之微韻 脣音')) return 'i';
  if (is('支脂之微韻 開口')) return 'i';
  if (is('支脂之微韻 合口 舌齒音')) return 'ui';
  if (is('支脂之微韻 合口 牙喉音')) return 'wi';

  // 遇攝
  if (is('魚韻 莊組')) return 'o';
  if (is('魚韻 舌齒音')) return 'yo';
  if (is('魚韻 牙喉音')) return 'yo';
  if (is('虞韻 脣音')) return 'u';
  if (is('虞韻 來母')) return 'u';
  if (is('虞韻 知組')) return 'yuu';
  if (is('虞韻 莊組')) return 'u';
  if (is('虞韻 舌齒音')) return 'yu';
  if (is('虞韻 以母')) return 'yu';
  if (is('虞韻 牙喉音')) return 'u';
  if (is('模韻 幫組')) return 'o';
  if (is('模韻 舌齒音')) return 'o';
  if (is('模韻 影母')) return 'wo';
  if (is('模韻 牙喉音')) return 'o';

  // 蟹攝
  if (is('祭齊韻 脣音')) return 'ei';
  if (is('祭齊韻 開口')) return 'ei';
  if (is('祭齊韻 合口')) return 'wei';
  if (is('泰佳皆夬韻 脣音')) return 'ai';
  if (is('泰佳皆夬韻 開口')) return 'ai';
  if (is('泰佳皆夬韻 合口')) return 'wai';
  if (is('灰韻')) return 'wai';
  if (is('咍韻')) return 'ai';
  if (is('廢韻 脣音')) return 'ei';
  if (is('廢韻 開口')) return 'ei';
  if (is('廢韻 合口')) return 'wai';

  // 臻攝
  if (is('眞韻 脣音')) return 'in';
  if (is('眞韻 開口')) return 'in';
  if (is('眞韻 合口 來母')) return 'in';
  if (is('眞韻 合口 莊組')) return is('舒聲') ? 'on' : 'it';
  if (is('眞韻 合口 舌齒音')) return is('舒聲') ? 'yun' : 'ot';
  if (is('眞韻 合口 云母')) return 'win';
  if (is('眞韻 合口 牙喉音')) return 'in';
  if (is('臻韻')) return 'in';
  if (is('文韻')) return 'un';
  if (is('欣韻')) return 'in';
  if (is('元韻 脣音')) return 'an';
  if (is('元韻 開口')) return 'en';
  if (is('元韻 合口')) return 'wen';
  if (is('魂痕韻')) return 'on';

  // 山攝
  if (is('寒刪山韻 脣音')) return 'an';
  if (is('寒刪山韻 開口')) return 'an';
  if (is('寒刪山韻 合口')) return 'wan';
  if (is('仙先韻 脣音')) return 'en';
  if (is('仙先韻 開口')) return 'en';
  if (is('仙先韻 合口')) return 'wen';

  // 效攝
  if (is('蕭宵韻')) return 'eu';
  if (is('肴韻')) return 'au';
  if (is('豪韻 脣音')) return 'ou';
  if (is('豪韻 舌齒音')) return 'au';
  if (is('豪韻 牙喉音')) return 'au';

  // 果攝
  if (is('歌韻 一等 脣音')) return 'a';
  if (is('歌韻 一等 開口')) return 'a';
  if (is('歌韻 一等 合口')) return 'wa';
  if (is('歌韻 三等 開口')) return 'ya';
  if (is('歌韻 三等 合口')) return 'wa';

  // 假攝
  if (is('麻韻 二等 脣音')) return 'a';
  if (is('麻韻 二等 開口')) return 'a';
  if (is('麻韻 二等 合口')) return 'wa';
  if (is('麻韻 三等 脣音')) return 'ya';
  if (is('麻韻 三等 開口')) return 'ya';

  // 宕攝
  if (is('陽韻 脣音')) return 'ang';
  if (is('陽韻 開口 莊組')) return 'ang';
  if (is('陽韻 開口 舌齒音')) return 'yang';
  if (is('陽韻 開口 牙喉音')) return 'yang';
  if (is('陽韻 合口 舌齒音')) return 'ang';
  if (is('陽韻 合口 影云母')) return 'wang';
  if (is('陽韻 合口 牙喉音')) return 'wyang';
  if (is('唐韻 脣音')) return 'ang';
  if (is('唐韻 開口')) return 'ang';
  if (is('唐韻 合口')) return 'wang';

  // 梗攝
  if (is('庚韻 二等 脣音')) return 'ang';
  if (is('庚韻 二等 開口')) return 'ang';
  if (is('庚韻 二等 合口')) return 'wang';
  if (is('庚韻 三等 脣音')) return 'eng';
  if (is('庚韻 三等 開口')) return 'eng';
  if (is('庚韻 三等 合口')) return 'weng';
  if (is('耕韻')) return 'wang';
  if (is('清青韻 脣音')) return 'eng';
  if (is('清青韻 開口')) return 'eng';
  if (is('清青韻 合口')) return 'weng';

  // 曾攝
  if (is('蒸韻 脣音')) return 'yong';
  if (is('蒸韻 開口 莊組')) return 'ong';
  if (is('蒸韻 開口 舌齒音')) return 'yong';
  if (is('蒸韻 開口 牙喉音')) return 'yong';
  if (is('蒸韻 合口 影云母')) return 'yong';
  if (is('蒸韻 合口 牙喉音')) return 'wyong';
  if (is('登韻')) return 'ong';

  // 流攝
  if (is('尤韻 幫滂並母')) return 'uu';
  if (is('尤韻 明母')) return 'ou';
  if (is('尤韻 莊組')) return 'uu';
  if (is('尤韻 舌齒音')) return 'iu';
  if (is('尤韻 牙喉音')) return 'iu';
  if (is('侯韻')) return 'ou';
  if (is('幽韻')) return 'iu';

  // 深攝
  if (is('侵韻')) return 'im';

  // 咸攝
  if (is('覃談韻')) return 'am';
  if (is('鹽添嚴韻')) return 'em';
  if (is('咸銜凡韻')) return 'am';

  throw new Error('無韻母規則');
}

let 聲母 = 聲母規則();
let 韻母 = 韻母規則();

if (is('入聲')) {
  if (韻母.endsWith('m')) 韻母 = 韻母.slice(0, -1) + 'p';
  else if (韻母.endsWith('n')) 韻母 = 韻母.slice(0, -1) + 't';
  else if (韻母.endsWith('ng')) 韻母 = 韻母.slice(0, -2) + 'k';
}

function 聲調規則() {
  if (['四聲', '四聲（數字）', '四聲（調值）'].includes(選項.聲調)) {
    const idx = ['四聲', '四聲（數字）', '四聲（調值）'].indexOf(選項.聲調);
    if (is('平聲')) return ['꜀', '1', '11'][idx];
    if (is('上聲')) return ['꜂', '2', '55'][idx];
    if (is('去聲')) return ['꜄', '3', '15'][idx];
    if (is('入聲')) return ['꜆', '4', '1'][idx];
    throw new Error('無聲調規則');
  }

  if (選項.聲調 === '六聲（調值）') {
    if (is('平聲 全濁')) return '11';
    if (is('平聲')) return '51';
    if (is('上去聲 全濁')) return '15';
    if (is('上去聲')) return '55';
    if (is('入聲 全濁')) return '1';
    if (is('入聲')) return '5';
    throw new Error('無聲調規則');
  }

  if (選項.聲調 === '六聲（符號）') {
    if (is('平聲 全濁')) return 'z';
    if (is('平聲')) return '';
    if (is('上去聲 全濁')) return 'h';
    if (is('上去聲')) return 'x';
    if (is('入聲 全濁')) {
      if (韻母.endsWith('p')) 韻母 = 韻母.slice(0, -1) + 'b';
      else if (韻母.endsWith('t')) 韻母 = 韻母.slice(0, -1) + 'd';
      else if (韻母.endsWith('k')) 韻母 = 韻母.slice(0, -1) + 'g';

      return '';
    }
    if (is('入聲')) return '';
    throw new Error('無聲調規則');
  }

  if (['八聲', '八聲（數字）', '八聲（調值）'].includes(選項.聲調)) {
    const idx = ['八聲', '八聲（數字）', '八聲（調值）'].indexOf(選項.聲調);
    if (!is('全濁')) {
      if (is('平聲')) return ['꜀', '1', '51'][idx];
      if (is('上聲')) return ['꜂', '2', '55'][idx];
      if (is('去聲')) return ['꜄', '3', '535'][idx];
      if (is('入聲')) return ['꜆', '4', '5'][idx];
    } else {
      if (is('平聲')) return ['꜁', '5', '11'][idx];
      if (is('上聲')) return ['꜃', '6', '15'][idx];
      if (is('去聲')) return ['꜅', '7', '315'][idx];
      if (is('入聲')) return ['꜇', '8', '1'][idx];
    }
    throw new Error('無聲調規則');
  }

  return '';
}

let 聲調 = 聲調規則();

if (韻母.startsWith('w') && (!is('牙喉音') || is('重紐A類 或 以母'))) 韻母 = 韻母.slice(1);

// 4. 音變規則

if (選項.音變 === '現代日語') {
  if (韻母.startsWith('w')) 韻母 = 韻母.slice(1); // 園 wen -> en

  if (韻母.endsWith('p')) 韻母 = 韻母.slice(0, -1) + 'u'; // 鄴 gep -> geu
  else if (韻母.endsWith('m')) 韻母 = 韻母.slice(0, -1) + 'n'; // 南 dam -> dan
  else if (韻母.endsWith('eng')) 韻母 = 韻母.slice(0, -2) + 'i'; // 生 seng -> sei
  else if (韻母.endsWith('ng')) 韻母 = 韻母.slice(0, -2) + 'u'; // 相 syang -> syau

  if (韻母.endsWith('au')) 韻母 = 韻母.slice(0, -2) + 'ou'; // 高 kau -> kou
  else if (韻母.endsWith('iu')) 韻母 = 韻母.slice(0, -2) + 'yuu'; // 宙 tiu -> tyuu
  else if (韻母.endsWith('eu')) 韻母 = 韻母.slice(0, -2) + 'you'; // 遙 eu -> you
  else if (韻母.endsWith('ang')) 韻母 = 韻母.slice(0, -3) + 'ong'; // 相 syang -> syong
}

let 聲韻;

if (['平假名', '片假名'].includes(選項.書寫系統)) {
  聲韻 = roma2kata(聲母 + 韻母);
  if (!選項.ヰヱヲ小假名) 聲韻 = small2large(聲韻);
  if (選項.書寫系統 === '平假名') 聲韻 = kata2hira(聲韻);
} else {
  if (選項.音變 === '現代日語') {
    if (聲母 === 'p') 聲母 = 'h'; // 甫 pu -> hu

    if (韻母.endsWith('t')) 韻母 = 韻母 + 'u'; // 遏 at -> atu
    else if (韻母.endsWith('ek')) 韻母 = 韻母 + 'i'; // 席 sek -> seki
    else if (韻母.endsWith('k')) 韻母 = 韻母 + 'u'; // 澤 tak -> taku
  }

  if (選項.書寫系統 === '平文式羅馬字') {
    if (選項.音變 === '現代日語') {
      if (聲母 === 's' && 韻母.startsWith('i')) 聲母 = 'sh'; // 四 si -> shi
      else if (聲母 === 'z' && 韻母.startsWith('i')) 聲母 = 'j'; // 人 zin -> jin
      else if (聲母 === 't' && 韻母.startsWith('i')) 聲母 = 'ch'; // 地 ti -> chi
      else if (聲母 === 't' && 韻母.startsWith('u')) 聲母 = 'ts'; // 追 tui -> tsui
      else if (聲母 === 'h' && 韻母.startsWith('u')) 聲母 = 'f'; // 甫 hu -> fu
      else if (聲母 === 's' && 韻母.startsWith('y')) { 聲母 = 'sh'; 韻母 = 韻母.slice(1); } // 小 syou -> shou
      else if (聲母 === 'z' && 韻母.startsWith('y')) { 聲母 = 'j'; 韻母 = 韻母.slice(1); } // 繞 zyou -> jou
      else if (聲母 === 't' && 韻母.startsWith('y')) { 聲母 = 'ch'; 韻母 = 韻母.slice(1); } // 兆 tyou -> chou

      if (聲母 === 'd' && 韻母.startsWith('i')) 聲母 = 'j'; // 膩 di -> ji
      else if (聲母 === 'd' && 韻母.startsWith('y')) { 聲母 = 'j'; 韻母 = 韻母.slice(1); } // 紐 dyuu -> juu

      if (韻母.endsWith('tu')) 韻母 = 韻母.slice(0, -1) + 'su'; // 遏 atu -> atsu
    }
  }

  聲韻 = 聲母 + 韻母;
}

return [...'꜀꜁꜂꜃'].includes(聲調) ? 聲調 + 聲韻 : 聲韻 + 聲調;
});

_defaultDerivers.ayaka_v8 = _schemata.ayaka_v8();

/**
 * 綾香思考音系
 *
 * https://ayaka.shn.hk/v8/
 *
 * @author Ayaka
 * @param {Qieyun.音韻地位} 音韻地位 切韻音系音韻地位
 * @param {string=} 字頭 字頭（可選）
 * @param {undefined=} 舊選項參數 已棄用，請用 `<方案名>.方案(選項)(音韻地位, 字頭?)` 代替
 * @return {string} 音韻地位對應的綾香思考音系
 * @property {推導方案<string>} schema 推導方案，可用於指定推導選項、獲取方案詳細資訊等
 */
export function ayaka_v8(音韻地位, 字頭, 舊選項參數) {
  return _perform("ayaka_v8", _defaultDerivers.ayaka_v8, 音韻地位, 字頭, 舊選項參數);
}
/** @type {推導方案<string>} */
ayaka_v8.schema = _schemata.ayaka_v8
